@@ -6,7 +6,6 @@
  */
 
 #include <quic/api/QuicBatchWriterFactory.h>
-#include <quic/common/events/QuicEventBase.h>
 
 namespace quic {
 
@@ -29,6 +28,13 @@ BatchWriterPtr makeSendmmsgGsoBatchWriter(uint32_t batchSize) {
   return BatchWriterPtr(new SendmmsgGSOPacketBatchWriter(batchSize));
 }
 
+BatchWriterPtr makeSendmmsgInplaceGsoInplaceBatchWriter(
+    uint32_t batchSize,
+    QuicConnectionStateBase& conn) {
+  return BatchWriterPtr(
+      new SendmmsgGSOInplacePacketBatchWriter(conn, batchSize));
+}
+
 BatchWriterPtr BatchWriterFactory::makeBatchWriter(
     const quic::QuicBatchingMode& batchingMode,
     uint32_t batchSize,
