@@ -53,15 +53,15 @@ class MiddleStartingIterationWrapper {
       wrappedAround_ = false;
     }
 
-    FOLLY_NODISCARD const MapType::value_type& dereference() const {
+    [[nodiscard]] const MapType::value_type& dereference() const {
       return *itr_;
     }
 
-    FOLLY_NODISCARD MapType::const_iterator rawIterator() const {
+    [[nodiscard]] MapType::const_iterator rawIterator() const {
       return itr_;
     }
 
-    FOLLY_NODISCARD bool equal(const MiddleStartingIterator& other) const {
+    [[nodiscard]] bool equal(const MiddleStartingIterator& other) const {
       return wrappedAround_ == other.wrappedAround_ && itr_ == other.itr_;
     }
 
@@ -94,11 +94,11 @@ class MiddleStartingIterationWrapper {
       const MapType::const_iterator& start)
       : streams_(streams), start_(&streams_, start) {}
 
-  FOLLY_NODISCARD MiddleStartingIterator cbegin() const {
+  [[nodiscard]] MiddleStartingIterator cbegin() const {
     return start_;
   }
 
-  FOLLY_NODISCARD MiddleStartingIterator cend() const {
+  [[nodiscard]] MiddleStartingIterator cend() const {
     MiddleStartingIterator itr(start_);
     itr.wrappedAround_ = true;
     return itr;
@@ -113,25 +113,6 @@ class MiddleStartingIterationWrapper {
 
 namespace quic {
 
-bool hasAcksToSchedule(const AckState& ackState) {
-  Optional<PacketNum> largestAckSend = largestAckToSend(ackState);
-  if (!largestAckSend) {
-    return false;
-  }
-  if (!ackState.largestAckScheduled) {
-    // Never scheduled an ack, we need to send
-    return true;
-  }
-  return *largestAckSend > *(ackState.largestAckScheduled);
-}
-
-Optional<PacketNum> largestAckToSend(const AckState& ackState) {
-  if (ackState.acks.empty()) {
-    return none;
-  }
-  return ackState.acks.back().end;
-}
-
 // Schedulers
 
 FrameScheduler::Builder::Builder(
@@ -237,11 +218,30 @@ FrameScheduler::FrameScheduler(
     QuicConnectionStateBase& conn)
     : name_(name), conn_(conn) {}
 
-SchedulingResult FrameScheduler::scheduleFramesForPacket(
+folly::Expected<SchedulingResult, QuicError>
+FrameScheduler::scheduleFramesForPacket(
     PacketBuilderInterface&& builder,
     uint32_t writableBytes) {
   size_t shortHeaderPadding = 0;
-  builder.encodePacketHeader();
+  const ShortHeader* shortHeader = builder.getPacketHeader().asShort();
+  const LongHeader* longHeader = builder.getPacketHeader().asLong();
+  bool initialPacket =
+      longHeader && longHeader->getHeaderType() == LongHeader::Types::Initial;
+  auto encodeRes = builder.encodePacketHeader();
+  if (encodeRes.hasError()) {
+    return folly::makeUnexpected(encodeRes.error());
+  }
+  // Add fixed padding at start of short header packets if configured
+  if (shortHeader && conn_.transportSettings.fixedShortHeaderPadding > 0) {
+    for (size_t i = 0; i < conn_.transportSettings.fixedShortHeaderPadding;
+         i++) {
+      auto writeRes = writeFrame(PaddingFrame(), builder);
+      if (writeRes.hasError()) {
+        return folly::makeUnexpected(writeRes.error());
+      }
+    }
+    shortHeaderPadding = conn_.transportSettings.fixedShortHeaderPadding;
+  }
   // We need to keep track of writable bytes after writing header.
   writableBytes = writableBytes > builder.getHeaderBytes()
       ? writableBytes - builder.getHeaderBytes()
@@ -252,24 +252,38 @@ SchedulingResult FrameScheduler::scheduleFramesForPacket(
   bool cryptoDataWritten = false;
   bool rstWritten = false;
   if (cryptoStreamScheduler_ && cryptoStreamScheduler_->hasData()) {
-    cryptoDataWritten = cryptoStreamScheduler_->writeCryptoData(wrapper);
+    auto cryptoDataRes = cryptoStreamScheduler_->writeCryptoData(wrapper);
+    if (cryptoDataRes.hasError()) {
+      return folly::makeUnexpected(cryptoDataRes.error());
+    }
+    cryptoDataWritten = cryptoDataRes.value();
   }
   if (rstScheduler_ && rstScheduler_->hasPendingRsts()) {
-    rstWritten = rstScheduler_->writeRsts(wrapper);
+    auto rstWrittenRes = rstScheduler_->writeRsts(wrapper);
+    if (rstWrittenRes.hasError()) {
+      return folly::makeUnexpected(rstWrittenRes.error());
+    }
+    rstWritten = rstWrittenRes.value();
   }
   // Long time ago we decided RST has higher priority than Acks.
   if (hasPendingAcks()) {
     if (cryptoDataWritten || rstWritten) {
       // If packet has non ack data, it is subject to congestion control. We
       // need to use the wrapper/
-      ackScheduler_->writeNextAcks(wrapper);
+      auto writeAcksRes = ackScheduler_->writeNextAcks(wrapper);
+      if (writeAcksRes.hasError()) {
+        return folly::makeUnexpected(writeAcksRes.error());
+      }
     } else {
       // If we start with writing acks, we will let the ack scheduler write
       // up to the full packet space. If the ack bytes exceeds the writable
       // bytes, this will be a pure ack packet and it will skip congestion
       // controller. Otherwise, we will give other schedulers an opportunity to
       // write up to writable bytes.
-      ackScheduler_->writeNextAcks(builder);
+      auto writeAcksRes = ackScheduler_->writeNextAcks(builder);
+      if (writeAcksRes.hasError()) {
+        return folly::makeUnexpected(writeAcksRes.error());
+      }
     }
   }
   // Immediate ACK frames are subject to congestion control but should be sent
@@ -281,10 +295,16 @@ SchedulingResult FrameScheduler::scheduleFramesForPacket(
   }
   if (windowUpdateScheduler_ &&
       windowUpdateScheduler_->hasPendingWindowUpdates()) {
-    windowUpdateScheduler_->writeWindowUpdates(wrapper);
+    auto result = windowUpdateScheduler_->writeWindowUpdates(wrapper);
+    if (result.hasError()) {
+      return folly::makeUnexpected(result.error());
+    }
   }
   if (blockedScheduler_ && blockedScheduler_->hasPendingBlockedFrames()) {
-    blockedScheduler_->writeBlockedFrames(wrapper);
+    auto result = blockedScheduler_->writeBlockedFrames(wrapper);
+    if (result.hasError()) {
+      return folly::makeUnexpected(result.error());
+    }
   }
   // Simple frames should be scheduled before stream frames and retx frames
   // because those frames might fill up all available bytes for writing.
@@ -303,28 +323,33 @@ SchedulingResult FrameScheduler::scheduleFramesForPacket(
   }
   if (datagramFrameScheduler_ &&
       datagramFrameScheduler_->hasPendingDatagramFrames()) {
-    datagramFrameScheduler_->writeDatagramFrames(wrapper);
+    auto datagramRes = datagramFrameScheduler_->writeDatagramFrames(wrapper);
+    if (datagramRes.hasError()) {
+      return folly::makeUnexpected(datagramRes.error());
+    }
   }
 
   if (builder.hasFramesPending()) {
-    const LongHeader* longHeader = builder.getPacketHeader().asLong();
-    bool initialPacket =
-        longHeader && longHeader->getHeaderType() == LongHeader::Types::Initial;
     if (initialPacket) {
       // This is the initial packet, we need to fill er up.
       while (builder.remainingSpaceInPkt() > 0) {
-        writeFrame(PaddingFrame(), builder);
+        auto writeRes = writeFrame(PaddingFrame(), builder);
+        if (writeRes.hasError()) {
+          return folly::makeUnexpected(writeRes.error());
+        }
       }
     }
-    const ShortHeader* shortHeader = builder.getPacketHeader().asShort();
     if (shortHeader) {
       size_t paddingModulo = conn_.transportSettings.paddingModulo;
       if (paddingModulo > 0) {
         size_t paddingIncrement = wrapper.remainingSpaceInPkt() % paddingModulo;
         for (size_t i = 0; i < paddingIncrement; i++) {
-          writeFrame(PaddingFrame(), builder);
+          auto writeRes = writeFrame(PaddingFrame(), builder);
+          if (writeRes.hasError()) {
+            return folly::makeUnexpected(writeRes.error());
+          }
         }
-        shortHeaderPadding = paddingIncrement;
+        shortHeaderPadding += paddingIncrement;
       }
     }
   }
@@ -369,7 +394,7 @@ bool StreamFrameScheduler::writeStreamLossBuffers(
        buffer != stream.lossBuffer.cend();
        ++buffer) {
     auto bufferLen = buffer->data.chainLength();
-    auto dataLen = writeStreamFrameHeader(
+    auto res = writeStreamFrameHeader(
         builder,
         stream.id,
         buffer->offset,
@@ -378,6 +403,11 @@ bool StreamFrameScheduler::writeStreamLossBuffers(
         buffer->eof,
         none /* skipLenHint */,
         stream.groupId);
+    if (res.hasError()) {
+      throw QuicInternalException(
+          res.error().message, *res.error().code.asLocalErrorCode());
+    }
+    auto dataLen = *res;
     if (dataLen) {
       wroteStreamFrame = true;
       writeStreamFrameData(builder, buffer->data, *dataLen);
@@ -441,14 +471,14 @@ StreamId StreamFrameScheduler::writeStreamsHelper(
 
 void StreamFrameScheduler::writeStreamsHelper(
     PacketBuilderInterface& builder,
-    PriorityQueue& writableStreams,
+    deprecated::PriorityQueue& writableStreams,
     uint64_t& connWritableBytes,
     bool streamPerPacket) {
   // Fill a packet with non-control stream data, in priority order
   for (size_t index = 0; index < writableStreams.levels.size() &&
        builder.remainingSpaceInPkt() > 0;
        index++) {
-    PriorityQueue::Level& level = writableStreams.levels[index];
+    deprecated::PriorityQueue::Level& level = writableStreams.levels[index];
     if (level.empty()) {
       // No data here, keep going
       continue;
@@ -539,7 +569,7 @@ bool StreamFrameScheduler::writeStreamFrame(
   bool canWriteFin = stream.finalWriteOffset.has_value() &&
       bufferLen <= flowControlLen && stream.writeBufMeta.offset == 0;
   auto writeOffset = stream.currentWriteOffset;
-  auto dataLen = writeStreamFrameHeader(
+  auto res = writeStreamFrameHeader(
       builder,
       stream.id,
       writeOffset,
@@ -548,6 +578,11 @@ bool StreamFrameScheduler::writeStreamFrame(
       canWriteFin,
       none /* skipLenHint */,
       stream.groupId);
+  if (res.hasError()) {
+    throw QuicInternalException(
+        res.error().message, *res.error().code.asLocalErrorCode());
+  }
+  auto dataLen = *res;
   if (!dataLen) {
     return false;
   }
@@ -561,81 +596,6 @@ bool StreamFrameScheduler::writeStreamFrame(
   return true;
 }
 
-AckScheduler::AckScheduler(
-    const QuicConnectionStateBase& conn,
-    const AckState& ackState)
-    : conn_(conn), ackState_(ackState) {}
-
-Optional<PacketNum> AckScheduler::writeNextAcks(
-    PacketBuilderInterface& builder) {
-  // Use default ack delay for long headers. Usually long headers are sent
-  // before crypto negotiation, so the peer might not know about the ack delay
-  // exponent yet, so we use the default.
-  uint8_t ackDelayExponentToUse =
-      builder.getPacketHeader().getHeaderForm() == HeaderForm::Long
-      ? kDefaultAckDelayExponent
-      : conn_.transportSettings.ackDelayExponent;
-  auto largestAckedPacketNum = *largestAckToSend(ackState_);
-  auto ackingTime = Clock::now();
-  DCHECK(ackState_.largestRecvdPacketTime.hasValue())
-      << "Missing received time for the largest acked packet";
-  // assuming that we're going to ack the largest received with highest pri
-  auto receivedTime = *ackState_.largestRecvdPacketTime;
-  std::chrono::microseconds ackDelay =
-      (ackingTime > receivedTime
-           ? std::chrono::duration_cast<std::chrono::microseconds>(
-                 ackingTime - receivedTime)
-           : 0us);
-
-  WriteAckFrameMetaData meta = {
-      ackState_, /* ackState*/
-      ackDelay, /* ackDelay */
-      static_cast<uint8_t>(ackDelayExponentToUse), /* ackDelayExponent */
-      conn_.connectionTime, /* connect timestamp */
-  };
-
-  Optional<WriteAckFrameResult> ackWriteResult;
-
-  bool isAckReceiveTimestampsSupported =
-      conn_.transportSettings.maybeAckReceiveTimestampsConfigSentToPeer &&
-      conn_.maybePeerAckReceiveTimestampsConfig;
-
-  uint64_t peerRequestedTimestampsCount =
-      conn_.maybePeerAckReceiveTimestampsConfig.has_value()
-      ? conn_.maybePeerAckReceiveTimestampsConfig.value()
-            .maxReceiveTimestampsPerAck
-      : 0;
-
-  if (conn_.transportSettings.readEcnOnIngress) {
-    // If ECN is enabled and we can use it, this will currently take
-    // priority over sending receive timestamps. There is currently no provision
-    // for a frame time that includes both ECN counts and receive timestamps.
-    // TODO: explore design changes for an ACK frame that supports both ECN and
-    // receive timestamps
-    ackWriteResult = writeAckFrame(meta, builder, FrameType::ACK_ECN);
-  } else if (
-      isAckReceiveTimestampsSupported && (peerRequestedTimestampsCount > 0)) {
-    // Use ACK_RECEIVE_TIMESTAMPS if its enabled on both endpoints AND the peer
-    // requests at least 1 timestamp
-    ackWriteResult = writeAckFrameWithReceivedTimestamps(
-        meta,
-        builder,
-        conn_.transportSettings.maybeAckReceiveTimestampsConfigSentToPeer
-            .value(),
-        peerRequestedTimestampsCount);
-  } else {
-    ackWriteResult = writeAckFrame(meta, builder, FrameType::ACK);
-  }
-  if (!ackWriteResult) {
-    return none;
-  }
-  return largestAckedPacketNum;
-}
-
-bool AckScheduler::hasPendingAcks() const {
-  return hasAcksToSchedule(ackState_);
-}
-
 RstStreamScheduler::RstStreamScheduler(const QuicConnectionStateBase& conn)
     : conn_(conn) {}
 
@@ -643,14 +603,31 @@ bool RstStreamScheduler::hasPendingRsts() const {
   return !conn_.pendingEvents.resets.empty();
 }
 
-bool RstStreamScheduler::writeRsts(PacketBuilderInterface& builder) {
+folly::Expected<bool, QuicError> RstStreamScheduler::writeRsts(
+    PacketBuilderInterface& builder) {
   bool rstWritten = false;
   for (const auto& resetStream : conn_.pendingEvents.resets) {
-    auto bytesWritten = writeFrame(resetStream.second, builder);
-    if (!bytesWritten) {
-      break;
+    auto streamId = resetStream.first;
+    QuicStreamState* streamState =
+        conn_.streamManager->getStream(streamId).value_or(nullptr);
+    CHECK(streamState) << "Stream " << streamId
+                       << " not found when going through resets";
+    if (streamState->pendingWrites.empty() &&
+        streamState->writeBufMeta.length == 0) {
+      //    We only write a RESET_STREAM or RESET_STREAM_AT frame for a stream
+      //    once we've written out all data that needs to be delivered reliably.
+      //    While this is not something that's mandated by the spec, we're doing
+      //    it in this implementation because it dramatically simplifies flow
+      //    control accounting.
+      auto bytesWrittenResult = writeFrame(resetStream.second, builder);
+      if (bytesWrittenResult.hasError()) {
+        return folly::makeUnexpected(bytesWrittenResult.error());
+      }
+      if (!bytesWrittenResult.value()) {
+        break;
+      }
+      rstWritten = true;
     }
-    rstWritten = true;
   }
   return rstWritten;
 }
@@ -689,7 +666,10 @@ bool PingFrameScheduler::hasPingFrame() const {
 }
 
 bool PingFrameScheduler::writePing(PacketBuilderInterface& builder) {
-  return 0 != writeFrame(PingFrame(), builder);
+  auto writeFrameResult = writeFrame(PingFrame(), builder);
+  // We shouldn't ever error on a PING.
+  CHECK(!writeFrameResult.hasError());
+  return writeFrameResult.value() != 0;
 }
 
 DatagramFrameScheduler::DatagramFrameScheduler(QuicConnectionStateBase& conn)
@@ -699,7 +679,7 @@ bool DatagramFrameScheduler::hasPendingDatagramFrames() const {
   return !conn_.datagramState.writeBuffer.empty();
 }
 
-bool DatagramFrameScheduler::writeDatagramFrames(
+folly::Expected<bool, QuicError> DatagramFrameScheduler::writeDatagramFrames(
     PacketBuilderInterface& builder) {
   bool sent = false;
   for (size_t i = 0; i <= conn_.datagramState.writeBuffer.size(); ++i) {
@@ -707,15 +687,26 @@ bool DatagramFrameScheduler::writeDatagramFrames(
     auto len = payload.chainLength();
     uint64_t spaceLeft = builder.remainingSpaceInPkt();
     QuicInteger frameTypeQuicInt(static_cast<uint8_t>(FrameType::DATAGRAM_LEN));
+    auto frameTypeSize = frameTypeQuicInt.getSize();
+    if (frameTypeSize.hasError()) {
+      return folly::makeUnexpected(frameTypeSize.error());
+    }
     QuicInteger datagramLenInt(len);
+    auto datagramLenSize = datagramLenInt.getSize();
+    if (datagramLenSize.hasError()) {
+      return folly::makeUnexpected(datagramLenSize.error());
+    }
     auto datagramFrameLength =
-        frameTypeQuicInt.getSize() + len + datagramLenInt.getSize();
+        frameTypeSize.value() + len + datagramLenSize.value();
     if (folly::to<uint64_t>(datagramFrameLength) <= spaceLeft) {
       auto datagramFrame = DatagramFrame(len, payload.move());
       auto res = writeFrame(datagramFrame, builder);
+      if (res.hasError()) {
+        return folly::makeUnexpected(res.error());
+      }
       // Must always succeed since we have already checked that there is enough
       // space to write the frame
-      CHECK_GT(res, 0);
+      CHECK_GT(res.value(), 0);
       QUIC_STATS(conn_.statsCallback, onDatagramWrite, len);
       conn_.datagramState.writeBuffer.pop_front();
       sent = true;
@@ -736,13 +727,16 @@ bool WindowUpdateScheduler::hasPendingWindowUpdates() const {
       conn_.pendingEvents.connWindowUpdate;
 }
 
-void WindowUpdateScheduler::writeWindowUpdates(
-    PacketBuilderInterface& builder) {
+folly::Expected<folly::Unit, QuicError>
+WindowUpdateScheduler::writeWindowUpdates(PacketBuilderInterface& builder) {
   if (conn_.pendingEvents.connWindowUpdate) {
     auto maxDataFrame = generateMaxDataFrame(conn_);
     auto maximumData = maxDataFrame.maximumData;
-    auto bytes = writeFrame(std::move(maxDataFrame), builder);
-    if (bytes) {
+    auto bytesResult = writeFrame(std::move(maxDataFrame), builder);
+    if (bytesResult.hasError()) {
+      return folly::makeUnexpected(bytesResult.error());
+    }
+    if (bytesResult.value()) {
       VLOG(4) << "Wrote max_data=" << maximumData << " " << conn_;
     }
   }
@@ -753,13 +747,17 @@ void WindowUpdateScheduler::writeWindowUpdates(
     }
     auto maxStreamDataFrame = generateMaxStreamDataFrame(*stream);
     auto maximumData = maxStreamDataFrame.maximumData;
-    auto bytes = writeFrame(std::move(maxStreamDataFrame), builder);
-    if (!bytes) {
+    auto bytesResult = writeFrame(std::move(maxStreamDataFrame), builder);
+    if (bytesResult.hasError()) {
+      return folly::makeUnexpected(bytesResult.error());
+    }
+    if (!bytesResult.value()) {
       break;
     }
     VLOG(4) << "Wrote max_stream_data stream=" << stream->id
             << " maximumData=" << maximumData << " " << conn_;
   }
+  return folly::unit;
 }
 
 BlockedScheduler::BlockedScheduler(const QuicConnectionStateBase& conn)
@@ -770,25 +768,33 @@ bool BlockedScheduler::hasPendingBlockedFrames() const {
       conn_.pendingEvents.sendDataBlocked;
 }
 
-void BlockedScheduler::writeBlockedFrames(PacketBuilderInterface& builder) {
+folly::Expected<folly::Unit, QuicError> BlockedScheduler::writeBlockedFrames(
+    PacketBuilderInterface& builder) {
   if (conn_.pendingEvents.sendDataBlocked) {
     // Connection is write blocked due to connection level flow control.
     DataBlockedFrame blockedFrame(
         conn_.flowControlState.peerAdvertisedMaxOffset);
     auto result = writeFrame(blockedFrame, builder);
-    if (!result) {
+    if (result.hasError()) {
+      return folly::makeUnexpected(result.error());
+    }
+    if (!result.value()) {
       // If there is not enough room to write data blocked frame in the
       // current packet, we won't be able to write stream blocked frames either
       // so just return.
-      return;
+      return folly::unit;
     }
   }
   for (const auto& blockedStream : conn_.streamManager->blockedStreams()) {
-    auto bytesWritten = writeFrame(blockedStream.second, builder);
-    if (!bytesWritten) {
+    auto bytesWrittenResult = writeFrame(blockedStream.second, builder);
+    if (bytesWrittenResult.hasError()) {
+      return folly::makeUnexpected(bytesWrittenResult.error());
+    }
+    if (!bytesWrittenResult.value()) {
       break;
     }
   }
+  return folly::unit;
 }
 
 CryptoStreamScheduler::CryptoStreamScheduler(
@@ -796,7 +802,8 @@ CryptoStreamScheduler::CryptoStreamScheduler(
     const QuicCryptoStream& cryptoStream)
     : conn_(conn), cryptoStream_(cryptoStream) {}
 
-bool CryptoStreamScheduler::writeCryptoData(PacketBuilderInterface& builder) {
+folly::Expected<bool, QuicError> CryptoStreamScheduler::writeCryptoData(
+    PacketBuilderInterface& builder) {
   bool cryptoDataWritten = false;
   uint64_t writableData =
       folly::to<uint64_t>(cryptoStream_.pendingWrites.chainLength());
@@ -805,21 +812,27 @@ bool CryptoStreamScheduler::writeCryptoData(PacketBuilderInterface& builder) {
   // will always take precedence over the crypto data.
   for (const auto& buffer : cryptoStream_.lossBuffer) {
     auto res = writeCryptoFrame(buffer.offset, buffer.data, builder);
-    if (!res) {
+    if (res.hasError()) {
+      return folly::makeUnexpected(res.error());
+    }
+    if (!res.value()) {
       return cryptoDataWritten;
     }
     VLOG(4) << "Wrote retransmitted crypto" << " offset=" << buffer.offset
-            << " bytes=" << res->len << " " << conn_;
+            << " bytes=" << res.value()->len << " " << conn_;
     cryptoDataWritten = true;
   }
 
   if (writableData != 0) {
     auto res = writeCryptoFrame(
         cryptoStream_.currentWriteOffset, cryptoStream_.pendingWrites, builder);
-    if (res) {
+    if (res.hasError()) {
+      return folly::makeUnexpected(res.error());
+    }
+    if (res.value()) {
       VLOG(4) << "Wrote crypto frame"
               << " offset=" << cryptoStream_.currentWriteOffset
-              << " bytesWritten=" << res->len << " " << conn_;
+              << " bytesWritten=" << res.value()->len << " " << conn_;
       cryptoDataWritten = true;
     }
   }
@@ -841,7 +854,10 @@ bool ImmediateAckFrameScheduler::hasPendingImmediateAckFrame() const {
 
 bool ImmediateAckFrameScheduler::writeImmediateAckFrame(
     PacketBuilderInterface& builder) {
-  return 0 != writeFrame(ImmediateAckFrame(), builder);
+  auto result = writeFrame(ImmediateAckFrame(), builder);
+  // We shouldn't ever error on an IMMEDIATE_ACK.
+  CHECK(!result.hasError());
+  return result.value() != 0;
 }
 
 CloningScheduler::CloningScheduler(
@@ -859,9 +875,13 @@ bool CloningScheduler::hasData() const {
       conn_.outstandings.numOutstanding() > conn_.outstandings.dsrCount;
 }
 
-SchedulingResult CloningScheduler::scheduleFramesForPacket(
+folly::Expected<SchedulingResult, QuicError>
+CloningScheduler::scheduleFramesForPacket(
     PacketBuilderInterface&& builder,
     uint32_t writableBytes) {
+  // Store header type information before any moves
+  auto builderPnSpace = builder.getPacketHeader().getPacketNumberSpace();
+  auto header = builder.getPacketHeader();
   // The writableBytes in this function shouldn't be limited by cwnd, since
   // we only use CloningScheduler for the cases that we want to bypass cwnd for
   // now.
@@ -879,7 +899,6 @@ SchedulingResult CloningScheduler::scheduleFramesForPacket(
   }
   // TODO: We can avoid the copy & rebuild of the header by creating an
   // independent header builder.
-  auto header = builder.getPacketHeader();
   std::move(builder).releaseOutputBuffer();
   // Look for an outstanding packet that's no larger than the writableBytes
   for (auto& outstandingPacket : conn_.outstandings.packets) {
@@ -892,7 +911,6 @@ SchedulingResult CloningScheduler::scheduleFramesForPacket(
     // clone packet. So re-create a RegularQuicPacketBuilder every time.
     // TODO: We can avoid the copy & rebuild of the header by creating an
     // independent header builder.
-    auto builderPnSpace = builder.getPacketHeader().getPacketNumberSpace();
     if (opPnSpace != builderPnSpace) {
       continue;
     }
@@ -926,6 +944,9 @@ SchedulingResult CloningScheduler::scheduleFramesForPacket(
         if (std::find_if(frames.begin(), frames.end(), [](const auto& frame) {
               return frame.type() == QuicWriteFrame::Type::WriteCryptoFrame;
             }) != frames.end()) {
+          if (conn_.transportSettings.cloneCryptoPacketsAtMostOnce) {
+            continue;
+          }
           auto mostRecentOutstandingPacketIdentifier =
               conn_.outstandings.packets.back().maybeClonedPacketIdentifier;
           if (mostRecentOutstandingPacketIdentifier ==
@@ -948,7 +969,10 @@ SchedulingResult CloningScheduler::scheduleFramesForPacket(
     }
 
     internalBuilder->accountForCipherOverhead(cipherOverhead_);
-    internalBuilder->encodePacketHeader();
+    auto encodeRes = internalBuilder->encodePacketHeader();
+    if (encodeRes.hasError()) {
+      return folly::makeUnexpected(encodeRes.error());
+    }
     PacketRebuilder rebuilder(*internalBuilder, conn_);
 
     // TODO: It's possible we write out a packet that's larger than the
@@ -961,10 +985,13 @@ SchedulingResult CloningScheduler::scheduleFramesForPacket(
     // Or we can throw away the built packet and send a ping.
 
     // Rebuilder will write the rest of frames
-    auto rebuildResult = rebuilder.rebuildFromPacket(outstandingPacket);
-    if (rebuildResult) {
+    auto rebuildResultExpected = rebuilder.rebuildFromPacket(outstandingPacket);
+    if (rebuildResultExpected.hasError()) {
+      return folly::makeUnexpected(rebuildResultExpected.error());
+    }
+    if (rebuildResultExpected.value()) {
       return SchedulingResult(
-          std::move(rebuildResult),
+          std::move(rebuildResultExpected.value()),
           std::move(*internalBuilder).buildPacket(),
           0);
     } else if (
