@@ -18,8 +18,8 @@
 #include <quic/congestion_control/Bandwidth.h>
 #include <quic/observer/SocketObserverContainer.h>
 #include <quic/observer/SocketObserverTypes.h>
+#include <quic/priority/PriorityQueue.h>
 #include <quic/state/QuicConnectionStats.h>
-#include <quic/state/QuicPriorityQueue.h>
 #include <quic/state/QuicStreamGroupRetransmissionPolicy.h>
 #include <quic/state/QuicStreamUtilities.h>
 #include <quic/state/StateData.h>
@@ -33,63 +33,6 @@ class DSRPacketizationRequestSender;
 
 class QuicSocket : virtual public QuicSocketLite {
  public:
-  /**
-   * Information about the transport, similar to what TCP has.
-   */
-  struct TransportInfo {
-    // Time when the connection started.
-    TimePoint connectionTime;
-    std::chrono::microseconds srtt{0us};
-    std::chrono::microseconds rttvar{0us};
-    std::chrono::microseconds lrtt{0us};
-    OptionalMicros maybeLrtt;
-    OptionalMicros maybeLrttAckDelay;
-    OptionalMicros maybeMinRtt;
-    OptionalMicros maybeMinRttNoAckDelay;
-    uint64_t mss{kDefaultUDPSendPacketLen};
-    CongestionControlType congestionControlType{CongestionControlType::None};
-    uint64_t writableBytes{0};
-    uint64_t congestionWindow{0};
-    uint64_t pacingBurstSize{0};
-    std::chrono::microseconds pacingInterval{0us};
-    uint32_t packetsRetransmitted{0};
-    uint32_t totalPacketsSent{0};
-    uint32_t totalAckElicitingPacketsSent{0};
-    uint32_t totalPacketsMarkedLost{0};
-    uint32_t totalPacketsMarkedLostByTimeout{0};
-    uint32_t totalPacketsMarkedLostByReorderingThreshold{0};
-    uint32_t totalPacketsSpuriouslyMarkedLost{0};
-    uint32_t timeoutBasedLoss{0};
-    std::chrono::microseconds pto{0us};
-    // Number of Bytes (packet header + body) that were sent
-    uint64_t bytesSent{0};
-    // Number of Bytes (packet header + body) that were acked
-    uint64_t bytesAcked{0};
-    // Number of Bytes (packet header + body) that were received
-    uint64_t bytesRecvd{0};
-    // Number of Bytes (packet header + body) that are in-flight
-    uint64_t bytesInFlight{0};
-    // Number of Bytes (packet header + body) that were retxed
-    uint64_t totalBytesRetransmitted{0};
-    // Number of Bytes (only the encoded packet's body) that were sent
-    uint64_t bodyBytesSent{0};
-    // Number of Bytes (only the encoded packet's body) that were acked
-    uint64_t bodyBytesAcked{0};
-    // Total number of stream bytes sent on this connection.
-    // Includes retransmissions of stream bytes.
-    uint64_t totalStreamBytesSent{0};
-    // Total number of 'new' stream bytes sent on this connection.
-    // Does not include retransmissions of stream bytes.
-    uint64_t totalNewStreamBytesSent{0};
-    uint32_t ptoCount{0};
-    uint32_t totalPTOCount{0};
-    Optional<PacketNum> largestPacketAckedByPeer;
-    Optional<PacketNum> largestPacketSent;
-    bool usedZeroRtt{false};
-    // State from congestion control module, if one is installed.
-    Optional<CongestionController::State> maybeCCState;
-  };
-
   /**
    * Sets the functions that mvfst will invoke to validate early data params
    * and encode early data params to NewSessionTicket.
@@ -158,18 +101,8 @@ class QuicSocket : virtual public QuicSocketLite {
   /**
    * Get the original Quic Server Connection ID chosen by client
    */
-  FOLLY_NODISCARD virtual Optional<ConnectionId>
-  getClientChosenDestConnectionId() const = 0;
-
-  /**
-   * Get the original peer socket address
-   */
-  virtual const folly::SocketAddress& getOriginalPeerAddress() const = 0;
-
-  /**
-   * Get the local socket address
-   */
-  virtual const folly::SocketAddress& getLocalAddress() const = 0;
+  [[nodiscard]] virtual Optional<ConnectionId> getClientChosenDestConnectionId()
+      const = 0;
 
   virtual bool replaySafe() const = 0;
 
@@ -193,11 +126,6 @@ class QuicSocket : virtual public QuicSocketLite {
   virtual folly::Expected<size_t, LocalErrorCode> getStreamWriteBufferedBytes(
       StreamId id) const = 0;
 
-  /**
-   * Get internal transport info similar to TCP information.
-   */
-  virtual TransportInfo getTransportInfo() const = 0;
-
   /**
    * Returns the current flow control windows for the connection.
    * Use getStreamFlowControl for stream flow control window.
@@ -226,18 +154,11 @@ class QuicSocket : virtual public QuicSocketLite {
   virtual folly::Expected<folly::Unit, LocalErrorCode>
   setStreamFlowControlWindow(StreamId id, uint64_t windowSize) = 0;
 
-  /**
-   * Sets the maximum pacing rate in Bytes per second to be used
-   * if pacing is enabled
-   */
-  virtual folly::Expected<folly::Unit, LocalErrorCode> setMaxPacingRate(
-      uint64_t rateBytesPerSec) = 0;
-
   /**
    * Get stream priority.
    */
-  virtual folly::Expected<Priority, LocalErrorCode> getStreamPriority(
-      StreamId id) = 0;
+  virtual folly::Expected<PriorityQueue::Priority, LocalErrorCode>
+  getStreamPriority(StreamId id) = 0;
 
   /**
    * Convenience function that sets the read callbacks of all streams to be
@@ -383,45 +304,6 @@ class QuicSocket : virtual public QuicSocketLite {
   virtual StreamDirectionality getStreamDirectionality(
       StreamId stream) noexcept = 0;
 
-  /**
-   * Callback class for receiving ack notifications
-   */
-  class DeliveryCallback : public ByteEventCallback {
-   public:
-    ~DeliveryCallback() override = default;
-
-    /**
-     * Invoked when the peer has acknowledged the receipt of the specified
-     * offset.  rtt is the current RTT estimate for the connection.
-     */
-    virtual void onDeliveryAck(
-        StreamId id,
-        uint64_t offset,
-        std::chrono::microseconds rtt) = 0;
-
-    /**
-     * Invoked on registered delivery callbacks when the bytes will never be
-     * delivered (due to a reset or other error).
-     */
-    virtual void onCanceled(StreamId id, uint64_t offset) = 0;
-
-   private:
-    // Temporary shim during transition to ByteEvent
-    void onByteEventRegistered(ByteEvent /* byteEvent */) final {
-      // Not supported
-    }
-    void onByteEvent(ByteEvent byteEvent) final {
-      CHECK_EQ((int)ByteEvent::Type::ACK, (int)byteEvent.type); // sanity
-      onDeliveryAck(byteEvent.id, byteEvent.offset, byteEvent.srtt);
-    }
-
-    // Temporary shim during transition to ByteEvent
-    void onByteEventCanceled(ByteEventCancellation cancellation) final {
-      CHECK_EQ((int)ByteEvent::Type::ACK, (int)cancellation.type); // sanity
-      onCanceled(cancellation.id, cancellation.offset);
-    }
-  };
-
   /**
    * Register a callback to be invoked when the stream offset was transmitted.
    *
@@ -496,110 +378,6 @@ class QuicSocket : virtual public QuicSocketLite {
    */
   virtual bool isDetachable() = 0;
 
-  /**
-   * Set a throttling signal provider
-   */
-  virtual void setThrottlingSignalProvider(
-      std::shared_ptr<ThrottlingSignalProvider>) = 0;
-
-  using Observer = SocketObserverContainer::Observer;
-  using ManagedObserver = SocketObserverContainer::ManagedObserver;
-
-  /**
-   * Adds an observer.
-   *
-   * If the observer is already added, this is a no-op.
-   *
-   * @param observer     Observer to add.
-   * @return             Whether the observer was added (fails if no list).
-   */
-  bool addObserver(Observer* observer) {
-    if (auto list = getSocketObserverContainer()) {
-      list->addObserver(observer);
-      return true;
-    }
-    return false;
-  }
-
-  /**
-   * Adds an observer.
-   *
-   * If the observer is already added, this is a no-op.
-   *
-   * @param observer     Observer to add.
-   * @return             Whether the observer was added (fails if no list).
-   */
-  bool addObserver(std::shared_ptr<Observer> observer) {
-    if (auto list = getSocketObserverContainer()) {
-      list->addObserver(std::move(observer));
-      return true;
-    }
-    return false;
-  }
-
-  /**
-   * Removes an observer.
-   *
-   * @param observer     Observer to remove.
-   * @return             Whether the observer was found and removed.
-   */
-  bool removeObserver(Observer* observer) {
-    if (auto list = getSocketObserverContainer()) {
-      return list->removeObserver(observer);
-    }
-    return false;
-  }
-
-  /**
-   * Removes an observer.
-   *
-   * @param observer     Observer to remove.
-   * @return             Whether the observer was found and removed.
-   */
-  bool removeObserver(std::shared_ptr<Observer> observer) {
-    if (auto list = getSocketObserverContainer()) {
-      return list->removeObserver(std::move(observer));
-    }
-    return false;
-  }
-
-  /**
-   * Get number of observers.
-   *
-   * @return             Number of observers.
-   */
-  [[nodiscard]] size_t numObservers() const {
-    if (auto list = getSocketObserverContainer()) {
-      return list->numObservers();
-    }
-    return 0;
-  }
-
-  /**
-   * Returns list of attached observers.
-   *
-   * @return             List of observers.
-   */
-  std::vector<Observer*> getObservers() {
-    if (auto list = getSocketObserverContainer()) {
-      return list->getObservers();
-    }
-    return {};
-  }
-
-  /**
-   * Returns list of attached observers that are of type T.
-   *
-   * @return             Attached observers of type T.
-   */
-  template <typename T = Observer>
-  std::vector<T*> findObservers() {
-    if (auto list = getSocketObserverContainer()) {
-      return list->findObservers<T>();
-    }
-    return {};
-  }
-
   /**
    * ===== Datagram API =====
    *
@@ -617,7 +395,7 @@ class QuicSocket : virtual public QuicSocketLite {
    * Returns the maximum allowed Datagram payload size.
    * 0 means Datagram is not supported
    */
-  FOLLY_NODISCARD virtual uint16_t getDatagramSizeLimit() const = 0;
+  [[nodiscard]] virtual uint16_t getDatagramSizeLimit() const = 0;
 
   /**
    * Writes a Datagram frame. If buf is larger than the size limit returned by
