@@ -42,12 +42,6 @@ QuicTransportBase::QuicTransportBase(
           std::move(evb),
           std::move(socket),
           useConnectionEndWithErrorCallback) {
-  writeLooper_->setPacingFunction([this]() -> auto {
-    if (isConnectionPaced(*conn_)) {
-      return conn_->pacer->getTimeUntilNextWrite();
-    }
-    return 0us;
-  });
   if (socket_) {
     folly::Function<Optional<folly::SocketCmsgMap>()> func = [&]() {
       return getAdditionalCmsgsForAsyncUDPSocket();
@@ -63,40 +57,6 @@ void QuicTransportBase::setPacingTimer(
   }
 }
 
-const std::shared_ptr<QLogger> QuicTransportBase::getQLogger() const {
-  return conn_->qLogger;
-}
-
-void QuicTransportBase::setQLogger(std::shared_ptr<QLogger> qLogger) {
-  // setQLogger can be called multiple times for the same connection and with
-  // the same qLogger we track the number of times it gets set and the number
-  // of times it gets reset, and only stop qlog collection when the number of
-  // resets equals the number of times the logger was set
-  if (!conn_->qLogger) {
-    CHECK_EQ(qlogRefcnt_, 0);
-  } else {
-    CHECK_GT(qlogRefcnt_, 0);
-  }
-
-  if (qLogger) {
-    conn_->qLogger = std::move(qLogger);
-    conn_->qLogger->setDcid(conn_->clientChosenDestConnectionId);
-    if (conn_->nodeType == QuicNodeType::Server) {
-      conn_->qLogger->setScid(conn_->serverConnectionId);
-    } else {
-      conn_->qLogger->setScid(conn_->clientConnectionId);
-    }
-    qlogRefcnt_++;
-  } else {
-    if (conn_->qLogger) {
-      qlogRefcnt_--;
-      if (qlogRefcnt_ == 0) {
-        conn_->qLogger = nullptr;
-      }
-    }
-  }
-}
-
 Optional<ConnectionId> QuicTransportBase::getClientConnectionId() const {
   return conn_->clientConnectionId;
 }
@@ -110,15 +70,6 @@ Optional<ConnectionId> QuicTransportBase::getClientChosenDestConnectionId()
   return conn_->clientChosenDestConnectionId;
 }
 
-const folly::SocketAddress& QuicTransportBase::getOriginalPeerAddress() const {
-  return conn_->originalPeerAddress;
-}
-
-const folly::SocketAddress& QuicTransportBase::getLocalAddress() const {
-  return socket_ && socket_->isBound() ? socket_->address()
-                                       : localFallbackAddress;
-}
-
 QuicTransportBase::~QuicTransportBase() {
   resetConnectionCallbacks();
   // Just in case this ended up hanging around.
@@ -173,7 +124,8 @@ folly::Expected<size_t, LocalErrorCode> QuicTransportBase::getStreamWriteOffset(
     return folly::makeUnexpected(LocalErrorCode::STREAM_NOT_EXISTS);
   }
   try {
-    auto stream = CHECK_NOTNULL(conn_->streamManager->getStream(id));
+    auto stream =
+        CHECK_NOTNULL(conn_->streamManager->getStream(id).value_or(nullptr));
     return stream->currentWriteOffset;
   } catch (const QuicInternalException& ex) {
     VLOG(4) << __func__ << " " << ex.what() << " " << *this;
@@ -196,7 +148,8 @@ QuicTransportBase::getStreamWriteBufferedBytes(StreamId id) const {
     return folly::makeUnexpected(LocalErrorCode::STREAM_NOT_EXISTS);
   }
   try {
-    auto stream = CHECK_NOTNULL(conn_->streamManager->getStream(id));
+    auto stream =
+        CHECK_NOTNULL(conn_->streamManager->getStream(id).value_or(nullptr));
     return stream->pendingWrites.chainLength();
   } catch (const QuicInternalException& ex) {
     VLOG(4) << __func__ << " " << ex.what() << " " << *this;
@@ -210,81 +163,6 @@ QuicTransportBase::getStreamWriteBufferedBytes(StreamId id) const {
   }
 }
 
-/**
- * Getters for details from the transport/security layers such as
- * RTT, rxmit, cwnd, mss, app protocol, handshake latency,
- * client proposed ciphers, etc.
- */
-
-QuicSocket::TransportInfo QuicTransportBase::getTransportInfo() const {
-  CongestionControlType congestionControlType = CongestionControlType::None;
-  uint64_t writableBytes = std::numeric_limits<uint64_t>::max();
-  uint64_t congestionWindow = std::numeric_limits<uint64_t>::max();
-  Optional<CongestionController::State> maybeCCState;
-  uint64_t burstSize = 0;
-  std::chrono::microseconds pacingInterval = 0ms;
-  if (conn_->congestionController) {
-    congestionControlType = conn_->congestionController->type();
-    writableBytes = conn_->congestionController->getWritableBytes();
-    congestionWindow = conn_->congestionController->getCongestionWindow();
-    maybeCCState = conn_->congestionController->getState();
-    if (isConnectionPaced(*conn_)) {
-      burstSize = conn_->pacer->getCachedWriteBatchSize();
-      pacingInterval = conn_->pacer->getTimeUntilNextWrite();
-    }
-  }
-  TransportInfo transportInfo;
-  transportInfo.connectionTime = conn_->connectionTime;
-  transportInfo.srtt = conn_->lossState.srtt;
-  transportInfo.rttvar = conn_->lossState.rttvar;
-  transportInfo.lrtt = conn_->lossState.lrtt;
-  transportInfo.maybeLrtt = conn_->lossState.maybeLrtt;
-  transportInfo.maybeLrttAckDelay = conn_->lossState.maybeLrttAckDelay;
-  if (conn_->lossState.mrtt != kDefaultMinRtt) {
-    transportInfo.maybeMinRtt = conn_->lossState.mrtt;
-  }
-  transportInfo.maybeMinRttNoAckDelay = conn_->lossState.maybeMrttNoAckDelay;
-  transportInfo.mss = conn_->udpSendPacketLen;
-  transportInfo.congestionControlType = congestionControlType;
-  transportInfo.writableBytes = writableBytes;
-  transportInfo.congestionWindow = congestionWindow;
-  transportInfo.pacingBurstSize = burstSize;
-  transportInfo.pacingInterval = pacingInterval;
-  transportInfo.packetsRetransmitted = conn_->lossState.rtxCount;
-  transportInfo.totalPacketsSent = conn_->lossState.totalPacketsSent;
-  transportInfo.totalAckElicitingPacketsSent =
-      conn_->lossState.totalAckElicitingPacketsSent;
-  transportInfo.totalPacketsMarkedLost =
-      conn_->lossState.totalPacketsMarkedLost;
-  transportInfo.totalPacketsMarkedLostByTimeout =
-      conn_->lossState.totalPacketsMarkedLostByTimeout;
-  transportInfo.totalPacketsMarkedLostByReorderingThreshold =
-      conn_->lossState.totalPacketsMarkedLostByReorderingThreshold;
-  transportInfo.totalPacketsSpuriouslyMarkedLost =
-      conn_->lossState.totalPacketsSpuriouslyMarkedLost;
-  transportInfo.timeoutBasedLoss = conn_->lossState.timeoutBasedRtxCount;
-  transportInfo.totalBytesRetransmitted =
-      conn_->lossState.totalBytesRetransmitted;
-  transportInfo.pto = calculatePTO(*conn_);
-  transportInfo.bytesSent = conn_->lossState.totalBytesSent;
-  transportInfo.bytesAcked = conn_->lossState.totalBytesAcked;
-  transportInfo.bytesRecvd = conn_->lossState.totalBytesRecvd;
-  transportInfo.bytesInFlight = conn_->lossState.inflightBytes;
-  transportInfo.bodyBytesSent = conn_->lossState.totalBodyBytesSent;
-  transportInfo.bodyBytesAcked = conn_->lossState.totalBodyBytesAcked;
-  transportInfo.totalStreamBytesSent = conn_->lossState.totalStreamBytesSent;
-  transportInfo.totalNewStreamBytesSent =
-      conn_->lossState.totalNewStreamBytesSent;
-  transportInfo.ptoCount = conn_->lossState.ptoCount;
-  transportInfo.totalPTOCount = conn_->lossState.totalPTOCount;
-  transportInfo.largestPacketAckedByPeer =
-      conn_->ackStates.appDataAckState.largestAckedByPeer;
-  transportInfo.largestPacketSent = conn_->lossState.largestSent;
-  transportInfo.usedZeroRtt = conn_->usedZeroRtt;
-  transportInfo.maybeCCState = maybeCCState;
-  return transportInfo;
-}
-
 folly::Expected<QuicSocket::FlowControlState, LocalErrorCode>
 QuicTransportBase::getConnectionFlowControl() const {
   return QuicSocket::FlowControlState(
@@ -303,7 +181,8 @@ QuicTransportBase::getMaxWritableOnStream(StreamId id) const {
     return folly::makeUnexpected(LocalErrorCode::INVALID_OPERATION);
   }
 
-  auto stream = CHECK_NOTNULL(conn_->streamManager->getStream(id));
+  auto stream =
+      CHECK_NOTNULL(conn_->streamManager->getStream(id).value_or(nullptr));
   return maxWritableOnStream(*stream);
 }
 
@@ -328,7 +207,8 @@ QuicTransportBase::setStreamFlowControlWindow(
   if (!conn_->streamManager->streamExists(id)) {
     return folly::makeUnexpected(LocalErrorCode::STREAM_NOT_EXISTS);
   }
-  auto stream = CHECK_NOTNULL(conn_->streamManager->getStream(id));
+  auto stream =
+      CHECK_NOTNULL(conn_->streamManager->getStream(id).value_or(nullptr));
   stream->flowControlState.windowSize = windowSize;
   maybeSendStreamWindowUpdate(*stream, Clock::now());
   updateWriteLooper(true);
@@ -471,7 +351,8 @@ folly::Expected<folly::Unit, LocalErrorCode> QuicTransportBase::peek(
   if (!conn_->streamManager->streamExists(id)) {
     return folly::makeUnexpected(LocalErrorCode::STREAM_NOT_EXISTS);
   }
-  auto stream = CHECK_NOTNULL(conn_->streamManager->getStream(id));
+  auto stream =
+      CHECK_NOTNULL(conn_->streamManager->getStream(id).value_or(nullptr));
 
   if (stream->streamReadError) {
     switch (stream->streamReadError->type()) {
@@ -493,7 +374,8 @@ folly::Expected<folly::Unit, LocalErrorCode> QuicTransportBase::consume(
   if (!conn_->streamManager->streamExists(id)) {
     return folly::makeUnexpected(LocalErrorCode::STREAM_NOT_EXISTS);
   }
-  auto stream = CHECK_NOTNULL(conn_->streamManager->getStream(id));
+  auto stream =
+      CHECK_NOTNULL(conn_->streamManager->getStream(id).value_or(nullptr));
   auto result = consume(id, stream->currentReadOffset, amount);
   if (result.hasError()) {
     return folly::makeUnexpected(result.error().first);
@@ -523,7 +405,8 @@ QuicTransportBase::consume(StreamId id, uint64_t offset, size_t amount) {
       return folly::makeUnexpected(
           ConsumeError{LocalErrorCode::STREAM_NOT_EXISTS, readOffset});
     }
-    auto stream = CHECK_NOTNULL(conn_->streamManager->getStream(id));
+    auto stream =
+        CHECK_NOTNULL(conn_->streamManager->getStream(id).value_or(nullptr));
     readOffset = stream->currentReadOffset;
     if (stream->currentReadOffset != offset) {
       return folly::makeUnexpected(
@@ -691,7 +574,8 @@ void QuicTransportBase::resetNonControlStreams(
       auto readCallbackIt = readCallbacks_.find(id);
       if (readCallbackIt != readCallbacks_.end() &&
           readCallbackIt->second.readCb) {
-        auto stream = CHECK_NOTNULL(conn_->streamManager->getStream(id));
+        auto stream = CHECK_NOTNULL(
+            conn_->streamManager->getStream(id).value_or(nullptr));
         if (!stream->groupId) {
           readCallbackIt->second.readCb->readError(
               id, QuicError(error, errorMsg.str()));
@@ -797,21 +681,8 @@ QuicTransportBase::readDatagramBufs(size_t atMost) {
   return retDatagrams;
 }
 
-folly::Expected<folly::Unit, LocalErrorCode>
-QuicTransportBase::setMaxPacingRate(uint64_t maxRateBytesPerSec) {
-  if (conn_->pacer) {
-    conn_->pacer->setMaxPacingRate(maxRateBytesPerSec);
-    return folly::unit;
-  } else {
-    LOG(WARNING)
-        << "Cannot set max pacing rate without a pacer. Pacing Enabled = "
-        << conn_->transportSettings.pacingEnabled;
-    return folly::makeUnexpected(LocalErrorCode::PACER_NOT_AVAILABLE);
-  }
-}
-
-folly::Expected<Priority, LocalErrorCode> QuicTransportBase::getStreamPriority(
-    StreamId id) {
+folly::Expected<PriorityQueue::Priority, LocalErrorCode>
+QuicTransportBase::getStreamPriority(StreamId id) {
   if (closeState_ != CloseState::OPEN) {
     return folly::makeUnexpected(LocalErrorCode::CONNECTION_CLOSED);
   }
@@ -821,12 +692,6 @@ folly::Expected<Priority, LocalErrorCode> QuicTransportBase::getStreamPriority(
   return folly::makeUnexpected(LocalErrorCode::STREAM_NOT_EXISTS);
 }
 
-void QuicTransportBase::setThrottlingSignalProvider(
-    std::shared_ptr<ThrottlingSignalProvider> throttlingSignalProvider) {
-  DCHECK(conn_);
-  conn_->throttlingSignalProvider = throttlingSignalProvider;
-}
-
 bool QuicTransportBase::isDetachable() {
   // only the client is detachable.
   return conn_->nodeType == QuicNodeType::Client;
@@ -852,7 +717,6 @@ void QuicTransportBase::attachEventBase(std::shared_ptr<QuicEventBase> evbIn) {
   updatePeekLooper();
   updateWriteLooper(false);
 
-#ifndef MVFST_USE_LIBEV
   if (getSocketObserverContainer() &&
       getSocketObserverContainer()
           ->hasObserversForEvent<
@@ -863,7 +727,6 @@ void QuicTransportBase::attachEventBase(std::shared_ptr<QuicEventBase> evbIn) {
               observer->evbAttach(observed, evb_.get());
             });
   }
-#endif
 }
 
 void QuicTransportBase::detachEventBase() {
@@ -935,48 +798,6 @@ void QuicTransportBase::appendCmsgs(const folly::SocketCmsgMap& options) {
   socket_->appendCmsgs(options);
 }
 
-void QuicTransportBase::setBackgroundModeParameters(
-    PriorityLevel maxBackgroundPriority,
-    float backgroundUtilizationFactor) {
-  backgroundPriorityThreshold_.assign(maxBackgroundPriority);
-  backgroundUtilizationFactor_.assign(backgroundUtilizationFactor);
-  conn_->streamManager->setPriorityChangesObserver(this);
-  onStreamPrioritiesChange();
-}
-
-void QuicTransportBase::clearBackgroundModeParameters() {
-  backgroundPriorityThreshold_.clear();
-  backgroundUtilizationFactor_.clear();
-  conn_->streamManager->resetPriorityChangesObserver();
-  onStreamPrioritiesChange();
-}
-
-// If backgroundPriorityThreshold_ and backgroundUtilizationFactor_ are set
-// and all streams have equal or lower priority than the threshold (value >=
-// threshold), set the connection's congestion controller to use background
-// mode with the set utilization factor. In all other cases, turn off the
-// congestion controller's background mode.
-void QuicTransportBase::onStreamPrioritiesChange() {
-  if (conn_->congestionController == nullptr) {
-    return;
-  }
-  if (!backgroundPriorityThreshold_.hasValue() ||
-      !backgroundUtilizationFactor_.hasValue()) {
-    conn_->congestionController->setBandwidthUtilizationFactor(1.0);
-    return;
-  }
-  bool allStreamsBackground = conn_->streamManager->getHighestPriorityLevel() >=
-      backgroundPriorityThreshold_.value();
-  float targetUtilization =
-      allStreamsBackground ? backgroundUtilizationFactor_.value() : 1.0f;
-  VLOG(10) << fmt::format(
-      "Updating transport background mode. Highest Priority={} Threshold={} TargetUtilization={}",
-      conn_->streamManager->getHighestPriorityLevel(),
-      backgroundPriorityThreshold_.value(),
-      targetUtilization);
-  conn_->congestionController->setBandwidthUtilizationFactor(targetUtilization);
-}
-
 bool QuicTransportBase::checkCustomRetransmissionProfilesEnabled() const {
   return quic::checkCustomRetransmissionProfilesEnabled(*conn_);
 }
