@@ -116,7 +116,7 @@ uint64_t maybeUnvalidatedClientWritableBytes(
       conn.udpSendPacketLen;
 }
 
-WriteQuicDataResult writeQuicDataToSocketImpl(
+folly::Expected<WriteQuicDataResult, QuicError> writeQuicDataToSocketImpl(
     QuicAsyncUDPSocket& sock,
     QuicConnectionStateBase& connection,
     const ConnectionId& srcConnId,
@@ -165,8 +165,11 @@ WriteQuicDataResult writeQuicDataToSocketImpl(
         aead,
         headerCipher,
         version);
-    probesWritten = probeResult.probesWritten;
-    bytesWritten += probeResult.bytesWritten;
+    if (!probeResult.hasValue()) {
+      return folly::makeUnexpected(probeResult.error());
+    }
+    probesWritten = probeResult->probesWritten;
+    bytesWritten += probeResult->bytesWritten;
     // We only get one chance to write out the probes.
     numProbePackets = 0;
     packetLimit =
@@ -186,11 +189,9 @@ WriteQuicDataResult writeQuicDataToSocketImpl(
           .pingFrames()
           .datagramFrames()
           .immediateAckFrames();
-  // Only add ACK frames if we need to send an ACK, or if the write reason isn't
-  // just streams.
+  // Only add ACK frames if we need to send an ACK.
   if (connection.transportSettings.opportunisticAcking ||
-      toWriteAppDataAcks(connection) ||
-      (hasNonAckDataToWrite(connection) != WriteDataReason::STREAM)) {
+      toWriteAppDataAcks(connection)) {
     schedulerBuilder.ackFrames();
   }
   if (!exceptCryptoStream) {
@@ -211,8 +212,11 @@ WriteQuicDataResult writeQuicDataToSocketImpl(
       headerCipher,
       version,
       writeLoopBeginTime);
-  packetsWritten += connectionDataResult.packetsWritten;
-  bytesWritten += connectionDataResult.bytesWritten;
+  if (!connectionDataResult.hasValue()) {
+    return folly::makeUnexpected(connectionDataResult.error());
+  }
+  packetsWritten += connectionDataResult->packetsWritten;
+  bytesWritten += connectionDataResult->bytesWritten;
   VLOG_IF(10, packetsWritten || probesWritten)
       << nodeToString(connection.nodeType) << " written data "
       << (exceptCryptoStream ? "without crypto data " : "")
@@ -232,7 +236,8 @@ void updateErrnoCount(
   }
 }
 
-DataPathResult continuousMemoryBuildScheduleEncrypt(
+[[nodiscard]] folly::Expected<DataPathResult, QuicError>
+continuousMemoryBuildScheduleEncrypt(
     QuicConnectionStateBase& connection,
     PacketHeader header,
     PacketNumberSpace pnSpace,
@@ -260,8 +265,11 @@ DataPathResult continuousMemoryBuildScheduleEncrypt(
   CHECK(scheduler.hasData());
   auto result =
       scheduler.scheduleFramesForPacket(std::move(pktBuilder), writableBytes);
+  if (result.hasError()) {
+    return folly::makeUnexpected(result.error());
+  }
   CHECK(connection.bufAccessor->ownsBuffer());
-  auto& packet = result.packet;
+  auto& packet = result->packet;
   if (!packet || packet->packet.frames.empty()) {
     rollbackBuf();
     ioBufBatch.flush();
@@ -322,10 +330,11 @@ DataPathResult continuousMemoryBuildScheduleEncrypt(
   bool ret = ioBufBatch.write(nullptr /* no need to pass buf */, encodedSize);
   updateErrnoCount(connection, ioBufBatch);
   return DataPathResult::makeWriteResult(
-      ret, std::move(result), encodedSize, encodedBodySize);
+      ret, std::move(result.value()), encodedSize, encodedBodySize);
 }
 
-DataPathResult iobufChainBasedBuildScheduleEncrypt(
+[[nodiscard]] folly::Expected<DataPathResult, QuicError>
+iobufChainBasedBuildScheduleEncrypt(
     QuicConnectionStateBase& connection,
     PacketHeader header,
     PacketNumberSpace pnSpace,
@@ -344,7 +353,10 @@ DataPathResult iobufChainBasedBuildScheduleEncrypt(
   pktBuilder.accountForCipherOverhead(cipherOverhead);
   auto result =
       scheduler.scheduleFramesForPacket(std::move(pktBuilder), writableBytes);
-  auto& packet = result.packet;
+  if (result.hasError()) {
+    return folly::makeUnexpected(result.error());
+  }
+  auto& packet = result->packet;
   if (!packet || packet->packet.frames.empty()) {
     ioBufBatch.flush();
     updateErrnoCount(connection, ioBufBatch);
@@ -396,7 +408,7 @@ DataPathResult iobufChainBasedBuildScheduleEncrypt(
   bool ret = ioBufBatch.write(std::move(packetBuf), encodedSize);
   updateErrnoCount(connection, ioBufBatch);
   return DataPathResult::makeWriteResult(
-      ret, std::move(result), encodedSize, encodedBodySize);
+      ret, std::move(result.value()), encodedSize, encodedBodySize);
 }
 
 } // namespace
@@ -530,7 +542,7 @@ void handleRetransmissionBufMetaWritten(
  * with new data, as well as retranmissions. Returns true if the data sent is
  * new data.
  */
-bool handleStreamWritten(
+folly::Expected<bool, QuicError> handleStreamWritten(
     QuicConnectionStateBase& conn,
     QuicStreamLike& stream,
     uint64_t frameOffset,
@@ -544,13 +556,13 @@ bool handleStreamWritten(
     handleNewStreamDataWritten(stream, frameLen, frameFin);
     writtenNewData = true;
   } else if (frameOffset > stream.currentWriteOffset) {
-    throw QuicTransportException(
+    return folly::makeUnexpected(QuicError(
+        TransportErrorCode::INTERNAL_ERROR,
         fmt::format(
             "Byte offset of first byte in written stream frame ({}) is "
             "greater than stream's current write offset ({})",
             frameOffset,
-            stream.currentWriteOffset),
-        TransportErrorCode::INTERNAL_ERROR);
+            stream.currentWriteOffset)));
   }
 
   if (writtenNewData) {
@@ -635,7 +647,7 @@ bool handleStreamBufMetaWritten(
   return false;
 }
 
-void updateConnection(
+folly::Expected<folly::Unit, QuicError> updateConnection(
     QuicConnectionStateBase& conn,
     Optional<ClonedPacketIdentifier> clonedPacketIdentifier,
     RegularQuicWritePacket packet,
@@ -666,8 +678,12 @@ void updateConnection(
       case QuicWriteFrame::Type::WriteStreamFrame: {
         const WriteStreamFrame& writeStreamFrame = *frame.asWriteStreamFrame();
         retransmittable = true;
-        auto stream = CHECK_NOTNULL(
-            conn.streamManager->getStream(writeStreamFrame.streamId));
+        auto streamResult =
+            conn.streamManager->getStream(writeStreamFrame.streamId);
+        if (!streamResult) {
+          return folly::makeUnexpected(streamResult.error());
+        }
+        auto stream = streamResult.value();
         bool newStreamDataWritten = false;
         if (writeStreamFrame.fromBufMeta) {
           newStreamDataWritten = handleStreamBufMetaWritten(
@@ -679,7 +695,7 @@ void updateConnection(
               packetNum,
               packetNumberSpace);
         } else {
-          newStreamDataWritten = handleStreamWritten(
+          auto streamWrittenResult = handleStreamWritten(
               conn,
               *stream,
               writeStreamFrame.offset,
@@ -687,9 +703,17 @@ void updateConnection(
               writeStreamFrame.fin,
               packetNum,
               packetNumberSpace);
+          if (streamWrittenResult.hasError()) {
+            return folly::makeUnexpected(streamWrittenResult.error());
+          }
+          newStreamDataWritten = streamWrittenResult.value();
         }
         if (newStreamDataWritten) {
-          updateFlowControlOnWriteToSocket(*stream, writeStreamFrame.len);
+          auto flowControlResult =
+              updateFlowControlOnWriteToSocket(*stream, writeStreamFrame.len);
+          if (flowControlResult.hasError()) {
+            return folly::makeUnexpected(flowControlResult.error());
+          }
           maybeWriteBlockAfterSocketWrite(*stream);
           maybeWriteDataBlockedAfterSocketWrite(conn);
           conn.streamManager->addTx(writeStreamFrame.streamId);
@@ -707,7 +731,7 @@ void updateConnection(
         // NewSessionTicket is sent in crypto frame encrypted with 1-rtt key,
         // however, it is not part of handshake
         auto encryptionLevel = protectionTypeToEncryptionLevel(protectionType);
-        handleStreamWritten(
+        auto cryptoWritten = handleStreamWritten(
             conn,
             *getCryptoStream(*conn.cryptoState, encryptionLevel),
             writeCryptoFrame.offset,
@@ -715,6 +739,9 @@ void updateConnection(
             false /* fin */,
             packetNum,
             packetNumberSpace);
+        if (cryptoWritten.hasError()) {
+          return folly::makeUnexpected(cryptoWritten.error());
+        }
         break;
       }
       case QuicWriteFrame::Type::WriteAckFrame: {
@@ -777,8 +804,12 @@ void updateConnection(
       case QuicWriteFrame::Type::MaxStreamDataFrame: {
         const MaxStreamDataFrame& maxStreamDataFrame =
             *frame.asMaxStreamDataFrame();
-        auto stream = CHECK_NOTNULL(
-            conn.streamManager->getStream(maxStreamDataFrame.streamId));
+        auto streamResult =
+            conn.streamManager->getStream(maxStreamDataFrame.streamId);
+        if (streamResult.hasError()) {
+          return folly::makeUnexpected(streamResult.error());
+        }
+        auto stream = streamResult.value();
         retransmittable = true;
         VLOG(10) << nodeToString(conn.nodeType)
                  << " sent packet with window update packetNum=" << packetNum
@@ -875,7 +906,7 @@ void updateConnection(
 
   if (!retransmittable && !isPing) {
     DCHECK(!clonedPacketIdentifier);
-    return;
+    return folly::unit;
   }
   conn.lossState.totalAckElicitingPacketsSent++;
 
@@ -964,6 +995,7 @@ void updateConnection(
   } else {
     ++conn.outstandings.packetCount[packetNumberSpace];
   }
+  return folly::unit;
 }
 
 uint64_t probePacketWritableBytes(QuicConnectionStateBase& conn) {
@@ -1047,7 +1079,7 @@ HeaderBuilder ShortHeaderBuilder(ProtectionType keyPhase) {
   };
 }
 
-WriteQuicDataResult writeCryptoAndAckDataToSocket(
+folly::Expected<WriteQuicDataResult, QuicError> writeCryptoAndAckDataToSocket(
     QuicAsyncUDPSocket& sock,
     QuicConnectionStateBase& connection,
     const ConnectionId& srcConnId,
@@ -1095,8 +1127,11 @@ WriteQuicDataResult writeCryptoAndAckDataToSocket(
         headerCipher,
         version,
         token);
-    probesWritten += probeResult.probesWritten;
-    bytesWritten += probeResult.bytesWritten;
+    if (probeResult.hasError()) {
+      return folly::makeUnexpected(probeResult.error());
+    }
+    probesWritten += probeResult->probesWritten;
+    bytesWritten += probeResult->bytesWritten;
   }
   packetLimit = probesWritten > packetLimit ? 0 : (packetLimit - probesWritten);
   // Only get one chance to write probes.
@@ -1118,8 +1153,12 @@ WriteQuicDataResult writeCryptoAndAckDataToSocket(
       Clock::now(),
       token);
 
-  packetsWritten += writeResult.packetsWritten;
-  bytesWritten += writeResult.bytesWritten;
+  if (writeResult.hasError()) {
+    return folly::makeUnexpected(writeResult.error());
+  }
+
+  packetsWritten += writeResult->packetsWritten;
+  bytesWritten += writeResult->bytesWritten;
 
   if (connection.transportSettings.immediatelyRetransmitInitialPackets &&
       packetsWritten > 0 && packetsWritten < packetLimit) {
@@ -1138,8 +1177,11 @@ WriteQuicDataResult writeCryptoAndAckDataToSocket(
         headerCipher,
         version,
         token);
-    probesWritten += cloneResult.probesWritten;
-    bytesWritten += cloneResult.bytesWritten;
+    if (cloneResult.hasError()) {
+      return folly::makeUnexpected(cloneResult.error());
+    }
+    probesWritten += cloneResult->probesWritten;
+    bytesWritten += cloneResult->bytesWritten;
   }
 
   VLOG_IF(10, packetsWritten || probesWritten)
@@ -1151,7 +1193,7 @@ WriteQuicDataResult writeCryptoAndAckDataToSocket(
   return result;
 }
 
-WriteQuicDataResult writeQuicDataToSocket(
+folly::Expected<WriteQuicDataResult, QuicError> writeQuicDataToSocket(
     QuicAsyncUDPSocket& sock,
     QuicConnectionStateBase& connection,
     const ConnectionId& srcConnId,
@@ -1174,7 +1216,8 @@ WriteQuicDataResult writeQuicDataToSocket(
       writeLoopBeginTime);
 }
 
-WriteQuicDataResult writeQuicDataExceptCryptoStreamToSocket(
+folly::Expected<WriteQuicDataResult, QuicError>
+writeQuicDataExceptCryptoStreamToSocket(
     QuicAsyncUDPSocket& socket,
     QuicConnectionStateBase& connection,
     const ConnectionId& srcConnId,
@@ -1196,7 +1239,7 @@ WriteQuicDataResult writeQuicDataExceptCryptoStreamToSocket(
       Clock::now());
 }
 
-uint64_t writeZeroRttDataToSocket(
+folly::Expected<uint64_t, QuicError> writeZeroRttDataToSocket(
     QuicAsyncUDPSocket& socket,
     QuicConnectionStateBase& connection,
     const ConnectionId& srcConnId,
@@ -1223,21 +1266,26 @@ uint64_t writeZeroRttDataToSocket(
                     .blockedFrames()
                     .simpleFrames())
           .build();
-  auto written = writeConnectionDataToSocket(
-                     socket,
-                     connection,
-                     srcConnId,
-                     dstConnId,
-                     std::move(builder),
-                     LongHeader::typeToPacketNumberSpace(type),
-                     scheduler,
-                     congestionControlWritableBytes,
-                     packetLimit,
-                     aead,
-                     headerCipher,
-                     version,
-                     Clock::now())
-                     .packetsWritten;
+  auto writeResult = writeConnectionDataToSocket(
+      socket,
+      connection,
+      srcConnId,
+      dstConnId,
+      std::move(builder),
+      LongHeader::typeToPacketNumberSpace(type),
+      scheduler,
+      congestionControlWritableBytes,
+      packetLimit,
+      aead,
+      headerCipher,
+      version,
+      Clock::now());
+
+  if (writeResult.hasError()) {
+    return folly::makeUnexpected(writeResult.error());
+  }
+
+  auto written = writeResult->packetsWritten;
   VLOG_IF(10, written > 0) << nodeToString(connection.nodeType)
                            << " written zero rtt data, packets=" << written
                            << " " << connection;
@@ -1257,52 +1305,88 @@ void writeCloseCommon(
   PacketNumberSpace pnSpace = header.getPacketNumberSpace();
   HeaderForm headerForm = header.getHeaderForm();
   PacketNum packetNum = header.getPacketSequenceNum();
-  // TODO: This too needs to be switchable between regular and inplace builder.
-  RegularQuicPacketBuilder packetBuilder(
-      kDefaultUDPSendPacketLen,
-      std::move(header),
+
+  // Create a buffer onto which we write the connection close.
+  BufAccessor bufAccessor(connection.udpSendPacketLen);
+  InplaceQuicPacketBuilder packetBuilder(
+      bufAccessor,
+      connection.udpSendPacketLen,
+      header,
       getAckState(connection, pnSpace).largestAckedByPeer.value_or(0));
-  packetBuilder.encodePacketHeader();
+  auto encodeResult = packetBuilder.encodePacketHeader();
+  if (encodeResult.hasError()) {
+    LOG(ERROR) << "Error encoding packet header: "
+               << encodeResult.error().message;
+    return;
+  }
   packetBuilder.accountForCipherOverhead(aead.getCipherOverhead());
   size_t written = 0;
   if (!closeDetails) {
-    written = writeFrame(
+    auto writeResult = writeFrame(
         ConnectionCloseFrame(
             QuicErrorCode(TransportErrorCode::NO_ERROR),
             std::string("No error")),
         packetBuilder);
+    if (writeResult.hasError()) {
+      LOG(ERROR) << "Error writing frame: " << writeResult.error().message;
+      return;
+    }
+    written = *writeResult;
   } else {
     switch (closeDetails->code.type()) {
-      case QuicErrorCode::Type::ApplicationErrorCode:
-        written = writeFrame(
+      case QuicErrorCode::Type::ApplicationErrorCode: {
+        auto writeResult = writeFrame(
             ConnectionCloseFrame(
                 QuicErrorCode(*closeDetails->code.asApplicationErrorCode()),
                 closeDetails->message,
                 quic::FrameType::CONNECTION_CLOSE_APP_ERR),
             packetBuilder);
+        if (writeResult.hasError()) {
+          LOG(ERROR) << "Error writing frame: " << writeResult.error().message;
+          return;
+        }
+        written = *writeResult;
         break;
-      case QuicErrorCode::Type::TransportErrorCode:
-        written = writeFrame(
+      }
+      case QuicErrorCode::Type::TransportErrorCode: {
+        auto writeResult = writeFrame(
             ConnectionCloseFrame(
                 QuicErrorCode(*closeDetails->code.asTransportErrorCode()),
                 closeDetails->message,
                 quic::FrameType::CONNECTION_CLOSE),
             packetBuilder);
+        if (writeResult.hasError()) {
+          LOG(ERROR) << "Error writing frame: " << writeResult.error().message;
+          return;
+        }
+        written = *writeResult;
         break;
-      case QuicErrorCode::Type::LocalErrorCode:
-        written = writeFrame(
+      }
+      case QuicErrorCode::Type::LocalErrorCode: {
+        auto writeResult = writeFrame(
             ConnectionCloseFrame(
                 QuicErrorCode(TransportErrorCode::INTERNAL_ERROR),
                 std::string("Internal error"),
                 quic::FrameType::CONNECTION_CLOSE),
             packetBuilder);
+        if (writeResult.hasError()) {
+          LOG(ERROR) << "Error writing frame: " << writeResult.error().message;
+          return;
+        }
+        written = *writeResult;
         break;
+      }
     }
   }
   if (pnSpace == PacketNumberSpace::Initial &&
       connection.nodeType == QuicNodeType::Client) {
     while (packetBuilder.remainingSpaceInPkt() > 0) {
-      writeFrame(PaddingFrame(), packetBuilder);
+      auto paddingResult = writeFrame(PaddingFrame(), packetBuilder);
+      if (paddingResult.hasError()) {
+        LOG(ERROR) << "Error writing padding frame: "
+                   << paddingResult.error().message;
+        return;
+      }
     }
   }
   if (written == 0) {
@@ -1310,8 +1394,6 @@ void writeCloseCommon(
     return;
   }
   auto packet = std::move(packetBuilder).buildPacket();
-  packet.header.coalesce();
-  packet.body.reserve(0, aead.getCipherOverhead());
   CHECK_GE(packet.body.tailroom(), aead.getCipherOverhead());
   auto bufUniquePtr = packet.body.clone();
   bufUniquePtr =
@@ -1488,7 +1570,7 @@ void encryptPacketHeader(
  *   network, since currently there is no way to rewind scheduler and connection
  *   state after the packets have been written to a batch.
  */
-WriteQuicDataResult writeConnectionDataToSocket(
+folly::Expected<WriteQuicDataResult, QuicError> writeConnectionDataToSocket(
     QuicAsyncUDPSocket& sock,
     QuicConnectionStateBase& connection,
     const ConnectionId& srcConnId,
@@ -1517,7 +1599,7 @@ WriteQuicDataResult writeConnectionDataToSocket(
     if (connection.loopDetectorCallback) {
       connection.writeDebugState.noWriteReason = NoWriteReason::EMPTY_SCHEDULER;
     }
-    return {0, 0, 0};
+    return WriteQuicDataResult{0, 0, 0};
   }
 
   VLOG(10) << nodeToString(connection.nodeType)
@@ -1568,7 +1650,7 @@ WriteQuicDataResult writeConnectionDataToSocket(
     if (!flushSuccess) {
       // Could not flush retried data. Return empty write result and wait for
       // next retry.
-      return {0, 0, 0};
+      return WriteQuicDataResult{0, 0, 0};
     }
   }
 
@@ -1627,42 +1709,49 @@ WriteQuicDataResult writeConnectionDataToSocket(
         aead,
         headerCipher);
 
-    if (!ret.buildSuccess) {
+    // This is a fatal error vs. a build error.
+    if (ret.hasError()) {
+      return folly::makeUnexpected(ret.error());
+    }
+    if (!ret->buildSuccess) {
       // If we're returning because we couldn't schedule more packets,
       // make sure we flush the buffer in this function.
       ioBufBatch.flush();
       updateErrnoCount(connection, ioBufBatch);
-      return {ioBufBatch.getPktSent(), 0, bytesWritten};
+      return WriteQuicDataResult{ioBufBatch.getPktSent(), 0, bytesWritten};
     }
     // If we build a packet, we updateConnection(), even if write might have
     // been failed. Because if it builds, a lot of states need to be updated no
     // matter the write result. We are basically treating this case as if we
     // pretend write was also successful but packet is lost somewhere in the
     // network.
-    bytesWritten += ret.encodedSize;
-    if (ret.result && ret.result->shortHeaderPadding > 0) {
+    bytesWritten += ret->encodedSize;
+    if (ret->result && ret->result->shortHeaderPadding > 0) {
       shortHeaderPaddingCount++;
-      shortHeaderPadding += ret.result->shortHeaderPadding;
+      shortHeaderPadding += ret->result->shortHeaderPadding;
     }
 
-    auto& result = ret.result;
-    updateConnection(
+    auto& result = ret->result;
+    auto updateConnResult = updateConnection(
         connection,
         std::move(result->clonedPacketIdentifier),
         std::move(result->packet->packet),
         sentTime,
-        folly::to<uint32_t>(ret.encodedSize),
-        folly::to<uint32_t>(ret.encodedBodySize),
+        folly::to<uint32_t>(ret->encodedSize),
+        folly::to<uint32_t>(ret->encodedBodySize),
         false /* isDSRPacket */);
+    if (updateConnResult.hasError()) {
+      return folly::makeUnexpected(updateConnResult.error());
+    }
 
     // if ioBufBatch.write returns false
     // it is because a flush() call failed
-    if (!ret.writeSuccess) {
+    if (!ret->writeSuccess) {
       if (connection.loopDetectorCallback) {
         connection.writeDebugState.noWriteReason =
             NoWriteReason::SOCKET_FAILURE;
       }
-      return {ioBufBatch.getPktSent(), 0, bytesWritten};
+      return WriteQuicDataResult{ioBufBatch.getPktSent(), 0, bytesWritten};
     }
 
     if ((connection.transportSettings.batchingMode ==
@@ -1688,10 +1777,10 @@ WriteQuicDataResult writeConnectionDataToSocket(
         connection.bufAccessor->length() == 0 &&
         connection.bufAccessor->headroom() == 0);
   }
-  return {ioBufBatch.getPktSent(), 0, bytesWritten};
+  return WriteQuicDataResult{ioBufBatch.getPktSent(), 0, bytesWritten};
 }
 
-WriteQuicDataResult writeProbingDataToSocket(
+folly::Expected<WriteQuicDataResult, QuicError> writeProbingDataToSocket(
     QuicAsyncUDPSocket& sock,
     QuicConnectionStateBase& connection,
     const ConnectionId& srcConnId,
@@ -1736,8 +1825,11 @@ WriteQuicDataResult writeProbingDataToSocket(
       version,
       writeLoopBeginTime,
       token);
-  auto probesWritten = cloningResult.packetsWritten;
-  auto bytesWritten = cloningResult.bytesWritten;
+  if (cloningResult.hasError()) {
+    return folly::makeUnexpected(cloningResult.error());
+  }
+  auto probesWritten = cloningResult->packetsWritten;
+  auto bytesWritten = cloningResult->bytesWritten;
   if (probesWritten < probesToSend) {
     // If we can use an IMMEDIATE_ACK, that's better than a PING.
     auto probeSchedulerBuilder = FrameScheduler::Builder(
@@ -1769,13 +1861,16 @@ WriteQuicDataResult writeProbingDataToSocket(
         headerCipher,
         version,
         writeLoopBeginTime);
-    probesWritten += probingResult.packetsWritten;
-    bytesWritten += probingResult.bytesWritten;
+    if (probingResult.hasError()) {
+      return folly::makeUnexpected(probingResult.error());
+    }
+    probesWritten += probingResult->packetsWritten;
+    bytesWritten += probingResult->bytesWritten;
   }
   VLOG_IF(10, probesWritten > 0)
       << nodeToString(connection.nodeType)
       << " writing probes using scheduler=CloningScheduler " << connection;
-  return {0, probesWritten, bytesWritten};
+  return WriteQuicDataResult{0, probesWritten, bytesWritten};
 }
 
 WriteDataReason shouldWriteData(/*const*/ QuicConnectionStateBase& conn) {
@@ -1918,12 +2013,13 @@ void implicitAckCryptoStream(
   implicitAck.largestAcked = ackBlocks.back().end;
   implicitAck.ackBlocks.emplace_back(
       ackBlocks.front().start, implicitAck.largestAcked);
-  processAckFrame(
+  auto result = processAckFrame(
       conn,
       packetNumSpace,
       implicitAck,
       [](const auto&) {
         // ackedPacketVisitor. No action needed.
+        return folly::unit;
       },
       [&](auto&, auto& packetFrame) {
         switch (packetFrame.type()) {
@@ -1944,13 +2040,17 @@ void implicitAckCryptoStream(
             // our outstanding packets.
           }
         }
+        return folly::unit;
       },
       // We shouldn't mark anything as lost from the implicit ACK, as it should
       // be ACKing the entire rangee.
       [](auto&, auto&, auto) {
         LOG(FATAL) << "Got loss from implicit crypto ACK.";
+        return folly::unit;
       },
       implicitAckTime);
+  // TODO handle error
+  CHECK(result.hasValue());
   // Clear our the loss buffer explicitly. The implicit ACK itself will not
   // remove data already in the loss buffer.
   auto cryptoStream = getCryptoStream(*conn.cryptoState, encryptionLevel);
@@ -2062,7 +2162,7 @@ void maybeInitiateKeyUpdate(QuicConnectionStateBase& conn) {
   }
 }
 
-void maybeVerifyPendingKeyUpdate(
+folly::Expected<folly::Unit, QuicError> maybeVerifyPendingKeyUpdate(
     QuicConnectionStateBase& conn,
     const OutstandingPacketWrapper& outstandingPacket,
     const RegularQuicPacket& ackPacket) {
@@ -2070,7 +2170,7 @@ void maybeVerifyPendingKeyUpdate(
             outstandingPacket.packet.header.getProtectionType()) ==
         EncryptionLevel::AppData)) {
     // This is not an app data packet. We can't have initiated a key update yet.
-    return;
+    return folly::unit;
   }
 
   if (conn.oneRttWritePendingVerificationPacketNumber &&
@@ -2083,11 +2183,12 @@ void maybeVerifyPendingKeyUpdate(
       conn.oneRttWritePendingVerificationPacketNumber.reset();
       conn.oneRttWritePendingVerification = false;
     } else {
-      throw QuicTransportException(
-          "Packet with key update was acked in the wrong phase",
-          TransportErrorCode::CRYPTO_ERROR);
+      return folly::makeUnexpected(QuicError(
+          TransportErrorCode::CRYPTO_ERROR,
+          "Packet with key update was acked in the wrong phase"));
     }
   }
+  return folly::unit;
 }
 
 // Unfortunate, we should make this more portable.
@@ -2132,4 +2233,35 @@ void maybeScheduleAckForCongestionFeedback(
   }
 }
 
+void updateNegotiatedAckFeatures(QuicConnectionStateBase& conn) {
+  bool isAckReceiveTimestampsSupported =
+      conn.transportSettings.maybeAckReceiveTimestampsConfigSentToPeer &&
+      conn.maybePeerAckReceiveTimestampsConfig;
+
+  uint64_t peerRequestedTimestampsCount =
+      conn.maybePeerAckReceiveTimestampsConfig.has_value()
+      ? conn.maybePeerAckReceiveTimestampsConfig.value()
+            .maxReceiveTimestampsPerAck
+      : 0;
+
+  conn.negotiatedAckReceiveTimestampSupport =
+      isAckReceiveTimestampsSupported && (peerRequestedTimestampsCount > 0);
+
+  conn.negotiatedExtendedAckFeatures = conn.peerAdvertisedExtendedAckFeatures &
+      conn.transportSettings.enableExtendedAckFeatures;
+  // Disable the ECN fields if we are not reading them
+  if (!conn.transportSettings.readEcnOnIngress) {
+    conn.negotiatedExtendedAckFeatures &=
+        ~static_cast<ExtendedAckFeatureMaskType>(
+            ExtendedAckFeatureMask::ECN_COUNTS);
+  }
+  // Disable the receive timestamps fields if we have not regoatiated receive
+  // timestamps support
+  if (!conn.negotiatedAckReceiveTimestampSupport) {
+    conn.negotiatedExtendedAckFeatures &=
+        ~static_cast<ExtendedAckFeatureMaskType>(
+            ExtendedAckFeatureMask::RECEIVE_TIMESTAMPS);
+  }
+}
+
 } // namespace quic
