@@ -14,7 +14,6 @@
 #include <quic/codec/Decode.h>
 #include <quic/common/TimeUtil.h>
 #include <quic/congestion_control/CongestionControllerFactory.h>
-#include <quic/congestion_control/QuicCubic.h>
 #include <quic/flowcontrol/QuicFlowController.h>
 #include <quic/handshake/TransportParameters.h>
 #include <quic/state/QuicStateFunctions.h>
@@ -57,12 +56,14 @@ std::unique_ptr<QuicClientConnectionState> undoAllClientStateForRetry(
   newConn->readCodec->setCodecParameters(CodecParameters(
       conn->peerAckDelayExponent,
       conn->originalVersion.value(),
-      conn->transportSettings.maybeAckReceiveTimestampsConfigSentToPeer));
+      conn->transportSettings.maybeAckReceiveTimestampsConfigSentToPeer,
+      conn->transportSettings.advertisedExtendedAckFeatures));
   newConn->earlyDataAppParamsValidator =
       std::move(conn->earlyDataAppParamsValidator);
   newConn->earlyDataAppParamsGetter = std::move(conn->earlyDataAppParamsGetter);
   newConn->happyEyeballsState = std::move(conn->happyEyeballsState);
   newConn->flowControlState = std::move(conn->flowControlState);
+  newConn->bufAccessor = conn->bufAccessor;
   newConn->pendingOneRttData.reserve(
       newConn->transportSettings.maxPacketsToBuffer);
   if (conn->congestionControllerFactory) {
@@ -94,12 +95,15 @@ std::unique_ptr<QuicClientConnectionState> undoAllClientStateForRetry(
       newConn->transportSettings,
       std::move(*conn->streamManager));
 
-  markZeroRttPacketsLost(*newConn, markPacketLoss);
+  auto result = markZeroRttPacketsLost(*newConn, markPacketLoss);
+  if (result.hasError()) {
+    LOG(FATAL) << "error marking packets lost";
+  }
 
   return newConn;
 }
 
-void processServerInitialParams(
+folly::Expected<folly::Unit, QuicError> processServerInitialParams(
     QuicClientConnectionState& conn,
     const ServerTransportParameters& serverParams,
     PacketNum packetNum) {
@@ -151,6 +155,25 @@ void processServerInitialParams(
       static_cast<TransportParameterId>(
           TransportParameterId::knob_frames_supported),
       serverParams.parameters);
+  auto extendedAckFeatures = getIntegerParameter(
+      static_cast<TransportParameterId>(
+          TransportParameterId::extended_ack_features),
+      serverParams.parameters);
+
+  auto reliableResetTpIter = findParameter(
+      serverParams.parameters,
+      static_cast<TransportParameterId>(
+          TransportParameterId::reliable_stream_reset));
+  if (reliableResetTpIter != serverParams.parameters.end()) {
+    if (!reliableResetTpIter->value->empty()) {
+      return folly::makeUnexpected(QuicError(
+          TransportErrorCode::TRANSPORT_PARAMETER_ERROR,
+          "Reliable reset transport parameter must be empty"));
+    }
+    conn.peerAdvertisedReliableStreamResetSupport = true;
+  } else {
+    conn.peerAdvertisedReliableStreamResetSupport = false;
+  }
   if (conn.version == QuicVersion::QUIC_V1 ||
       conn.version == QuicVersion::QUIC_V1_ALIAS ||
       conn.version == QuicVersion::QUIC_V1_ALIAS2) {
@@ -165,9 +188,9 @@ void processServerInitialParams(
             conn.readCodec->getServerConnectionId() ||
         originalDestinationConnId.value() !=
             conn.originalDestinationConnectionId) {
-      throw QuicTransportException(
-          "Initial CID does not match.",
-          TransportErrorCode::TRANSPORT_PARAMETER_ERROR);
+      return folly::makeUnexpected(QuicError(
+          TransportErrorCode::TRANSPORT_PARAMETER_ERROR,
+          "Initial CID does not match."));
     }
   }
 
@@ -177,11 +200,11 @@ void processServerInitialParams(
     packetSize = kDefaultUDPSendPacketLen;
   }
   if (*packetSize < kMinMaxUDPPayload) {
-    throw QuicTransportException(
+    return folly::makeUnexpected(QuicError(
+        TransportErrorCode::TRANSPORT_PARAMETER_ERROR,
         folly::to<std::string>(
             "Max packet size too small. received max_packetSize = ",
-            *packetSize),
-        TransportErrorCode::TRANSPORT_PARAMETER_ERROR);
+            *packetSize)));
   }
 
   VLOG(10) << "Client advertised flow control ";
@@ -199,18 +222,24 @@ void processServerInitialParams(
       maxStreamDataBidiRemote.value_or(0);
   conn.flowControlState.peerAdvertisedInitialMaxStreamOffsetUni =
       maxStreamDataUni.value_or(0);
-  conn.streamManager->setMaxLocalBidirectionalStreams(
+  auto resultBidi = conn.streamManager->setMaxLocalBidirectionalStreams(
       maxStreamsBidi.value_or(0));
+  if (resultBidi.hasError()) {
+    return folly::makeUnexpected(resultBidi.error());
+  }
   conn.peerAdvertisedInitialMaxStreamsBidi = maxStreamsBidi.value_or(0);
-  conn.streamManager->setMaxLocalUnidirectionalStreams(
+  auto resultUni = conn.streamManager->setMaxLocalUnidirectionalStreams(
       maxStreamsUni.value_or(0));
+  if (resultUni.hasError()) {
+    return folly::makeUnexpected(resultUni.error());
+  }
   conn.peerAdvertisedInitialMaxStreamsUni = maxStreamsUni.value_or(0);
   conn.peerIdleTimeout = std::chrono::milliseconds(idleTimeout.value_or(0));
   conn.peerIdleTimeout = timeMin(conn.peerIdleTimeout, kMaxIdleTimeout);
   if (ackDelayExponent && *ackDelayExponent > kMaxAckDelayExponent) {
-    throw QuicTransportException(
-        "ack_delay_exponent too large",
-        TransportErrorCode::TRANSPORT_PARAMETER_ERROR);
+    return folly::makeUnexpected(QuicError(
+        TransportErrorCode::TRANSPORT_PARAMETER_ERROR,
+        "ack_delay_exponent too large"));
   }
   conn.peerAckDelayExponent =
       ackDelayExponent.value_or(kDefaultAckDelayExponent);
@@ -244,9 +273,9 @@ void processServerInitialParams(
   if (maxDatagramFrameSize.has_value()) {
     if (maxDatagramFrameSize.value() > 0 &&
         maxDatagramFrameSize.value() <= kMaxDatagramPacketOverhead) {
-      throw QuicTransportException(
-          "max_datagram_frame_size too small",
-          TransportErrorCode::TRANSPORT_PARAMETER_ERROR);
+      return folly::makeUnexpected(QuicError(
+          TransportErrorCode::TRANSPORT_PARAMETER_ERROR,
+          "max_datagram_frame_size too small"));
     }
     conn.datagramState.maxWriteFrameSize = maxDatagramFrameSize.value();
   }
@@ -271,6 +300,9 @@ void processServerInitialParams(
   }
 
   conn.peerAdvertisedKnobFrameSupport = knobFrameSupported.value_or(0) > 0;
+  conn.peerAdvertisedExtendedAckFeatures = extendedAckFeatures.value_or(0);
+
+  return folly::unit;
 }
 
 void cacheServerInitialParams(
@@ -284,7 +316,9 @@ void cacheServerInitialParams(
     bool peerAdvertisedKnobFrameSupport,
     bool peerAdvertisedAckReceiveTimestampsEnabled,
     uint64_t peerAdvertisedMaxReceiveTimestampsPerAck,
-    uint64_t peerAdvertisedReceiveTimestampsExponent) {
+    uint64_t peerAdvertisedReceiveTimestampsExponent,
+    bool peerAdvertisedReliableStreamResetSupport,
+    ExtendedAckFeatureMaskType peerAdvertisedExtendedAckFeatures) {
   conn.serverInitialParamsSet_ = true;
   conn.peerAdvertisedInitialMaxData = peerAdvertisedInitialMaxData;
   conn.peerAdvertisedInitialMaxStreamDataBidiLocal =
@@ -297,6 +331,8 @@ void cacheServerInitialParams(
       peerAdvertisedInitialMaxStreamsBidi;
   conn.peerAdvertisedInitialMaxStreamsUni = peerAdvertisedInitialMaxStreamUni;
   conn.peerAdvertisedKnobFrameSupport = peerAdvertisedKnobFrameSupport;
+  conn.peerAdvertisedReliableStreamResetSupport =
+      peerAdvertisedReliableStreamResetSupport;
 
   if (peerAdvertisedAckReceiveTimestampsEnabled) {
     conn.maybePeerAckReceiveTimestampsConfig.assign(
@@ -310,6 +346,7 @@ void cacheServerInitialParams(
   } else {
     conn.maybePeerAckReceiveTimestampsConfig.clear();
   }
+  conn.peerAdvertisedExtendedAckFeatures = peerAdvertisedExtendedAckFeatures;
 }
 
 CachedServerTransportParameters getServerCachedTransportParameters(
@@ -334,17 +371,21 @@ CachedServerTransportParameters getServerCachedTransportParameters(
   transportParams.knobFrameSupport = conn.peerAdvertisedKnobFrameSupport;
   transportParams.ackReceiveTimestampsEnabled =
       conn.maybePeerAckReceiveTimestampsConfig.has_value();
+  transportParams.reliableStreamResetSupport =
+      conn.peerAdvertisedReliableStreamResetSupport;
   if (conn.maybePeerAckReceiveTimestampsConfig) {
     transportParams.maxReceiveTimestampsPerAck =
         conn.maybePeerAckReceiveTimestampsConfig->maxReceiveTimestampsPerAck;
     transportParams.receiveTimestampsExponent =
         conn.maybePeerAckReceiveTimestampsConfig->receiveTimestampsExponent;
   }
+  transportParams.extendedAckFeatures = conn.peerAdvertisedExtendedAckFeatures;
 
   return transportParams;
 }
 
-void updateTransportParamsFromCachedEarlyParams(
+folly::Expected<folly::Unit, QuicError>
+updateTransportParamsFromCachedEarlyParams(
     QuicClientConnectionState& conn,
     const CachedServerTransportParameters& transportParams) {
   conn.peerIdleTimeout = std::chrono::milliseconds(transportParams.idleTimeout);
@@ -356,11 +397,19 @@ void updateTransportParamsFromCachedEarlyParams(
       transportParams.initialMaxStreamDataBidiRemote;
   conn.flowControlState.peerAdvertisedInitialMaxStreamOffsetUni =
       transportParams.initialMaxStreamDataUni;
-  conn.streamManager->setMaxLocalBidirectionalStreams(
+  auto resultBidi = conn.streamManager->setMaxLocalBidirectionalStreams(
       transportParams.initialMaxStreamsBidi);
-  conn.streamManager->setMaxLocalUnidirectionalStreams(
+  if (resultBidi.hasError()) {
+    return folly::makeUnexpected(resultBidi.error());
+  }
+  auto resultUni = conn.streamManager->setMaxLocalUnidirectionalStreams(
       transportParams.initialMaxStreamsUni);
+  if (resultUni.hasError()) {
+    return folly::makeUnexpected(resultUni.error());
+  }
   conn.peerAdvertisedKnobFrameSupport = transportParams.knobFrameSupport;
+  conn.peerAdvertisedReliableStreamResetSupport =
+      transportParams.reliableStreamResetSupport;
   if (transportParams.ackReceiveTimestampsEnabled) {
     conn.maybePeerAckReceiveTimestampsConfig.assign(
         {std::min(
@@ -373,5 +422,7 @@ void updateTransportParamsFromCachedEarlyParams(
   } else {
     conn.maybePeerAckReceiveTimestampsConfig.clear();
   }
+  conn.peerAdvertisedExtendedAckFeatures = transportParams.extendedAckFeatures;
+  return folly::unit;
 }
 } // namespace quic
