@@ -85,6 +85,19 @@ folly::Expected<ParsedLongHeader, TransportErrorCode> tryParseLongHeader(
   return std::move(parsedLongHeader.value());
 }
 
+static PacketDropReason getDecryptErrorReason(ProtectionType protectionType) {
+  switch (protectionType) {
+    case ProtectionType::Initial:
+      return PacketDropReason::DECRYPTION_ERROR_INITIAL;
+    case ProtectionType::Handshake:
+      return PacketDropReason::DECRYPTION_ERROR_HANDSHAKE;
+    case ProtectionType::ZeroRtt:
+      return PacketDropReason::DECRYPTION_ERROR_0RTT;
+    default:
+      return PacketDropReason::DECRYPTION_ERROR;
+  }
+}
+
 CodecResult QuicReadCodec::parseLongHeaderPacket(
     BufQueue& queue,
     const AckStates& ackStates) {
@@ -236,7 +249,7 @@ CodecResult QuicReadCodec::parseLongHeaderPacket(
             << " packetNumLen=" << parsePacketNumberLength(initialByte)
             << " protectionType=" << toString(protectionType) << " "
             << connIdToHex();
-    return CodecResult(Nothing());
+    return CodecResult(Nothing(getDecryptErrorReason(protectionType)));
   }
   decrypted = std::move(*decryptAttempt);
 
@@ -245,8 +258,14 @@ CodecResult QuicReadCodec::parseLongHeaderPacket(
     decrypted = folly::IOBuf::create(0);
   }
 
-  return decodeRegularPacket(
-      std::move(longHeader), params_, std::move(decrypted));
+  auto packetRes =
+      decodeRegularPacket(std::move(longHeader), params_, std::move(decrypted));
+
+  if (!packetRes.hasValue()) {
+    return CodecResult(CodecError(std::move(packetRes.error())));
+  }
+
+  return CodecResult(std::move(*packetRes));
 }
 
 CodecResult QuicReadCodec::tryParseShortHeaderPacket(
@@ -347,7 +366,7 @@ CodecResult QuicReadCodec::tryParseShortHeaderPacket(
     VLOG(10) << "Unable to decrypt packet=" << packetNum.first
              << " protectionType=" << (int)protectionType << " "
              << connIdToHex();
-    return CodecResult(Nothing());
+    return CodecResult(Nothing(PacketDropReason::DECRYPTION_ERROR));
   }
   decrypted = std::move(*decryptAttempt);
   if (!decrypted) {
@@ -378,8 +397,13 @@ CodecResult QuicReadCodec::tryParseShortHeaderPacket(
   // around avoids the timing signals mentioned in the spec, but we could also
   // drop it after 3 * PTO.
 
-  return decodeRegularPacket(
+  auto packetRes = decodeRegularPacket(
       std::move(*shortHeader), params_, std::move(decrypted));
+
+  if (!packetRes.hasValue()) {
+    return CodecResult(CodecError(std::move(packetRes.error())));
+  }
+  return CodecResult(std::move(*packetRes));
 }
 
 CodecResult QuicReadCodec::parsePacket(
@@ -418,9 +442,9 @@ CodecResult QuicReadCodec::parsePacket(
       const uint8_t* tokenSource =
           data->data() + (dataLength - sizeof(StatelessResetToken));
       if (!cryptoEqual_) {
-        throw QuicInternalException(
-            "crypto constant time comparison function is not set.",
-            LocalErrorCode::INTERNAL_ERROR);
+        return CodecResult(CodecError(QuicError(
+            QuicErrorCode(LocalErrorCode::INTERNAL_ERROR),
+            "crypto constant time comparison function is not set.")));
       }
       // Only allocate & copy the token if it matches the token we have
       if (cryptoEqual_(
@@ -498,10 +522,8 @@ void QuicReadCodec::setNextOneRttReadCipher(
 
 void QuicReadCodec::setZeroRttReadCipher(
     std::unique_ptr<Aead> zeroRttReadCipher) {
-  if (nodeType_ == QuicNodeType::Client) {
-    throw QuicTransportException(
-        "Invalid cipher", TransportErrorCode::INTERNAL_ERROR);
-  }
+  CHECK(nodeType_ == QuicNodeType::Server)
+      << "Setting zero rtt read cipher on client.";
   zeroRttReadCipher_ = std::move(zeroRttReadCipher);
 }
 
@@ -627,8 +649,21 @@ CodecResult::CodecResult(RetryPacket&& retryPacketIn)
   new (&retry) RetryPacket(std::move(retryPacketIn));
 }
 
-CodecResult::CodecResult(Nothing&&) : type_(CodecResult::Type::NOTHING) {
-  new (&none) Nothing();
+CodecResult::CodecResult(Nothing&& nothing)
+    : type_(CodecResult::Type::NOTHING) {
+  new (&none) Nothing(std::move(nothing));
+}
+
+CodecResult::CodecResult(CodecError&& codecErrorIn)
+    : type_(CodecResult::Type::CODEC_ERROR) {
+  new (&error) CodecError(std::move(codecErrorIn));
+}
+
+CodecError* CodecResult::codecError() {
+  if (type_ == CodecResult::Type::CODEC_ERROR) {
+    return &error;
+  }
+  return nullptr;
 }
 
 void CodecResult::destroyCodecResult() {
@@ -648,6 +683,9 @@ void CodecResult::destroyCodecResult() {
     case CodecResult::Type::NOTHING:
       none.~Nothing();
       break;
+    case CodecResult::Type::CODEC_ERROR:
+      error.~CodecError();
+      break;
   }
 }
 
@@ -672,6 +710,9 @@ CodecResult::CodecResult(CodecResult&& other) noexcept {
     case CodecResult::Type::NOTHING:
       new (&none) Nothing(std::move(other.none));
       break;
+    case CodecResult::Type::CODEC_ERROR:
+      new (&error) CodecError(std::move(other.error));
+      break;
   }
   type_ = other.type_;
 }
@@ -694,6 +735,9 @@ CodecResult& CodecResult::operator=(CodecResult&& other) noexcept {
     case CodecResult::Type::NOTHING:
       new (&none) Nothing(std::move(other.none));
       break;
+    case CodecResult::Type::CODEC_ERROR:
+      new (&error) CodecError(std::move(other.error));
+      break;
   }
   type_ = other.type_;
   return *this;
