@@ -48,7 +48,8 @@ std::unique_ptr<folly::IOBuf> createAckFrame(
     std::vector<NormalizedAckBlock> ackBlocks = {},
     bool useRealValuesForLargestAcked = false,
     bool useRealValuesForAckDelay = false,
-    bool addEcnCounts = false) {
+    bool addEcnCounts = false,
+    bool useExtendedAck = false) {
   std::unique_ptr<folly::IOBuf> ackFrame = folly::IOBuf::create(0);
   BufAppender wcursor(ackFrame.get(), 10);
   auto appenderOp = [&](auto val) { wcursor.writeBE(val); };
@@ -76,6 +77,14 @@ std::unique_ptr<folly::IOBuf> createAckFrame(
     ackBlocks[i].gap.encode(appenderOp);
     ackBlocks[i].blockLen.encode(appenderOp);
   }
+  if (useExtendedAck) {
+    // Write extended ack with ECN if enabled.
+    QuicInteger extendedAckFeatures(
+        addEcnCounts ? static_cast<ExtendedAckFeatureMaskType>(
+                           ExtendedAckFeatureMask::ECN_COUNTS)
+                     : 0);
+    extendedAckFeatures.encode(appenderOp);
+  }
   if (addEcnCounts) {
     QuicInteger ect0(1); // ECT-0 count
     QuicInteger ect1(2); // ECT-1 count
@@ -87,6 +96,43 @@ std::unique_ptr<folly::IOBuf> createAckFrame(
   return ackFrame;
 }
 
+std::unique_ptr<folly::IOBuf> createRstStreamFrame(
+    StreamId streamId,
+    ApplicationErrorCode errorCode,
+    uint64_t finalSize,
+    folly::Optional<uint64_t> reliableSize = folly::none) {
+  std::unique_ptr<folly::IOBuf> rstStreamFrame = folly::IOBuf::create(0);
+  BufAppender wcursor(rstStreamFrame.get(), 10);
+  auto appenderOp = [&](auto val) { wcursor.writeBE(val); };
+
+  FrameType frameType =
+      reliableSize ? FrameType::RST_STREAM_AT : FrameType::RST_STREAM;
+
+  // Write the frame type
+  QuicInteger frameTypeQuicInt(static_cast<uint8_t>(frameType));
+  frameTypeQuicInt.encode(appenderOp);
+
+  // Write the stream id
+  QuicInteger streamIdQuicInt(streamId);
+  streamIdQuicInt.encode(appenderOp);
+
+  // Write the error code
+  QuicInteger errorCodeQuicInt(static_cast<uint64_t>(errorCode));
+  errorCodeQuicInt.encode(appenderOp);
+
+  // Write the final size
+  QuicInteger finalSizeQuicInt(finalSize);
+  finalSizeQuicInt.encode(appenderOp);
+
+  if (reliableSize) {
+    // Write the reliable size
+    QuicInteger reliableSizeQuicInt(*reliableSize);
+    reliableSizeQuicInt.encode(appenderOp);
+  }
+
+  return rstStreamFrame;
+}
+
 template <class StreamIdType = StreamId>
 std::unique_ptr<folly::IOBuf> createStreamFrame(
     Optional<QuicInteger> streamId,
@@ -249,10 +295,12 @@ TEST_F(DecodeTest, ValidAckFrame) {
       firstAckBlockLength,
       ackBlocks);
   folly::io::Cursor cursor(result.get());
-  auto ackFrame = decodeAckFrame(
+  auto res = decodeAckFrame(
       cursor,
       makeHeader(),
       CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasValue());
+  auto ackFrame = *res;
   EXPECT_EQ(ackFrame.ackBlocks.size(), 2);
   EXPECT_EQ(ackFrame.largestAcked, 1000);
   // Since 100 is the encoded value, we use the decoded value.
@@ -278,10 +326,12 @@ TEST_F(DecodeTest, AckEcnFrame) {
       false, // useRealValuesForAckDelay
       true); // addEcnCounts
   folly::io::Cursor cursor(result.get());
-  auto ackFrame = decodeAckFrameWithECN(
+  auto res = decodeAckFrameWithECN(
       cursor,
       makeHeader(),
       CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasValue());
+  auto ackFrame = *res->asReadAckFrame();
   EXPECT_EQ(ackFrame.ackBlocks.size(), 2);
   EXPECT_EQ(ackFrame.largestAcked, 1000);
   // Since 100 is the encoded value, we use the decoded value.
@@ -293,6 +343,124 @@ TEST_F(DecodeTest, AckEcnFrame) {
   EXPECT_EQ(ackFrame.ecnCECount, 3);
 }
 
+TEST_F(DecodeTest, AckExtendedFrameWithECN) {
+  QuicInteger largestAcked(1000);
+  QuicInteger ackDelay(100);
+  QuicInteger numAdditionalBlocks(1);
+  QuicInteger firstAckBlockLength(10);
+
+  std::vector<NormalizedAckBlock> ackBlocks;
+  ackBlocks.emplace_back(QuicInteger(10), QuicInteger(10));
+
+  auto result = createAckFrame(
+      largestAcked,
+      ackDelay,
+      numAdditionalBlocks,
+      firstAckBlockLength,
+      ackBlocks,
+      false, // useRealValuesForLargestAcked
+      false, // useRealValuesForAckDelay
+      true, // addEcnCounts
+      true); // useExtendedAck
+  folly::io::Cursor cursor(result.get());
+  auto ackFrameRes = decodeAckExtendedFrame(
+      cursor,
+      makeHeader(),
+      CodecParameters(
+          kDefaultAckDelayExponent,
+          QuicVersion::MVFST,
+          none,
+          static_cast<ExtendedAckFeatureMaskType>(
+              ExtendedAckFeatureMask::ECN_COUNTS)));
+  ASSERT_TRUE(ackFrameRes.hasValue());
+  auto ackFrame = *ackFrameRes;
+  EXPECT_EQ(ackFrame.ackBlocks.size(), 2);
+  EXPECT_EQ(ackFrame.largestAcked, 1000);
+  // Since 100 is the encoded value, we use the decoded value.
+  EXPECT_EQ(ackFrame.ackDelay.count(), 100 << kDefaultAckDelayExponent);
+
+  EXPECT_EQ(ackFrame.frameType, FrameType::ACK_EXTENDED);
+
+  // These values are hardcoded in the createAckFrame function
+  EXPECT_EQ(ackFrame.ecnECT0Count, 1);
+  EXPECT_EQ(ackFrame.ecnECT1Count, 2);
+  EXPECT_EQ(ackFrame.ecnCECount, 3);
+}
+
+TEST_F(DecodeTest, AckExtendedFrameWithNoFeatures) {
+  QuicInteger largestAcked(1000);
+  QuicInteger ackDelay(100);
+  QuicInteger numAdditionalBlocks(1);
+  QuicInteger firstAckBlockLength(10);
+
+  std::vector<NormalizedAckBlock> ackBlocks;
+  ackBlocks.emplace_back(QuicInteger(10), QuicInteger(10));
+
+  auto result = createAckFrame(
+      largestAcked,
+      ackDelay,
+      numAdditionalBlocks,
+      firstAckBlockLength,
+      ackBlocks,
+      false, // useRealValuesForLargestAcked
+      false, // useRealValuesForAckDelay
+      false, // addEcnCounts
+      true); // useExtendedAck
+  folly::io::Cursor cursor(result.get());
+  auto ackFrameRes = decodeAckExtendedFrame(
+      cursor,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  ASSERT_TRUE(ackFrameRes.hasValue());
+  auto ackFrame = *ackFrameRes;
+  EXPECT_EQ(ackFrame.ackBlocks.size(), 2);
+  EXPECT_EQ(ackFrame.largestAcked, 1000);
+  // Since 100 is the encoded value, we use the decoded value.
+  EXPECT_EQ(ackFrame.ackDelay.count(), 100 << kDefaultAckDelayExponent);
+
+  EXPECT_EQ(ackFrame.frameType, FrameType::ACK_EXTENDED);
+
+  EXPECT_EQ(ackFrame.ecnECT0Count, 0);
+  EXPECT_EQ(ackFrame.ecnECT1Count, 0);
+  EXPECT_EQ(ackFrame.ecnCECount, 0);
+}
+
+TEST_F(DecodeTest, AckExtendedFrameThrowsWithUnsupportedFeatures) {
+  QuicInteger largestAcked(1000);
+  QuicInteger ackDelay(100);
+  QuicInteger numAdditionalBlocks(1);
+  QuicInteger firstAckBlockLength(10);
+
+  std::vector<NormalizedAckBlock> ackBlocks;
+  ackBlocks.emplace_back(QuicInteger(10), QuicInteger(10));
+
+  auto result = createAckFrame(
+      largestAcked,
+      ackDelay,
+      numAdditionalBlocks,
+      firstAckBlockLength,
+      ackBlocks,
+      false, // useRealValuesForLargestAcked
+      false, // useRealValuesForAckDelay
+      true, // addEcnCounts
+      true); // useExtendedAck
+  folly::io::Cursor cursor(result.get());
+
+  // Try to decode extended ack with ECN but we only support Timestamps
+  auto decodeResult = decodeAckExtendedFrame(
+      cursor,
+      makeHeader(),
+      CodecParameters(
+          kDefaultAckDelayExponent,
+          QuicVersion::MVFST,
+          none,
+          static_cast<ExtendedAckFeatureMaskType>(
+              ExtendedAckFeatureMask::RECEIVE_TIMESTAMPS)));
+  EXPECT_TRUE(decodeResult.hasError());
+  EXPECT_EQ(
+      decodeResult.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
+}
+
 TEST_F(DecodeTest, AckFrameLargestAckExceedsRange) {
   // An integer larger than the representable range of quic integer.
   QuicInteger largestAcked(std::numeric_limits<uint64_t>::max());
@@ -307,10 +475,12 @@ TEST_F(DecodeTest, AckFrameLargestAckExceedsRange) {
       {},
       true);
   folly::io::Cursor cursor(result.get());
-  auto ackFrame = decodeAckFrame(
+  auto res = decodeAckFrame(
       cursor,
       makeHeader(),
       CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasValue());
+  auto ackFrame = *res;
   // it will interpret this as a 8 byte range with the max value.
   EXPECT_EQ(ackFrame.largestAcked, 4611686018427387903);
 }
@@ -329,12 +499,12 @@ TEST_F(DecodeTest, AckFrameLargestAckInvalid) {
       {},
       true);
   folly::io::Cursor cursor(result.get());
-  EXPECT_THROW(
-      decodeAckFrame(
-          cursor,
-          makeHeader(),
-          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST)),
-      QuicTransportException);
+  auto res = decodeAckFrame(
+      cursor,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, AckFrameDelayEncodingInvalid) {
@@ -352,12 +522,12 @@ TEST_F(DecodeTest, AckFrameDelayEncodingInvalid) {
       false,
       true);
   folly::io::Cursor cursor(result.get());
-  EXPECT_THROW(
-      decodeAckFrame(
-          cursor,
-          makeHeader(),
-          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST)),
-      QuicTransportException);
+  auto res = decodeAckFrame(
+      cursor,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, AckFrameDelayExceedsRange) {
@@ -369,12 +539,12 @@ TEST_F(DecodeTest, AckFrameDelayExceedsRange) {
   auto result = createAckFrame(
       largestAcked, ackDelay, numAdditionalBlocks, firstAckBlockLength);
   folly::io::Cursor cursor(result.get());
-  EXPECT_THROW(
-      decodeAckFrame(
-          cursor,
-          makeHeader(),
-          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST)),
-      QuicTransportException);
+  auto res = decodeAckFrame(
+      cursor,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, AckFrameAdditionalBlocksUnderflow) {
@@ -393,12 +563,12 @@ TEST_F(DecodeTest, AckFrameAdditionalBlocksUnderflow) {
       firstAckBlockLength,
       ackBlocks);
   folly::io::Cursor cursor(result.get());
-  EXPECT_THROW(
-      decodeAckFrame(
-          cursor,
-          makeHeader(),
-          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST)),
-      QuicTransportException);
+  auto res = decodeAckFrame(
+      cursor,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, AckFrameAdditionalBlocksOverflow) {
@@ -419,10 +589,12 @@ TEST_F(DecodeTest, AckFrameAdditionalBlocksOverflow) {
       firstAckBlockLength,
       ackBlocks);
   folly::io::Cursor cursor(result.get());
-  decodeAckFrame(
-      cursor,
-      makeHeader(),
-      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  ASSERT_FALSE(
+      decodeAckFrame(
+          cursor,
+          makeHeader(),
+          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST))
+          .hasError());
 }
 
 TEST_F(DecodeTest, AckFrameMissingFields) {
@@ -439,52 +611,52 @@ TEST_F(DecodeTest, AckFrameMissingFields) {
       largestAcked, none, numAdditionalBlocks, firstAckBlockLength, ackBlocks);
   folly::io::Cursor cursor1(result1.get());
 
-  EXPECT_THROW(
-      decodeAckFrame(
-          cursor1,
-          makeHeader(),
-          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST)),
-      QuicTransportException);
+  auto res = decodeAckFrame(
+      cursor1,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 
   auto result2 = createAckFrame(
       largestAcked, ackDelay, none, firstAckBlockLength, ackBlocks);
   folly::io::Cursor cursor2(result2.get());
-  EXPECT_THROW(
-      decodeAckFrame(
-          cursor2,
-          makeHeader(),
-          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST)),
-      QuicTransportException);
+  res = decodeAckFrame(
+      cursor2,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 
   auto result3 = createAckFrame(
       largestAcked, ackDelay, none, firstAckBlockLength, ackBlocks);
   folly::io::Cursor cursor3(result3.get());
-  EXPECT_THROW(
-      decodeAckFrame(
-          cursor3,
-          makeHeader(),
-          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST)),
-      QuicTransportException);
+  res = decodeAckFrame(
+      cursor3,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 
   auto result4 = createAckFrame(
       largestAcked, ackDelay, numAdditionalBlocks, none, ackBlocks);
   folly::io::Cursor cursor4(result4.get());
-  EXPECT_THROW(
-      decodeAckFrame(
-          cursor4,
-          makeHeader(),
-          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST)),
-      QuicTransportException);
+  res = decodeAckFrame(
+      cursor4,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 
   auto result5 = createAckFrame(
       largestAcked, ackDelay, numAdditionalBlocks, firstAckBlockLength, {});
   folly::io::Cursor cursor5(result5.get());
-  EXPECT_THROW(
-      decodeAckFrame(
-          cursor5,
-          makeHeader(),
-          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST)),
-      QuicTransportException);
+  res = decodeAckFrame(
+      cursor5,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, AckFrameFirstBlockLengthInvalid) {
@@ -496,12 +668,12 @@ TEST_F(DecodeTest, AckFrameFirstBlockLengthInvalid) {
   auto result = createAckFrame(
       largestAcked, ackDelay, numAdditionalBlocks, firstAckBlockLength);
   folly::io::Cursor cursor(result.get());
-  EXPECT_THROW(
-      decodeAckFrame(
-          cursor,
-          makeHeader(),
-          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST)),
-      QuicTransportException);
+  auto res = decodeAckFrame(
+      cursor,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, AckFrameBlockLengthInvalid) {
@@ -521,12 +693,12 @@ TEST_F(DecodeTest, AckFrameBlockLengthInvalid) {
       firstAckBlockLength,
       ackBlocks);
   folly::io::Cursor cursor(result.get());
-  EXPECT_THROW(
-      decodeAckFrame(
-          cursor,
-          makeHeader(),
-          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST)),
-      QuicTransportException);
+  auto res = decodeAckFrame(
+      cursor,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, AckFrameBlockGapInvalid) {
@@ -546,12 +718,12 @@ TEST_F(DecodeTest, AckFrameBlockGapInvalid) {
       firstAckBlockLength,
       ackBlocks);
   folly::io::Cursor cursor(result.get());
-  EXPECT_THROW(
-      decodeAckFrame(
-          cursor,
-          makeHeader(),
-          CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST)),
-      QuicTransportException);
+  auto res = decodeAckFrame(
+      cursor,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, AckFrameBlockLengthZero) {
@@ -573,10 +745,12 @@ TEST_F(DecodeTest, AckFrameBlockLengthZero) {
       ackBlocks);
   folly::io::Cursor cursor(result.get());
 
-  auto readAckFrame = decodeAckFrame(
+  auto res = decodeAckFrame(
       cursor,
       makeHeader(),
       CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(res.hasValue());
+  auto readAckFrame = *res;
   EXPECT_EQ(readAckFrame.ackBlocks[0].endPacket, 1000);
   EXPECT_EQ(readAckFrame.ackBlocks[0].startPacket, 990);
   EXPECT_EQ(readAckFrame.ackBlocks[1].endPacket, 978);
@@ -597,7 +771,9 @@ TEST_F(DecodeTest, StreamDecodeSuccess) {
       streamId, offset, length, folly::IOBuf::copyBuffer("a"));
   BufQueue queue;
   queue.append(streamFrame->clone());
-  auto decodedFrame = decodeStreamFrame(queue, streamType);
+  auto decodedFrameRes = decodeStreamFrame(queue, streamType);
+  ASSERT_TRUE(decodedFrameRes.hasValue());
+  auto decodedFrame = decodedFrameRes.value();
   EXPECT_EQ(decodedFrame.offset, 10);
   EXPECT_EQ(decodedFrame.data->computeChainDataLength(), 1);
   EXPECT_EQ(decodedFrame.streamId, 10);
@@ -612,7 +788,9 @@ TEST_F(DecodeTest, StreamLengthStreamIdInvalid) {
       createStreamFrame<uint8_t>(streamId, none, none, nullptr, true);
   BufQueue queue;
   queue.append(streamFrame->clone());
-  EXPECT_THROW(decodeStreamFrame(queue, streamType), QuicTransportException);
+  auto result = decodeStreamFrame(queue, streamType);
+  EXPECT_TRUE(result.hasError());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, StreamOffsetNotPresent) {
@@ -624,7 +802,9 @@ TEST_F(DecodeTest, StreamOffsetNotPresent) {
       createStreamFrame(streamId, none, length, folly::IOBuf::copyBuffer("a"));
   BufQueue queue;
   queue.append(streamFrame->clone());
-  EXPECT_THROW(decodeStreamFrame(queue, streamType), QuicTransportException);
+  auto result = decodeStreamFrame(queue, streamType);
+  EXPECT_TRUE(result.hasError());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, StreamIncorrectDataLength) {
@@ -637,7 +817,9 @@ TEST_F(DecodeTest, StreamIncorrectDataLength) {
       streamId, offset, length, folly::IOBuf::copyBuffer("a"));
   BufQueue queue;
   queue.append(streamFrame->clone());
-  EXPECT_THROW(decodeStreamFrame(queue, streamType), QuicTransportException);
+  auto result = decodeStreamFrame(queue, streamType);
+  EXPECT_TRUE(result.hasError());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, StreamNoRemainingData) {
@@ -649,7 +831,9 @@ TEST_F(DecodeTest, StreamNoRemainingData) {
 
   const auto streamType =
       StreamTypeField(static_cast<uint8_t>(FrameType::STREAM));
-  EXPECT_THROW(decodeStreamFrame(queue, streamType), QuicTransportException);
+  auto result = decodeStreamFrame(queue, streamType);
+  EXPECT_TRUE(result.hasError());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, DatagramNoRemainingData) {
@@ -660,7 +844,9 @@ TEST_F(DecodeTest, DatagramNoRemainingData) {
   queue.trimStartAtMost(4);
 
   // invalid len
-  EXPECT_THROW(decodeDatagramFrame(queue, true), QuicTransportException);
+  auto result = decodeDatagramFrame(queue, true);
+  EXPECT_TRUE(result.hasError());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 std::unique_ptr<folly::IOBuf> CreateMaxStreamsIdFrame(
@@ -678,12 +864,14 @@ void MaxStreamsIdCheckSuccess(StreamId maxStreamsId) {
   std::unique_ptr<folly::IOBuf> buf = CreateMaxStreamsIdFrame(maxStreamsId);
 
   folly::io::Cursor cursorBiDi(buf.get());
-  MaxStreamsFrame maxStreamsBiDiFrame = decodeBiDiMaxStreamsFrame(cursorBiDi);
-  EXPECT_EQ(maxStreamsBiDiFrame.maxStreams, maxStreamsId);
+  auto maxStreamsBiDiFrameRes = decodeBiDiMaxStreamsFrame(cursorBiDi);
+  ASSERT_TRUE(maxStreamsBiDiFrameRes.hasValue());
+  EXPECT_EQ(maxStreamsBiDiFrameRes->maxStreams, maxStreamsId);
 
   folly::io::Cursor cursorUni(buf.get());
-  MaxStreamsFrame maxStreamsUniFrame = decodeUniMaxStreamsFrame(cursorUni);
-  EXPECT_EQ(maxStreamsUniFrame.maxStreams, maxStreamsId);
+  auto maxStreamsUniFrameRes = decodeUniMaxStreamsFrame(cursorUni);
+  ASSERT_TRUE(maxStreamsUniFrameRes.hasValue());
+  EXPECT_EQ(maxStreamsUniFrameRes->maxStreams, maxStreamsId);
 }
 
 // Uni and BiDi have same max limits so uses single 'frame' to check both.
@@ -691,10 +879,14 @@ void MaxStreamsIdCheckInvalid(StreamId maxStreamsId) {
   std::unique_ptr<folly::IOBuf> buf = CreateMaxStreamsIdFrame(maxStreamsId);
 
   folly::io::Cursor cursorBiDi(buf.get());
-  EXPECT_THROW(decodeBiDiMaxStreamsFrame(cursorBiDi), QuicTransportException);
+  auto bidiResult = decodeBiDiMaxStreamsFrame(cursorBiDi);
+  EXPECT_TRUE(bidiResult.hasError());
+  EXPECT_EQ(bidiResult.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 
   folly::io::Cursor cursorUni(buf.get());
-  EXPECT_THROW(decodeUniMaxStreamsFrame(cursorUni), QuicTransportException);
+  auto uniResult = decodeUniMaxStreamsFrame(cursorUni);
+  EXPECT_TRUE(uniResult.hasError());
+  EXPECT_EQ(uniResult.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, MaxStreamsIdChecks) {
@@ -714,8 +906,8 @@ TEST_F(DecodeTest, CryptoDecodeSuccess) {
       createCryptoFrame(offset, length, folly::IOBuf::copyBuffer("a"));
   folly::io::Cursor cursor(cryptoFrame.get());
   auto decodedFrame = decodeCryptoFrame(cursor);
-  EXPECT_EQ(decodedFrame.offset, 10);
-  EXPECT_EQ(decodedFrame.data->computeChainDataLength(), 1);
+  EXPECT_EQ(decodedFrame->offset, 10);
+  EXPECT_EQ(decodedFrame->data->computeChainDataLength(), 1);
 }
 
 TEST_F(DecodeTest, CryptoOffsetNotPresent) {
@@ -723,14 +915,18 @@ TEST_F(DecodeTest, CryptoOffsetNotPresent) {
   auto cryptoFrame =
       createCryptoFrame(none, length, folly::IOBuf::copyBuffer("a"));
   folly::io::Cursor cursor(cryptoFrame.get());
-  EXPECT_THROW(decodeCryptoFrame(cursor), QuicTransportException);
+  auto result = decodeCryptoFrame(cursor);
+  EXPECT_TRUE(result.hasError());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, CryptoLengthNotPresent) {
   QuicInteger offset(0);
   auto cryptoFrame = createCryptoFrame(offset, none, nullptr);
   folly::io::Cursor cursor(cryptoFrame.get());
-  EXPECT_THROW(decodeCryptoFrame(cursor), QuicTransportException);
+  auto result = decodeCryptoFrame(cursor);
+  EXPECT_TRUE(result.hasError());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, CryptoIncorrectDataLength) {
@@ -739,7 +935,9 @@ TEST_F(DecodeTest, CryptoIncorrectDataLength) {
   auto cryptoFrame =
       createCryptoFrame(offset, length, folly::IOBuf::copyBuffer("a"));
   folly::io::Cursor cursor(cryptoFrame.get());
-  EXPECT_THROW(decodeCryptoFrame(cursor), QuicTransportException);
+  auto result = decodeCryptoFrame(cursor);
+  EXPECT_TRUE(result.hasError());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, PaddingFrameTest) {
@@ -748,14 +946,14 @@ TEST_F(DecodeTest, PaddingFrameTest) {
   memset(buf->writableData(), 0, 1);
 
   folly::io::Cursor cursor(buf.get());
-  decodePaddingFrame(cursor);
+  ASSERT_FALSE(decodePaddingFrame(cursor).hasError());
 }
 
 TEST_F(DecodeTest, PaddingFrameNoBytesTest) {
   auto buf = folly::IOBuf::create(sizeof(UnderlyingFrameType));
 
   folly::io::Cursor cursor(buf.get());
-  decodePaddingFrame(cursor);
+  ASSERT_FALSE(decodePaddingFrame(cursor).hasError());
 }
 
 TEST_F(DecodeTest, DecodeMultiplePaddingInterleavedTest) {
@@ -767,7 +965,7 @@ TEST_F(DecodeTest, DecodeMultiplePaddingInterleavedTest) {
   memset(buf->writableData() + 10, 5, 1);
 
   folly::io::Cursor cursor(buf.get());
-  decodePaddingFrame(cursor);
+  ASSERT_FALSE(decodePaddingFrame(cursor).hasError());
   // If we encountered an interleaved frame, leave the whole thing
   // as is
   EXPECT_EQ(cursor.totalLength(), 11);
@@ -779,7 +977,7 @@ TEST_F(DecodeTest, DecodeMultiplePaddingTest) {
   memset(buf->writableData(), 0, 10);
 
   folly::io::Cursor cursor(buf.get());
-  decodePaddingFrame(cursor);
+  ASSERT_FALSE(decodePaddingFrame(cursor).hasError());
   EXPECT_EQ(cursor.totalLength(), 0);
 }
 
@@ -804,13 +1002,15 @@ TEST_F(DecodeTest, NewTokenDecodeSuccess) {
       createNewTokenFrame(length, folly::IOBuf::copyBuffer("a"));
   folly::io::Cursor cursor(newTokenFrame.get());
   auto decodedFrame = decodeNewTokenFrame(cursor);
-  EXPECT_EQ(decodedFrame.token->computeChainDataLength(), 1);
+  EXPECT_EQ(decodedFrame->token->computeChainDataLength(), 1);
 }
 
 TEST_F(DecodeTest, NewTokenLengthNotPresent) {
   auto newTokenFrame = createNewTokenFrame(none, folly::IOBuf::copyBuffer("a"));
   folly::io::Cursor cursor(newTokenFrame.get());
-  EXPECT_THROW(decodeNewTokenFrame(cursor), QuicTransportException);
+  auto result = decodeNewTokenFrame(cursor);
+  EXPECT_TRUE(result.hasError());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, NewTokenIncorrectDataLength) {
@@ -818,7 +1018,9 @@ TEST_F(DecodeTest, NewTokenIncorrectDataLength) {
   auto newTokenFrame =
       createNewTokenFrame(length, folly::IOBuf::copyBuffer("a"));
   folly::io::Cursor cursor(newTokenFrame.get());
-  EXPECT_THROW(decodeNewTokenFrame(cursor), QuicTransportException);
+  auto result = decodeNewTokenFrame(cursor);
+  EXPECT_TRUE(result.hasError());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(DecodeTest, ParsePlaintextNewToken) {
@@ -885,8 +1087,10 @@ TEST_F(DecodeTest, StreamGroupDecodeSuccess) {
       groupId);
   BufQueue queue;
   queue.append(streamFrame->clone());
-  auto decodedFrame =
+  auto decodedFrameRes =
       decodeStreamFrame(queue, streamType, true /* isGroupFrame */);
+  ASSERT_TRUE(decodedFrameRes.hasValue());
+  auto decodedFrame = decodedFrameRes.value();
   EXPECT_EQ(decodedFrame.offset, 10);
   EXPECT_EQ(decodedFrame.data->computeChainDataLength(), 1);
   EXPECT_EQ(decodedFrame.streamId, 10);
@@ -904,7 +1108,9 @@ TEST_F(DecodeTest, AckFrequencyFrameDecodeValid) {
   ASSERT_NE(ackFrequencyFrame, nullptr);
 
   folly::io::Cursor cursor(ackFrequencyFrame.get());
-  auto decodedFrame = decodeAckFrequencyFrame(cursor);
+  auto res = decodeAckFrequencyFrame(cursor);
+  EXPECT_TRUE(res.hasValue());
+  auto decodedFrame = *res->asAckFrequencyFrame();
   EXPECT_EQ(decodedFrame.sequenceNumber, 1);
   EXPECT_EQ(decodedFrame.packetTolerance, 100);
   EXPECT_EQ(decodedFrame.updateMaxAckDelay, 100000);
@@ -920,7 +1126,63 @@ TEST_F(DecodeTest, AckFrequencyFrameDecodeInvalidReserved) {
   ASSERT_NE(ackFrequencyFrame, nullptr);
 
   folly::io::Cursor cursor(ackFrequencyFrame.get());
-  EXPECT_THROW(decodeAckFrequencyFrame(cursor), QuicTransportException);
+  auto res = decodeAckFrequencyFrame(cursor);
+  EXPECT_TRUE(res.hasError());
+  EXPECT_EQ(res.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
+}
+
+TEST_F(DecodeTest, RstStreamFrame) {
+  auto buf = createRstStreamFrame(0, 0, 10);
+  BufQueue queue(std::move(buf));
+  auto frame = parseFrame(
+      queue,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  auto rstStreamFrame = frame->asRstStreamFrame();
+  EXPECT_EQ(rstStreamFrame->streamId, 0);
+  EXPECT_EQ(rstStreamFrame->errorCode, 0);
+  EXPECT_EQ(rstStreamFrame->finalSize, 10);
+  EXPECT_FALSE(rstStreamFrame->reliableSize.hasValue());
+}
+
+TEST_F(DecodeTest, RstStreamAtFrame) {
+  auto buf = createRstStreamFrame(0, 0, 10, 9);
+  BufQueue queue(std::move(buf));
+  auto frame = parseFrame(
+      queue,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  auto rstStreamFrameRes = frame->asRstStreamFrame();
+  ASSERT_TRUE(rstStreamFrameRes);
+  auto rstStreamFrame = *rstStreamFrameRes;
+  EXPECT_EQ(rstStreamFrame.streamId, 0);
+  EXPECT_EQ(rstStreamFrame.errorCode, 0);
+  EXPECT_EQ(rstStreamFrame.finalSize, 10);
+  EXPECT_EQ(*rstStreamFrame.reliableSize, 9);
+}
+
+TEST_F(DecodeTest, RstStreamAtFrameRelSizeGreaterThanOffset) {
+  auto buf = createRstStreamFrame(0, 0, 10, 11);
+  BufQueue queue(std::move(buf));
+  auto result = parseFrame(
+      queue,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(result.hasError());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
+}
+
+TEST_F(DecodeTest, RstStreamAtTruncated) {
+  auto buf = createRstStreamFrame(0, 0, 10, 9);
+  buf->coalesce();
+  buf->trimEnd(1);
+  BufQueue queue(std::move(buf));
+  auto result = parseFrame(
+      queue,
+      makeHeader(),
+      CodecParameters(kDefaultAckDelayExponent, QuicVersion::MVFST));
+  EXPECT_TRUE(result.hasError());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 } // namespace quic::test
