@@ -14,11 +14,8 @@
 #include <quic/common/test/TestUtils.h>
 #include <quic/fizz/server/handshake/FizzServerQuicHandshakeContext.h>
 #include <quic/server/state/ServerStateMachine.h>
-#include <quic/state/QuicStateFunctions.h>
-#include <quic/state/QuicStreamFunctions.h>
 #include <quic/state/StateData.h>
 #include <quic/state/stream/StreamSendHandlers.h>
-#include <quic/state/stream/StreamStateFunctions.h>
 
 using namespace testing;
 
@@ -47,7 +44,7 @@ TEST_F(QuicPacketRebuilderTest, RebuildEmpty) {
       kDefaultUDPSendPacketLen,
       ShortHeader(ProtectionType::KeyPhaseZero, getTestConnectionId(), 0),
       0 /* largestAcked */);
-  regularBuilder.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder.encodePacketHeader().hasError());
   QuicConnectionStateBase conn(QuicNodeType::Client);
   PacketRebuilder rebuilder(regularBuilder, conn);
   auto packet = std::move(regularBuilder).buildPacket();
@@ -72,10 +69,11 @@ TEST_F(QuicPacketRebuilderTest, RebuildSmallInitial) {
       kDefaultUDPSendPacketLen, std::move(initialHeader2), 0);
 
   PingFrame pingFrame{};
-  writeFrame(pingFrame, regularBuilder1);
+  ASSERT_FALSE(writeFrame(pingFrame, regularBuilder1).hasError());
   MaxStreamsFrame maxStreamsFrame(4321, true);
-  writeFrame(QuicSimpleFrame(maxStreamsFrame), regularBuilder1);
-  regularBuilder1.encodePacketHeader();
+  ASSERT_FALSE(
+      writeFrame(QuicSimpleFrame(maxStreamsFrame), regularBuilder1).hasError());
+  ASSERT_FALSE(regularBuilder1.encodePacketHeader().hasError());
   QuicConnectionStateBase conn(QuicNodeType::Client);
   PacketRebuilder rebuilder(regularBuilder2, conn);
   auto packet = std::move(regularBuilder1).buildPacket();
@@ -83,8 +81,10 @@ TEST_F(QuicPacketRebuilderTest, RebuildSmallInitial) {
   EXPECT_FALSE(packet.header.empty());
   ASSERT_EQ(packet.packet.frames.size(), 2);
   EXPECT_FALSE(packet.body.empty());
-  regularBuilder2.encodePacketHeader();
-  ASSERT_TRUE(rebuilder.rebuildFromPacket(outstanding).has_value());
+  ASSERT_FALSE(regularBuilder2.encodePacketHeader().hasError());
+  auto rebuildResult = rebuilder.rebuildFromPacket(outstanding);
+  ASSERT_FALSE(rebuildResult.hasError());
+  ASSERT_TRUE(rebuildResult.value().hasValue());
   auto rebuilt = std::move(regularBuilder2).buildPacket();
   EXPECT_FALSE(rebuilt.header.empty());
   ASSERT_EQ(rebuilt.packet.frames.size(), 3);
@@ -100,7 +100,7 @@ TEST_F(QuicPacketRebuilderTest, RebuildPacket) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder1(
       kDefaultUDPSendPacketLen, std::move(shortHeader1), 0 /* largestAcked */);
-  regularBuilder1.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder1.encodePacketHeader().hasError());
   // Get a bunch frames
   ConnectionCloseFrame connCloseFrame(
       QuicErrorCode(TransportErrorCode::FRAME_ENCODING_ERROR),
@@ -118,7 +118,8 @@ TEST_F(QuicPacketRebuilderTest, RebuildPacket) {
       .ackDelayExponent = static_cast<uint8_t>(kDefaultAckDelayExponent)};
   QuicServerConnectionState conn(
       FizzServerQuicHandshakeContext::Builder().build());
-  conn.streamManager->setMaxLocalBidirectionalStreams(10);
+  ASSERT_FALSE(
+      conn.streamManager->setMaxLocalBidirectionalStreams(10).hasError());
   auto stream = conn.streamManager->createNextBidirectionalStream().value();
   auto streamId = stream->id;
   auto buf =
@@ -131,10 +132,11 @@ TEST_F(QuicPacketRebuilderTest, RebuildPacket) {
   PingFrame pingFrame{};
   // Write them with a regular builder
   // Write the ACK frame first since it has special rebuilder handling.
-  writeAckFrame(ackMeta, regularBuilder1);
-  writeFrame(connCloseFrame, regularBuilder1);
-  writeFrame(QuicSimpleFrame(maxStreamsFrame), regularBuilder1);
-  writeFrame(pingFrame, regularBuilder1);
+  ASSERT_FALSE(writeAckFrame(ackMeta, regularBuilder1).hasError());
+  ASSERT_FALSE(writeFrame(connCloseFrame, regularBuilder1).hasError());
+  ASSERT_FALSE(
+      writeFrame(QuicSimpleFrame(maxStreamsFrame), regularBuilder1).hasError());
+  ASSERT_FALSE(writeFrame(pingFrame, regularBuilder1).hasError());
   writeStreamFrameHeader(
       regularBuilder1,
       streamId,
@@ -145,10 +147,12 @@ TEST_F(QuicPacketRebuilderTest, RebuildPacket) {
       none /* skipLenHint */);
   writeStreamFrameData(
       regularBuilder1, buf->clone(), buf->computeChainDataLength());
-  writeFrame(maxDataFrame, regularBuilder1);
-  writeFrame(maxStreamDataFrame, regularBuilder1);
-  writeCryptoFrame(
-      cryptoOffset, ChainedByteRangeHead(cryptoBuf), regularBuilder1);
+  ASSERT_FALSE(writeFrame(maxDataFrame, regularBuilder1).hasError());
+  ASSERT_FALSE(writeFrame(maxStreamDataFrame, regularBuilder1).hasError());
+  ASSERT_FALSE(
+      writeCryptoFrame(
+          cryptoOffset, ChainedByteRangeHead(cryptoBuf), regularBuilder1)
+          .hasError());
   auto packet1 = std::move(regularBuilder1).buildPacket();
   ASSERT_EQ(8, packet1.packet.frames.size());
   stream->retransmissionBuffer.emplace(
@@ -171,10 +175,12 @@ TEST_F(QuicPacketRebuilderTest, RebuildPacket) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder2(
       kDefaultUDPSendPacketLen, std::move(shortHeader2), 0 /* largestAcked */);
-  regularBuilder2.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder2.encodePacketHeader().hasError());
   PacketRebuilder rebuilder(regularBuilder2, conn);
   auto outstanding = makeDummyOutstandingPacket(packet1.packet, 1000);
-  EXPECT_TRUE(rebuilder.rebuildFromPacket(outstanding).has_value());
+  auto rebuildResult = rebuilder.rebuildFromPacket(outstanding);
+  ASSERT_FALSE(rebuildResult.hasError());
+  ASSERT_TRUE(rebuildResult.value().hasValue());
   auto packet2 = std::move(regularBuilder2).buildPacket();
   // rebuilder writes frames to regularBuilder2
   EXPECT_EQ(packet1.packet.frames.size(), packet2.packet.frames.size());
@@ -259,10 +265,11 @@ TEST_F(QuicPacketRebuilderTest, RebuildAfterResetStream) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder1(
       kDefaultUDPSendPacketLen, std::move(shortHeader1), 0 /* largestAcked */);
-  regularBuilder1.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder1.encodePacketHeader().hasError());
   QuicServerConnectionState conn(
       FizzServerQuicHandshakeContext::Builder().build());
-  conn.streamManager->setMaxLocalBidirectionalStreams(10);
+  ASSERT_FALSE(
+      conn.streamManager->setMaxLocalBidirectionalStreams(10).hasError());
   auto stream = conn.streamManager->createNextBidirectionalStream().value();
   auto streamId = stream->id;
   auto buf = folly::IOBuf::copyBuffer("A million miles away.");
@@ -280,15 +287,18 @@ TEST_F(QuicPacketRebuilderTest, RebuildAfterResetStream) {
   ASSERT_EQ(1, packet1.packet.frames.size());
 
   // Then we reset the stream
-  sendRstSMHandler(*stream, GenericApplicationErrorCode::UNKNOWN);
+  ASSERT_FALSE(sendRstSMHandler(*stream, GenericApplicationErrorCode::UNKNOWN)
+                   .hasError());
   ShortHeader shortHeader2(
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder2(
       kDefaultUDPSendPacketLen, std::move(shortHeader2), 0 /* largestAcked */);
-  regularBuilder2.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder2.encodePacketHeader().hasError());
   PacketRebuilder rebuilder(regularBuilder2, conn);
   auto outstanding = makeDummyOutstandingPacket(packet1.packet, 1000);
-  EXPECT_FALSE(rebuilder.rebuildFromPacket(outstanding).has_value());
+  auto rebuildResult = rebuilder.rebuildFromPacket(outstanding);
+  ASSERT_FALSE(rebuildResult.hasError());
+  EXPECT_FALSE(rebuildResult.value().hasValue());
 }
 
 TEST_F(QuicPacketRebuilderTest, FinOnlyStreamRebuild) {
@@ -296,10 +306,11 @@ TEST_F(QuicPacketRebuilderTest, FinOnlyStreamRebuild) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder1(
       kDefaultUDPSendPacketLen, std::move(shortHeader1), 0 /* largestAcked */);
-  regularBuilder1.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder1.encodePacketHeader().hasError());
   QuicServerConnectionState conn(
       FizzServerQuicHandshakeContext::Builder().build());
-  conn.streamManager->setMaxLocalBidirectionalStreams(10);
+  ASSERT_FALSE(
+      conn.streamManager->setMaxLocalBidirectionalStreams(10).hasError());
   auto stream = conn.streamManager->createNextBidirectionalStream().value();
   auto streamId = stream->id;
 
@@ -318,10 +329,12 @@ TEST_F(QuicPacketRebuilderTest, FinOnlyStreamRebuild) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder2(
       kDefaultUDPSendPacketLen, std::move(shortHeader2), 0 /* largestAcked */);
-  regularBuilder2.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder2.encodePacketHeader().hasError());
   PacketRebuilder rebuilder(regularBuilder2, conn);
   auto outstanding = makeDummyOutstandingPacket(packet1.packet, 2000);
-  EXPECT_TRUE(rebuilder.rebuildFromPacket(outstanding).has_value());
+  auto rebuildResult = rebuilder.rebuildFromPacket(outstanding);
+  ASSERT_FALSE(rebuildResult.hasError());
+  ASSERT_TRUE(rebuildResult.value().hasValue());
   auto packet2 = std::move(regularBuilder2).buildPacket();
   EXPECT_EQ(packet1.packet.frames.size(), packet2.packet.frames.size());
   EXPECT_TRUE(
@@ -341,11 +354,12 @@ TEST_F(QuicPacketRebuilderTest, RebuildDataStreamAndEmptyCryptoStream) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder1(
       kDefaultUDPSendPacketLen, std::move(shortHeader1), 0 /* largestAcked */);
-  regularBuilder1.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder1.encodePacketHeader().hasError());
   // Get a bunch frames
   QuicServerConnectionState conn(
       FizzServerQuicHandshakeContext::Builder().build());
-  conn.streamManager->setMaxLocalBidirectionalStreams(10);
+  ASSERT_FALSE(
+      conn.streamManager->setMaxLocalBidirectionalStreams(10).hasError());
   auto stream = conn.streamManager->createNextBidirectionalStream().value();
   StreamId streamId = stream->id;
   auto buf =
@@ -364,8 +378,10 @@ TEST_F(QuicPacketRebuilderTest, RebuildDataStreamAndEmptyCryptoStream) {
       none /* skipLenHint */);
   writeStreamFrameData(
       regularBuilder1, buf->clone(), buf->computeChainDataLength());
-  writeCryptoFrame(
-      cryptoOffset, ChainedByteRangeHead(cryptoBuf), regularBuilder1);
+  ASSERT_FALSE(
+      writeCryptoFrame(
+          cryptoOffset, ChainedByteRangeHead(cryptoBuf), regularBuilder1)
+          .hasError());
   auto packet1 = std::move(regularBuilder1).buildPacket();
   ASSERT_EQ(2, packet1.packet.frames.size());
   stream->retransmissionBuffer.emplace(
@@ -381,10 +397,12 @@ TEST_F(QuicPacketRebuilderTest, RebuildDataStreamAndEmptyCryptoStream) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder2(
       kDefaultUDPSendPacketLen, std::move(shortHeader2), 0 /* largestAcked */);
-  regularBuilder2.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder2.encodePacketHeader().hasError());
   PacketRebuilder rebuilder(regularBuilder2, conn);
   auto outstanding = makeDummyOutstandingPacket(packet1.packet, 1000);
-  EXPECT_TRUE(rebuilder.rebuildFromPacket(outstanding).has_value());
+  auto rebuildResult = rebuilder.rebuildFromPacket(outstanding);
+  ASSERT_FALSE(rebuildResult.hasError());
+  ASSERT_TRUE(rebuildResult.value().hasValue());
   auto packet2 = std::move(regularBuilder2).buildPacket();
   // rebuilder writes frames to regularBuilder2
   EXPECT_EQ(packet1.packet.frames.size(), packet2.packet.frames.size() + 1);
@@ -406,7 +424,7 @@ TEST_F(QuicPacketRebuilderTest, CannotRebuildEmptyCryptoStream) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder1(
       kDefaultUDPSendPacketLen, std::move(shortHeader1), 0 /* largestAcked */);
-  regularBuilder1.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder1.encodePacketHeader().hasError());
   // Get a bunch frames
   QuicServerConnectionState conn(
       FizzServerQuicHandshakeContext::Builder().build());
@@ -414,8 +432,10 @@ TEST_F(QuicPacketRebuilderTest, CannotRebuildEmptyCryptoStream) {
   auto cryptoBuf = folly::IOBuf::copyBuffer("NewSessionTicket");
 
   // Write them with a regular builder
-  writeCryptoFrame(
-      cryptoOffset, ChainedByteRangeHead(cryptoBuf), regularBuilder1);
+  ASSERT_FALSE(
+      writeCryptoFrame(
+          cryptoOffset, ChainedByteRangeHead(cryptoBuf), regularBuilder1)
+          .hasError());
   auto packet1 = std::move(regularBuilder1).buildPacket();
   ASSERT_EQ(1, packet1.packet.frames.size());
   // Do not add the buf to crypto stream's retransmission buffer,
@@ -426,10 +446,12 @@ TEST_F(QuicPacketRebuilderTest, CannotRebuildEmptyCryptoStream) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder2(
       kDefaultUDPSendPacketLen, std::move(shortHeader2), 0 /* largestAcked */);
-  regularBuilder2.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder2.encodePacketHeader().hasError());
   PacketRebuilder rebuilder(regularBuilder2, conn);
   auto outstanding = makeDummyOutstandingPacket(packet1.packet, 1000);
-  EXPECT_FALSE(rebuilder.rebuildFromPacket(outstanding).has_value());
+  auto rebuildResult = rebuilder.rebuildFromPacket(outstanding);
+  ASSERT_FALSE(rebuildResult.hasError());
+  EXPECT_FALSE(rebuildResult.value().hasValue());
 }
 
 TEST_F(QuicPacketRebuilderTest, CannotRebuild) {
@@ -437,7 +459,7 @@ TEST_F(QuicPacketRebuilderTest, CannotRebuild) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder1(
       kDefaultUDPSendPacketLen, std::move(shortHeader1), 0 /* largestAcked */);
-  regularBuilder1.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder1.encodePacketHeader().hasError());
   // Get a bunch frames
   ConnectionCloseFrame connCloseFrame(
       QuicErrorCode(TransportErrorCode::FRAME_ENCODING_ERROR),
@@ -454,17 +476,18 @@ TEST_F(QuicPacketRebuilderTest, CannotRebuild) {
       .ackDelayExponent = static_cast<uint8_t>(kDefaultAckDelayExponent)};
   QuicServerConnectionState conn(
       FizzServerQuicHandshakeContext::Builder().build());
-  conn.streamManager->setMaxLocalBidirectionalStreams(10);
+  ASSERT_FALSE(
+      conn.streamManager->setMaxLocalBidirectionalStreams(10).hasError());
   auto stream = conn.streamManager->createNextBidirectionalStream().value();
   auto streamId = stream->id;
   auto buf =
       folly::IOBuf::copyBuffer("You can't deny you are looking for the sunset");
   PingFrame pingFrame;
   // Write them with a regular builder
-  writeFrame(connCloseFrame, regularBuilder1);
-  writeFrame(maxStreamIdFrame, regularBuilder1);
-  writeFrame(pingFrame, regularBuilder1);
-  writeAckFrame(ackMeta, regularBuilder1);
+  ASSERT_FALSE(writeFrame(connCloseFrame, regularBuilder1).hasError());
+  ASSERT_FALSE(writeFrame(maxStreamIdFrame, regularBuilder1).hasError());
+  ASSERT_FALSE(writeFrame(pingFrame, regularBuilder1).hasError());
+  ASSERT_FALSE(writeAckFrame(ackMeta, regularBuilder1).hasError());
   writeStreamFrameHeader(
       regularBuilder1,
       streamId,
@@ -492,10 +515,12 @@ TEST_F(QuicPacketRebuilderTest, CannotRebuild) {
           2,
       std::move(shortHeader2),
       0 /* largestAcked */);
-  regularBuilder2.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder2.encodePacketHeader().hasError());
   PacketRebuilder rebuilder(regularBuilder2, conn);
   auto outstanding = makeDummyOutstandingPacket(packet1.packet, 1000);
-  EXPECT_FALSE(rebuilder.rebuildFromPacket(outstanding).has_value());
+  auto rebuildResult = rebuilder.rebuildFromPacket(outstanding);
+  ASSERT_FALSE(rebuildResult.hasError());
+  EXPECT_FALSE(rebuildResult.value().hasValue());
 }
 
 TEST_F(QuicPacketRebuilderTest, CloneCounter) {
@@ -503,9 +528,9 @@ TEST_F(QuicPacketRebuilderTest, CloneCounter) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder(
       kDefaultUDPSendPacketLen, std::move(shortHeader1), 0 /* largestAcked */);
-  regularBuilder.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder.encodePacketHeader().hasError());
   MaxDataFrame maxDataFrame(31415926);
-  writeFrame(maxDataFrame, regularBuilder);
+  ASSERT_FALSE(writeFrame(maxDataFrame, regularBuilder).hasError());
   auto packet = std::move(regularBuilder).buildPacket();
   auto outstandingPacket = makeDummyOutstandingPacket(packet.packet, 1000);
   QuicServerConnectionState conn(
@@ -514,9 +539,9 @@ TEST_F(QuicPacketRebuilderTest, CloneCounter) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder2(
       kDefaultUDPSendPacketLen, std::move(shortHeader2), 0 /* largestAcked */);
-  regularBuilder2.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder2.encodePacketHeader().hasError());
   PacketRebuilder rebuilder(regularBuilder2, conn);
-  rebuilder.rebuildFromPacket(outstandingPacket);
+  ASSERT_FALSE(rebuilder.rebuildFromPacket(outstandingPacket).hasError());
   EXPECT_TRUE(outstandingPacket.maybeClonedPacketIdentifier.has_value());
   EXPECT_EQ(1, conn.outstandings.numClonedPackets());
 }
@@ -526,9 +551,9 @@ TEST_F(QuicPacketRebuilderTest, PurePingWillRebuild) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder(
       kDefaultUDPSendPacketLen, std::move(shortHeader1), 0);
-  regularBuilder.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder.encodePacketHeader().hasError());
   PingFrame pingFrame;
-  writeFrame(pingFrame, regularBuilder);
+  ASSERT_FALSE(writeFrame(pingFrame, regularBuilder).hasError());
   auto packet = std::move(regularBuilder).buildPacket();
   auto outstandingPacket = makeDummyOutstandingPacket(packet.packet, 50);
   EXPECT_EQ(1, outstandingPacket.packet.frames.size());
@@ -538,9 +563,11 @@ TEST_F(QuicPacketRebuilderTest, PurePingWillRebuild) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder2(
       kDefaultUDPSendPacketLen, std::move(shortHeader2), 0);
-  regularBuilder2.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder2.encodePacketHeader().hasError());
   PacketRebuilder rebuilder(regularBuilder2, conn);
-  EXPECT_TRUE(rebuilder.rebuildFromPacket(outstandingPacket).has_value());
+  auto rebuildResult = rebuilder.rebuildFromPacket(outstandingPacket);
+  ASSERT_FALSE(rebuildResult.hasError());
+  ASSERT_TRUE(rebuildResult.value().hasValue());
   EXPECT_TRUE(outstandingPacket.maybeClonedPacketIdentifier.has_value());
   EXPECT_EQ(1, conn.outstandings.numClonedPackets());
 }
@@ -548,7 +575,8 @@ TEST_F(QuicPacketRebuilderTest, PurePingWillRebuild) {
 TEST_F(QuicPacketRebuilderTest, LastStreamFrameSkipLen) {
   QuicServerConnectionState conn(
       FizzServerQuicHandshakeContext::Builder().build());
-  conn.streamManager->setMaxLocalBidirectionalStreams(100);
+  ASSERT_FALSE(
+      conn.streamManager->setMaxLocalBidirectionalStreams(100).hasError());
   auto stream = conn.streamManager->createNextBidirectionalStream().value();
   auto streamId = stream->id;
   auto buf1 =
@@ -559,7 +587,7 @@ TEST_F(QuicPacketRebuilderTest, LastStreamFrameSkipLen) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder(
       kDefaultUDPSendPacketLen, std::move(shortHeader), 0);
-  regularBuilder.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder.encodePacketHeader().hasError());
   writeStreamFrameHeader(
       regularBuilder,
       streamId,
@@ -592,7 +620,6 @@ TEST_F(QuicPacketRebuilderTest, LastStreamFrameSkipLen) {
       std::forward_as_tuple(buf1->computeChainDataLength()),
       std::forward_as_tuple(std::make_unique<WriteStreamBuffer>(
           ChainedByteRangeHead(buf2), buf1->computeChainDataLength(), true)));
-
   MockQuicPacketBuilder mockBuilder;
   size_t packetLimit = 1200;
   EXPECT_CALL(mockBuilder, remainingSpaceInPkt()).WillRepeatedly(Invoke([&]() {
@@ -614,17 +641,22 @@ TEST_F(QuicPacketRebuilderTest, LastStreamFrameSkipLen) {
   EXPECT_CALL(mockBuilder, write(_))
       .Times(4)
       .WillRepeatedly(Invoke([&](const QuicInteger& quicInt) {
-        packetLimit -= quicInt.getSize();
+        auto sizeResult = quicInt.getSize();
+        ASSERT_FALSE(sizeResult.hasError());
+        packetLimit -= sizeResult.value();
       }));
 
   PacketRebuilder rebuilder(mockBuilder, conn);
-  EXPECT_TRUE(rebuilder.rebuildFromPacket(outstandingPacket).has_value());
+  auto rebuildResult = rebuilder.rebuildFromPacket(outstandingPacket);
+  ASSERT_FALSE(rebuildResult.hasError());
+  ASSERT_TRUE(rebuildResult.value().hasValue());
 }
 
 TEST_F(QuicPacketRebuilderTest, LastStreamFrameFinOnlySkipLen) {
   QuicServerConnectionState conn(
       FizzServerQuicHandshakeContext::Builder().build());
-  conn.streamManager->setMaxLocalBidirectionalStreams(100);
+  ASSERT_FALSE(
+      conn.streamManager->setMaxLocalBidirectionalStreams(100).hasError());
   auto stream = conn.streamManager->createNextBidirectionalStream().value();
   auto streamId = stream->id;
   auto buf1 =
@@ -634,7 +666,7 @@ TEST_F(QuicPacketRebuilderTest, LastStreamFrameFinOnlySkipLen) {
       ProtectionType::KeyPhaseZero, getTestConnectionId(), 0);
   RegularQuicPacketBuilder regularBuilder(
       kDefaultUDPSendPacketLen, std::move(shortHeader), 0);
-  regularBuilder.encodePacketHeader();
+  ASSERT_FALSE(regularBuilder.encodePacketHeader().hasError());
   writeStreamFrameHeader(
       regularBuilder,
       streamId,
@@ -688,10 +720,14 @@ TEST_F(QuicPacketRebuilderTest, LastStreamFrameFinOnlySkipLen) {
   EXPECT_CALL(mockBuilder, write(_))
       .Times(5)
       .WillRepeatedly(Invoke([&](const QuicInteger& quicInt) {
-        packetLimit -= quicInt.getSize();
+        auto sizeResult = quicInt.getSize();
+        ASSERT_FALSE(sizeResult.hasError());
+        packetLimit -= sizeResult.value();
       }));
 
   PacketRebuilder rebuilder(mockBuilder, conn);
-  EXPECT_TRUE(rebuilder.rebuildFromPacket(outstandingPacket).has_value());
+  auto rebuildResult = rebuilder.rebuildFromPacket(outstandingPacket);
+  ASSERT_FALSE(rebuildResult.hasError());
+  ASSERT_TRUE(rebuildResult.value().hasValue());
 }
 } // namespace quic::test
