@@ -63,8 +63,11 @@ std::unique_ptr<QuicReadCodec> makeEncryptedCodec(
 TEST_F(QuicReadCodecTest, EmptyBuffer) {
   auto emptyQueue = bufToQueue(folly::IOBuf::create(0));
   AckStates ackStates;
-  EXPECT_FALSE(
-      parseSuccess(makeUnencryptedCodec()->parsePacket(emptyQueue, ackStates)));
+  auto packet = makeUnencryptedCodec()->parsePacket(emptyQueue, ackStates);
+  EXPECT_EQ(
+      packet.nothing()->reason,
+      PacketDropReason(PacketDropReason::UNEXPECTED_NOTHING));
+  EXPECT_FALSE(parseSuccess(std::move(packet)));
 }
 
 TEST_F(QuicReadCodecTest, TooSmallBuffer) {
@@ -196,10 +199,13 @@ TEST_F(QuicReadCodecTest, LongHeaderPacketLenMismatch) {
 
   RegularQuicPacketBuilder builder(
       kDefaultUDPSendPacketLen, std::move(headerIn), 0 /* largestAcked */);
-  builder.encodePacketHeader();
+  auto encodeResult = builder.encodePacketHeader();
+  ASSERT_FALSE(encodeResult.hasError());
   builder.accountForCipherOverhead(0);
   auto cryptoFrameBuf = folly::IOBuf::copyBuffer("CHLO");
-  writeCryptoFrame(0, ChainedByteRangeHead(cryptoFrameBuf), builder);
+  auto writeResult =
+      writeCryptoFrame(0, ChainedByteRangeHead(cryptoFrameBuf), builder);
+  ASSERT_FALSE(writeResult.hasError());
   auto packet = packetToBuf(std::move(builder).buildPacket());
   auto packetQueue = bufToQueue(std::move(packet));
 
@@ -277,7 +283,8 @@ TEST_F(QuicReadCodecTest, StreamWithShortHeaderOnlyHeader) {
   ShortHeader header(ProtectionType::KeyPhaseZero, connId, packetNum);
   RegularQuicPacketBuilder builder(
       kDefaultUDPSendPacketLen, std::move(header), 0 /* largestAcked */);
-  builder.encodePacketHeader();
+  auto encodeResult = builder.encodePacketHeader();
+  ASSERT_FALSE(encodeResult.hasError());
   auto packetBuf = packetToBuf(std::move(builder).buildPacket());
 
   auto aead = std::make_unique<MockAead>();
@@ -312,6 +319,9 @@ TEST_F(QuicReadCodecTest, PacketDecryptFail) {
   auto packetQueue = bufToQueue(packetToBuf(streamPacket));
   auto packet = makeEncryptedCodec(connId, std::move(aead))
                     ->parsePacket(packetQueue, ackStates);
+  EXPECT_EQ(
+      packet.nothing()->reason,
+      PacketDropReason(PacketDropReason::DECRYPTION_ERROR));
   EXPECT_FALSE(parseSuccess(std::move(packet)));
 }
 
@@ -607,6 +617,9 @@ TEST_F(QuicReadCodecTest, FailToDecryptLongHeaderNoReset) {
   AckStates ackStates;
   auto packetQueue = bufToQueue(packetToBuf(streamPacket));
   auto packet = codec->parsePacket(packetQueue, ackStates);
+  EXPECT_EQ(
+      packet.nothing()->reason,
+      PacketDropReason(PacketDropReason::DECRYPTION_ERROR_0RTT));
   EXPECT_FALSE(isReset(std::move(packet)));
 }
 
@@ -801,25 +814,25 @@ TEST_F(QuicReadCodecTest, TestZeroRttPacketsAfterHandshakeDone) {
 TEST_F(QuicReadCodecTest, parseEmptyStreamFrame) {
   auto buf = folly::IOBuf::copyBuffer("\x08");
   auto bufQueue = quic::BufQueue(std::move(buf));
-  EXPECT_THROW(
-      parseFrame(
-          bufQueue,
-          PacketHeader(ShortHeader(
-              ProtectionType::KeyPhaseOne, ConnectionId::createRandom(10))),
-          CodecParameters()),
-      QuicTransportException);
+  auto result = parseFrame(
+      bufQueue,
+      PacketHeader(ShortHeader(
+          ProtectionType::KeyPhaseOne, ConnectionId::createRandom(10))),
+      CodecParameters());
+  EXPECT_FALSE(result.hasValue());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(QuicReadCodecTest, parseEmptyDatagramFrame) {
   auto buf = folly::IOBuf::copyBuffer("\x31");
   auto bufQueue = quic::BufQueue(std::move(buf));
-  EXPECT_THROW(
-      parseFrame(
-          bufQueue,
-          PacketHeader(ShortHeader(
-              ProtectionType::KeyPhaseOne, ConnectionId::createRandom(10))),
-          CodecParameters()),
-      QuicTransportException);
+  auto result = parseFrame(
+      bufQueue,
+      PacketHeader(ShortHeader(
+          ProtectionType::KeyPhaseOne, ConnectionId::createRandom(10))),
+      CodecParameters());
+  EXPECT_FALSE(result.hasValue());
+  EXPECT_EQ(result.error().code, TransportErrorCode::FRAME_ENCODING_ERROR);
 }
 
 TEST_F(QuicReadCodecTest, KeyUpdateIncomingValid) {
