@@ -40,16 +40,17 @@ const RegularQuicWritePacket& writeQuicPacket(
   auto version = conn.version.value_or(*conn.originalVersion);
   auto aead = createNoOpAead();
   auto headerCipher = createNoOpHeaderCipher();
-  writeDataToQuicStream(stream, data.clone(), eof);
-  writeQuicDataToSocket(
-      sock,
-      conn,
-      srcConnId,
-      dstConnId,
-      *aead,
-      *headerCipher,
-      version,
-      conn.transportSettings.writeConnectionDataPacketsLimit);
+  CHECK(!writeDataToQuicStream(stream, data.clone(), eof).hasError());
+  CHECK(!writeQuicDataToSocket(
+             sock,
+             conn,
+             srcConnId,
+             dstConnId,
+             *aead,
+             *headerCipher,
+             version,
+             conn.transportSettings.writeConnectionDataPacketsLimit)
+             .hasError());
   CHECK(
       conn.outstandings.packets.rend() !=
       getLastOutstandingPacket(conn, PacketNumberSpace::AppData));
@@ -64,16 +65,17 @@ PacketNum rstStreamAndSendPacket(
   auto aead = createNoOpAead();
   auto headerCipher = createNoOpHeaderCipher();
   auto version = conn.version.value_or(*conn.originalVersion);
-  sendRstSMHandler(stream, errorCode);
-  writeQuicDataToSocket(
-      sock,
-      conn,
-      *conn.clientConnectionId,
-      *conn.serverConnectionId,
-      *aead,
-      *headerCipher,
-      version,
-      conn.transportSettings.writeConnectionDataPacketsLimit);
+  CHECK(!sendRstSMHandler(stream, errorCode).hasError());
+  CHECK(!writeQuicDataToSocket(
+             sock,
+             conn,
+             *conn.clientConnectionId,
+             *conn.serverConnectionId,
+             *aead,
+             *headerCipher,
+             version,
+             conn.transportSettings.writeConnectionDataPacketsLimit)
+             .hasError());
 
   for (const auto& packet : conn.outstandings.packets) {
     for (const auto& frame : packet.packet.frames) {
@@ -91,6 +93,14 @@ PacketNum rstStreamAndSendPacket(
   return 0;
 }
 
+void writeStreamFrameData(
+    PacketBuilderInterface& builder,
+    Buf writeBuffer,
+    uint64_t dataLen) {
+  ChainedByteRangeHead dataHead(writeBuffer);
+  writeStreamFrameData(builder, dataHead, dataLen);
+}
+
 RegularQuicPacketBuilder::Packet createAckPacket(
     QuicConnectionStateBase& dstConn,
     PacketNum pn,
@@ -175,7 +185,8 @@ class AcceptingTicketCipher : public fizz::server::TicketCipher {
         kDefaultStreamFlowControlWindow,
         kDefaultStreamFlowControlWindow,
         kDefaultMaxStreamsBidirectional,
-        kDefaultMaxStreamsUnidirectional);
+        kDefaultMaxStreamsUnidirectional,
+        0 /*extendedAckSupport*/);
     appToken.version = QuicVersion::MVFST;
     resState.appToken = encodeAppToken(appToken);
     return resState;
@@ -294,9 +305,9 @@ RegularQuicPacketBuilder::Packet createStreamPacket(
     builder.reset(new RegularQuicPacketBuilder(
         packetSizeLimit, std::move(header), largestAcked));
   }
-  builder->encodePacketHeader();
+  CHECK(!builder->encodePacketHeader().hasError());
   builder->accountForCipherOverhead(cipherOverhead);
-  auto dataLen = *writeStreamFrameHeader(
+  auto res = *writeStreamFrameHeader(
       *builder,
       streamId,
       offset,
@@ -304,9 +315,12 @@ RegularQuicPacketBuilder::Packet createStreamPacket(
       data.computeChainDataLength(),
       eof,
       none /* skipLenHint */);
+  CHECK(res.hasValue()) << "failed to write stream frame header";
+  auto dataLen = *res;
+  auto dataBuf = data.clone();
   writeStreamFrameData(
       *builder,
-      data.clone(),
+      std::move(dataBuf),
       std::min(folly::to<size_t>(dataLen), data.computeChainDataLength()));
   return std::move(*builder).buildPacket();
 }
@@ -339,7 +353,7 @@ RegularQuicPacketBuilder::Packet createInitialCryptoPacket(
   if (!builder) {
     builder = &fallbackBuilder;
   }
-  builder->encodePacketHeader();
+  CHECK(!builder->encodePacketHeader().hasError());
   builder->accountForCipherOverhead(aead.getCipherOverhead());
   auto res = writeCryptoFrame(offset, data, *builder);
   CHECK(res.hasValue()) << "failed to write crypto frame";
@@ -382,7 +396,7 @@ RegularQuicPacketBuilder::Packet createCryptoPacket(
   }
   RegularQuicPacketBuilder builder(
       packetSizeLimit, std::move(*header), largestAcked);
-  builder.encodePacketHeader();
+  CHECK(!builder.encodePacketHeader().hasError());
   builder.accountForCipherOverhead(aead.getCipherOverhead());
   auto res = writeCryptoFrame(offset, data, builder);
   CHECK(res.hasValue()) << "failed to write crypto frame";
