@@ -47,6 +47,10 @@ void LibevQuicAsyncUDPSocket::pauseRead() {
   removeEvent(EV_READ);
 }
 
+bool LibevQuicAsyncUDPSocket::isReadPaused() const {
+  return readCallback_ == nullptr;
+}
+
 void LibevQuicAsyncUDPSocket::resumeRead(ReadCallback* cb) {
   CHECK(!readCallback_) << "A read callback is already installed";
   CHECK_NE(fd_, -1)
@@ -113,9 +117,10 @@ int LibevQuicAsyncUDPSocket::getGSO() {
 }
 
 int LibevQuicAsyncUDPSocket::writem(
-    folly::Range<folly::SocketAddress const*> /* addrs */,
-    const std::unique_ptr<folly::IOBuf>* /* bufs */,
-    size_t /* count */) {
+    folly::Range<folly::SocketAddress const*>,
+    iovec*,
+    size_t*,
+    size_t) {
   LOG(FATAL) << __func__ << "is not implemented in LibevQuicAsyncUDPSocket";
   return -1;
 }
