@@ -8,31 +8,30 @@
 #include <quic/congestion_control/Bbr2.h>
 
 #include <quic/congestion_control/CongestionControlFunctions.h>
-#include <sys/types.h>
 #include <chrono>
 #include <cstdint>
-#include <cstdlib>
 #include <limits>
 
 namespace quic {
 
 constexpr uint64_t kMaxBwFilterLen = 2; // Measured in number of ProbeBW cycles
-constexpr std::chrono::microseconds kMinRttFilterLen = 10s;
-constexpr std::chrono::microseconds kProbeRTTInterval = 5s;
+constexpr std::chrono::microseconds kProbeRTTInterval = 10s;
 constexpr std::chrono::microseconds kProbeRttDuration = 200ms;
+constexpr std::chrono::microseconds kMinRttFilterLen = 10s + kProbeRttDuration;
 constexpr uint64_t kMaxExtraAckedFilterLen =
     10; // Measured in packet-timed round trips
 
-constexpr float kStartupPacingGain = 2.89; // 2 / ln(2)
-constexpr float kDrainPacingGain = 1 / kStartupPacingGain;
+constexpr float kStartupPacingGain = 2.885; // 2 / ln(2)
+constexpr float kDrainPacingGain = 0.5;
 constexpr float kProbeBwDownPacingGain = 0.9;
 constexpr float kProbeBwCruiseRefillPacingGain = 1.0;
 constexpr float kProbeBwUpPacingGain = 1.25;
 constexpr float kProbeRttPacingGain = 1.0;
 
-constexpr float kStartupCwndGain = 2.89;
+constexpr float kStartupCwndGain = 2.885;
 constexpr float kProbeBwCruiseRefillCwndGain = 2.0;
-constexpr float kProbeBwUpDownCwndGain = 2.0;
+constexpr float kProbeBwDownCwndGain = 2.0;
+constexpr float kProbeBwUpCwndGain = 2.25;
 constexpr float kProbeRttCwndGain = 0.5;
 
 constexpr float kBeta = 0.7;
@@ -40,13 +39,7 @@ constexpr float kBeta = 0.7;
 constexpr float kLossThreshold = 0.02;
 constexpr float kHeadroomFactor = 0.15;
 
-#ifndef CLANG_LAZY_INIT
-#define CLANG_LAZY_INIT
-#endif
-CLANG_LAZY_INIT quic::Bandwidth kMinPacingRateForSendQuantum{1200 * 1000, 1s};
-// The experimental pacer currently achieves ~99% of the target rate
-// we should not reduce the target by adding an extra margin.
-// TODO: add the margin back if the pacer performance improves further.
+// TODO: Restore this margin
 constexpr uint8_t kPacingMarginPercent = 0;
 
 Bbr2CongestionController::Bbr2CongestionController(
@@ -60,8 +53,17 @@ Bbr2CongestionController::Bbr2CongestionController(
       cwndBytes_(
           conn_.udpSendPacketLen * conn_.transportSettings.initCwndInMss) {
   resetCongestionSignals();
+  resetFullBw();
   resetLowerBounds();
-  // Start with the default pacing settings.
+  // If we explicitly don't want to pace the init cwnd, reset the pacing rate.
+  // Otherwise, leave it to the pacer's initial state.
+  if (!conn_.transportSettings.ccaConfig.paceInitCwnd) {
+    if (conn_.pacer) {
+      conn_.pacer->refreshPacingRate(cwndBytes_, 0us);
+    } else {
+      LOG(WARNING) << "BBR2 was initialized on a connection without a pacer";
+    }
+  }
   enterStartup();
 }
 
@@ -100,22 +102,23 @@ void Bbr2CongestionController::onPacketSent(
 void Bbr2CongestionController::onPacketAckOrLoss(
     const AckEvent* FOLLY_NULLABLE ackEvent,
     const LossEvent* FOLLY_NULLABLE lossEvent) {
-  if (conn_.qLogger) {
-    conn_.qLogger->addCongestionMetricUpdate(
-        conn_.lossState.inflightBytes,
-        getCongestionWindow(),
-        kCongestionPacketAck,
-        bbr2StateToString(state_));
-    conn_.qLogger->addNetworkPathModelUpdate(
-        inflightHi_.value_or(0),
-        inflightLo_.value_or(0),
-        bandwidthHi_.has_value() ? bandwidthHi_->units : 0,
-        bandwidthHi_.has_value() ? bandwidthHi_->interval
-                                 : std::chrono::microseconds(1),
-        bandwidthLo_.has_value() ? bandwidthLo_->units : 0,
-        bandwidthLo_.has_value() ? bandwidthLo_->interval
-                                 : std::chrono::microseconds(1));
-  }
+  SCOPE_EXIT {
+    if (conn_.qLogger) {
+      conn_.qLogger->addCongestionMetricUpdate(
+          conn_.lossState.inflightBytes,
+          getCongestionWindow(),
+          kCongestionPacketAck,
+          bbr2StateToString(state_));
+      conn_.qLogger->addNetworkPathModelUpdate(
+          inflightHi_.value_or(0),
+          inflightLo_.value_or(0),
+          0, // bandwidthHi_ no longer available.
+          std::chrono::microseconds(1), // bandwidthHi_ no longer available.
+          bandwidthLo_.has_value() ? bandwidthLo_->units : 0,
+          bandwidthLo_.has_value() ? bandwidthLo_->interval
+                                   : std::chrono::microseconds(1));
+    }
+  };
   if (ackEvent) {
     subtractAndCheckUnderflow(
         conn_.lossState.inflightBytes, ackEvent->ackedBytes);
@@ -131,32 +134,17 @@ void Bbr2CongestionController::onPacketAckOrLoss(
             << ")";
   };
 
-  if (lossEvent && lossEvent->lostPackets > 0 &&
-      conn_.transportSettings.ccaConfig.conservativeRecovery) {
-    // The pseudo code in BBRHandleLostPacket is included in
-    // updateProbeBwCyclePhase. No need to repeat it here.
-
-    // We don't expose the loss type here, so always use fast recovery for
-    // non-persistent congestion
-    saveCwnd();
-    inPacketConservation_ = true;
-    // Mark the connection as app-limited so bw samples during recovery are not
-    // taken into account.
-    setAppLimited();
-    packetConservationStartTime_ = Clock::now();
-    if (lossEvent->persistentCongestion) {
-      cwndBytes_ = kMinCwndInMssForBbr * conn_.udpSendPacketLen;
-    } else {
-      auto newlyAcked = ackEvent ? ackEvent->ackedBytes : 0;
-      cwndBytes_ = conn_.lossState.inflightBytes +
-          std::max(newlyAcked, conn_.udpSendPacketLen);
-    }
+  if (lossEvent && lossEvent->lostPackets > 0) {
+    auto ackedBytes = ackEvent ? ackEvent->ackedBytes : 0;
+    onPacketLoss(*lossEvent, ackedBytes);
   }
 
   if (ackEvent) {
-    if (ackEvent->implicit) {
+    currentAckEvent_ = ackEvent;
+
+    if (currentAckEvent_->implicit) {
       // Implicit acks should not be used for bandwidth or rtt estimation
-      setCwnd(ackEvent->ackedBytes, 0);
+      setCwnd();
       return;
     }
 
@@ -168,44 +156,41 @@ void Bbr2CongestionController::onPacketAckOrLoss(
       }
     }
 
-    if (inPacketConservation_ &&
-        packetConservationStartTime_ <=
-            ackEvent->largestNewlyAckedPacketSentTime) {
-      inPacketConservation_ = false;
-      restoreCwnd();
-    }
+    updateBandwidthSampleFromAck(*ackEvent);
+    auto lastAckedPacket = currentAckEvent_->getLargestNewlyAckedPacket();
+    inflightBytesAtLastAckedPacket_ = lastAckedPacket
+        ? lastAckedPacket->outstandingPacketMetadata.inflightBytes
+        : conn_.lossState.inflightBytes;
+    lastAckedPacketAppLimited_ =
+        lastAckedPacket ? lastAckedPacket->isAppLimited : true;
 
     // UpdateModelAndState
-    updateLatestDeliverySignals(*ackEvent);
-    updateRound(*ackEvent);
+    updateLatestDeliverySignals();
+    updateRound();
 
-    // Update cwndLimited state
-    if (roundStart_) {
-      cwndLimitedInRound_ = false;
-    }
+    updateRecoveryOnAck();
 
     updateCongestionSignals(lossEvent);
-    updateAckAggregation(*ackEvent);
+    updateAckAggregation();
+    checkFullBwReached();
     checkStartupDone();
     checkDrain();
 
-    auto inflightBytesAtLargestAckedPacket =
-        ackEvent->getLargestNewlyAckedPacket()
-        ? ackEvent->getLargestNewlyAckedPacket()
-              ->outstandingPacketMetadata.inflightBytes
-        : conn_.lossState.inflightBytes;
-    auto lostBytes = lossEvent ? lossEvent->lostBytes : 0;
-    updateProbeBwCyclePhase(
-        ackEvent->ackedBytes, inflightBytesAtLargestAckedPacket, lostBytes);
+    updateProbeBwCyclePhase();
     updateMinRtt();
-    checkProbeRtt(ackEvent->ackedBytes);
-    advanceLatestDeliverySignals(*ackEvent);
+    checkProbeRtt();
+    advanceLatestDeliverySignals();
     boundBwForModel();
 
     // UpdateControlParameters
     setPacing();
     setSendQuantum();
-    setCwnd(ackEvent->ackedBytes, lostBytes);
+    setCwnd();
+
+    // Update cwndLimited state before the next ack
+    if (roundStart_) {
+      cwndLimitedInRound_ = false;
+    }
   }
 }
 
@@ -223,14 +208,14 @@ CongestionControlType Bbr2CongestionController::type() const noexcept {
   return CongestionControlType::BBR2;
 }
 
-bool Bbr2CongestionController::isInBackgroundMode() const {
-  return false;
-}
-
 Optional<Bandwidth> Bbr2CongestionController::getBandwidth() const {
   return bandwidth_;
 }
 
+uint64_t Bbr2CongestionController::getBDP() const {
+  return getBDPWithGain(1.0);
+}
+
 bool Bbr2CongestionController::isAppLimited() const {
   return appLimited_;
 }
@@ -244,9 +229,82 @@ void Bbr2CongestionController::setAppLimited() noexcept {
 }
 
 // Internals
+
+void Bbr2CongestionController::onPacketLoss(
+    const LossEvent& lossEvent,
+    uint64_t ackedBytes) {
+  if ((state_ == State::Startup || state_ == State::Drain) &&
+      !conn_.transportSettings.ccaConfig.enableRecoveryInStartup) {
+    // Recovery is not enabled in Startup.
+    return;
+  }
+  if ((isProbeBwState(state_) || state_ == State::ProbeRTT) &&
+      !conn_.transportSettings.ccaConfig.enableRecoveryInProbeStates) {
+    // Recovery is not enabled in Probe states.
+    return;
+  }
+
+  saveCwnd();
+  recoveryStartTime_ = Clock::now();
+  if (!isInRecovery()) {
+    recoveryState_ = RecoveryState::CONSERVATIVE;
+    recoveryWindow_ = conn_.lossState.inflightBytes + ackedBytes;
+
+    // Ensure conservative recovery lasts for at least a whole round trip.
+    nextRoundDelivered_ = conn_.lossState.totalBytesAcked;
+  }
+
+  if (lossEvent.persistentCongestion) {
+    recoveryWindow_ = kMinCwndInMssForBbr * conn_.udpSendPacketLen;
+  } else {
+    recoveryWindow_ =
+        (recoveryWindow_ > kMinCwndInMssForBbr * conn_.udpSendPacketLen)
+        ? recoveryWindow_ - lossEvent.lostBytes
+        : kMinCwndInMssForBbr * conn_.udpSendPacketLen;
+  }
+
+  recoveryWindow_ = boundedCwnd(
+      recoveryWindow_,
+      conn_.udpSendPacketLen,
+      conn_.transportSettings.maxCwndInMss,
+      kMinCwndInMssForBbr);
+}
+
+void Bbr2CongestionController::updateRecoveryOnAck() {
+  if (!isInRecovery()) {
+    return;
+  }
+
+  if (roundStart_ && recoveryState_ != RecoveryState::GROWTH) {
+    recoveryState_ = RecoveryState::GROWTH;
+  }
+
+  if (recoveryStartTime_ <= currentAckEvent_->largestNewlyAckedPacketSentTime) {
+    recoveryState_ = RecoveryState::NOT_RECOVERY;
+    restoreCwnd();
+  } else {
+    if (recoveryState_ == RecoveryState::GROWTH) {
+      recoveryWindow_ += currentAckEvent_->ackedBytes;
+    }
+
+    uint64_t recoveryIncrease =
+        conn_.transportSettings.ccaConfig.conservativeRecovery
+        ? conn_.udpSendPacketLen
+        : currentAckEvent_->ackedBytes;
+    recoveryWindow_ = std::max(
+        recoveryWindow_, conn_.lossState.inflightBytes + recoveryIncrease);
+    recoveryWindow_ = boundedCwnd(
+        recoveryWindow_,
+        conn_.udpSendPacketLen,
+        conn_.transportSettings.maxCwndInMss,
+        kMinCwndInMssForBbr);
+  }
+}
+
 void Bbr2CongestionController::resetCongestionSignals() {
   lossBytesInRound_ = 0;
   lossEventsInRound_ = 0;
+  largestLostPacketNumInRound_ = 0;
   bandwidthLatest_ = Bandwidth();
   inflightLatest_ = 0;
 }
@@ -255,6 +313,7 @@ void Bbr2CongestionController::resetLowerBounds() {
   bandwidthLo_.reset();
   inflightLo_.reset();
 }
+
 void Bbr2CongestionController::enterStartup() {
   state_ = State::Startup;
   updatePacingAndCwndGain();
@@ -275,7 +334,7 @@ void Bbr2CongestionController::setPacing() {
           << " kPacingMarginPercent=" << kPacingMarginPercent
           << " units=" << pacingWindow << " interval=" << minRtt_.count();
 
-  if (state_ == State::Startup && !filledPipe_) {
+  if (state_ == State::Startup && !fullBwReached_) {
     pacingWindow = std::max(
         pacingWindow,
         conn_.udpSendPacketLen * conn_.transportSettings.initCwndInMss);
@@ -285,45 +344,39 @@ void Bbr2CongestionController::setPacing() {
 
 void Bbr2CongestionController::setSendQuantum() {
   auto rate = bandwidth_ * pacingGain_ * (100 - kPacingMarginPercent) / 100;
-  auto floor = 2 * conn_.udpSendPacketLen;
-  if (rate < kMinPacingRateForSendQuantum) {
-    floor = conn_.udpSendPacketLen;
-  }
-  auto rateIn1Ms = rate * 1ms;
-  sendQuantum_ = std::min(rateIn1Ms, decltype(rateIn1Ms)(64 * 1024));
-  sendQuantum_ = std::max(sendQuantum_, floor);
+  auto burstInPacerTick = rate * conn_.transportSettings.pacingTickInterval;
+  sendQuantum_ =
+      std::min(burstInPacerTick, decltype(burstInPacerTick)(64 * 1024));
+  sendQuantum_ = std::max(sendQuantum_, 2 * conn_.udpSendPacketLen);
 }
 
-void Bbr2CongestionController::setCwnd(
-    uint64_t ackedBytes,
-    uint64_t lostBytes) {
+void Bbr2CongestionController::setCwnd() {
   // BBRUpdateMaxInflight()
-  auto inflightMax = addQuantizationBudget(
-      getBDPWithGain(cwndGain_) + maxExtraAckedFilter_.GetBest());
-  // BBRModulateCwndForRecovery()
-  if (lostBytes > 0 && !conn_.transportSettings.ccaConfig.ignoreLoss) {
-    cwndBytes_ = std::max(
-        cwndBytes_ - std::min(lostBytes, cwndBytes_),
-        kMinCwndInMssForBbr * conn_.udpSendPacketLen);
+  const auto& ackedBytes = currentAckEvent_->ackedBytes;
+  auto targetBDP = getBDPWithGain(cwndGain_);
+  if (fullBwReached_) {
+    targetBDP += maxExtraAckedFilter_.GetBest();
+  } else if (conn_.transportSettings.ccaConfig.enableAckAggregationInStartup) {
+    targetBDP += latestExtraAcked_;
   }
-  if (inPacketConservation_) {
-    cwndBytes_ =
-        std::max(cwndBytes_, conn_.lossState.inflightBytes + ackedBytes);
+  auto inflightMax = addQuantizationBudget(targetBDP);
+
+  if (fullBwReached_) {
+    cwndBytes_ = std::min(cwndBytes_ + ackedBytes, inflightMax);
+  } else if (
+      cwndBytes_ < inflightMax ||
+      conn_.lossState.totalBytesAcked <
+          conn_.transportSettings.initCwndInMss * conn_.udpSendPacketLen) {
+    cwndBytes_ += ackedBytes;
   }
 
-  if (!inPacketConservation_) {
-    if (filledPipe_) {
-      cwndBytes_ = std::min(cwndBytes_ + ackedBytes, inflightMax);
-    } else if (
-        cwndBytes_ < inflightMax ||
-        conn_.lossState.totalBytesAcked <
-            conn_.transportSettings.initCwndInMss * conn_.udpSendPacketLen) {
-      cwndBytes_ += ackedBytes;
-    }
-    cwndBytes_ =
-        std::max(cwndBytes_, kMinCwndInMssForBbr * conn_.udpSendPacketLen);
+  if (isInRecovery()) {
+    cwndBytes_ = std::min(cwndBytes_, recoveryWindow_);
   }
 
+  cwndBytes_ =
+      std::max(cwndBytes_, kMinCwndInMssForBbr * conn_.udpSendPacketLen);
+
   // BBRBoundCwndForProbeRTT()
   if (state_ == State::ProbeRTT) {
     cwndBytes_ = std::min(cwndBytes_, getProbeRTTCwnd());
@@ -343,8 +396,13 @@ void Bbr2CongestionController::setCwnd(
       !conn_.transportSettings.ccaConfig.ignoreLoss) {
     cap = std::min(cap, *inflightLo_);
   }
-  cap = std::max(cap, kMinCwndInMssForBbr * conn_.udpSendPacketLen);
   cwndBytes_ = std::min(cwndBytes_, cap);
+
+  cwndBytes_ = boundedCwnd(
+      cwndBytes_,
+      conn_.udpSendPacketLen,
+      conn_.transportSettings.maxCwndInMss,
+      kMinCwndInMssForBbr);
 }
 
 void Bbr2CongestionController::checkProbeRttDone() {
@@ -362,9 +420,10 @@ void Bbr2CongestionController::restoreCwnd() {
   cwndBytes_ = std::max(cwndBytes_, previousCwndBytes_);
   VLOG(6) << "Restored cwnd: " << cwndBytes_;
 }
+
 void Bbr2CongestionController::exitProbeRtt() {
   resetLowerBounds();
-  if (filledPipe_) {
+  if (fullBwReached_) {
     startProbeBwDown();
     startProbeBwCruise();
   } else {
@@ -372,17 +431,15 @@ void Bbr2CongestionController::exitProbeRtt() {
   }
 }
 
-void Bbr2CongestionController::updateLatestDeliverySignals(
-    const AckEvent& ackEvent) {
+void Bbr2CongestionController::updateLatestDeliverySignals() {
   lossRoundStart_ = false;
 
-  bandwidthLatest_ =
-      std::max(bandwidthLatest_, getBandwidthSampleFromAck(ackEvent));
+  bandwidthLatest_ = std::max(bandwidthLatest_, currentBwSample_);
   VLOG(6) << "Bandwidth latest=" << bandwidthLatest_.normalizedDescribe()
           << "  AppLimited=" << bandwidthLatest_.isAppLimited;
-  inflightLatest_ = std::max(inflightLatest_, bandwidthLatest_.units);
+  inflightLatest_ = std::max(inflightLatest_, currentAckMaxInflightBytes_);
 
-  auto pkt = ackEvent.getLargestNewlyAckedPacket();
+  auto pkt = currentAckEvent_->getLargestNewlyAckedPacket();
   if (pkt &&
       pkt->outstandingPacketMetadata.totalBytesSent > lossRoundEndBytesSent_) {
     // Uses bytes sent instead of ACKed in the spec. This doesn't affect the
@@ -407,19 +464,29 @@ void Bbr2CongestionController::updateCongestionSignals(
   }
 
   // Update loss signal
-  if (lossEvent && lossEvent->lostBytes > 0) {
+  if (lossEvent && lossEvent->lostBytes > 0 &&
+      !lossEvent->lostPacketNumbers.empty()) {
     lossBytesInRound_ += lossEvent->lostBytes;
-    lossEventsInRound_ += 1;
+
+    // Only count non-contiguous losses as lossEvents
+    auto lastLossPn = largestLostPacketNumInRound_;
+    for (auto& pn : lossEvent->lostPacketNumbers) {
+      if (pn > lastLossPn + 1) {
+        lossEventsInRound_ += 1;
+      }
+      lastLossPn = pn;
+    }
+
+    // lossEvent->largestLostPacketNum should always be set if we have losses.
+    largestLostPacketNumInRound_ = std::max(
+        lossEvent->largestLostPacketNum.value_or(0),
+        largestLostPacketNumInRound_);
   }
 
   if (!lossRoundStart_) {
     return; // we're still within the same round
   }
-  // AdaptLowerBoundsFromCongestion - once per round-trip
-  if (state_ == State::ProbeBw_Up) {
-    return;
-  }
-  if (lossBytesInRound_ > 0) {
+  if (lossBytesInRound_ > 0 && !isProbingBandwidth(state_)) {
     // InitLowerBounds
     if (!bandwidthLo_.has_value()) {
       bandwidthLo_ = maxBwFilter_.GetBest();
@@ -429,19 +496,25 @@ void Bbr2CongestionController::updateCongestionSignals(
     }
 
     // LossLowerBounds
-    bandwidthLo_ = std::max(bandwidthLatest_, *bandwidthLo_ * kBeta);
+    auto bwLoBeta =
+        (conn_.transportSettings.ccaConfig.overrideBwShortBeta < 0.5 ||
+         conn_.transportSettings.ccaConfig.overrideBwShortBeta > 1.0)
+        ? kBeta
+        : conn_.transportSettings.ccaConfig.overrideBwShortBeta;
+    bandwidthLo_ = std::max(bandwidthLatest_, *bandwidthLo_ * bwLoBeta);
     inflightLo_ =
         std::max(inflightLatest_, static_cast<uint64_t>(*inflightLo_ * kBeta));
   }
 
   lossBytesInRound_ = 0;
   lossEventsInRound_ = 0;
+  largestLostPacketNumInRound_ = 0;
 }
 
-void Bbr2CongestionController::updateAckAggregation(const AckEvent& ackEvent) {
+void Bbr2CongestionController::updateAckAggregation() {
   /* Find excess ACKed beyond expected amount over this interval */
   auto interval =
-      Clock::now() - extraAckedStartTimestamp_.value_or(TimePoint());
+      Clock::now() - extraAckedStartTimestamp_.value_or(conn_.connectionTime);
   auto expectedDelivered = bandwidth_ *
       std::chrono::duration_cast<std::chrono::microseconds>(interval);
   /* Reset interval if ACK rate is below expected rate: */
@@ -450,64 +523,70 @@ void Bbr2CongestionController::updateAckAggregation(const AckEvent& ackEvent) {
     extraAckedStartTimestamp_ = Clock::now();
     expectedDelivered = 0;
   }
-  extraAckedDelivered_ += ackEvent.ackedBytes;
-  auto extra = extraAckedDelivered_ - expectedDelivered;
-  extra = std::min(extra, cwndBytes_);
-  maxExtraAckedFilter_.Update(extra, roundCount_);
+  extraAckedDelivered_ += currentAckEvent_->ackedBytes;
+  latestExtraAcked_ = extraAckedDelivered_ - expectedDelivered;
+  latestExtraAcked_ = std::min(latestExtraAcked_, cwndBytes_);
+  maxExtraAckedFilter_.Update(latestExtraAcked_, roundCount_);
 }
+
 void Bbr2CongestionController::checkStartupDone() {
-  checkStartupFullBandwidth();
   checkStartupHighLoss();
 
-  if (state_ == State::Startup && filledPipe_) {
+  if (state_ == State::Startup && fullBwReached_) {
+    QUIC_STATS(conn_.statsCallback, onBBR2ExitStartup);
     enterDrain();
   }
 }
 
-void Bbr2CongestionController::checkStartupFullBandwidth() {
-  if (filledPipe_ || !roundStart_ || isAppLimited()) {
-    return; /* no need to check for a full pipe now */
-  }
-  if (maxBwFilter_.GetBest() >=
-      filledPipeBandwidth_ * 1.25) { /* still growing? */
-    filledPipeBandwidth_ =
-        maxBwFilter_.GetBest(); /* record new baseline level */
-    filledPipeCount_ = 0;
-    return;
-  }
-  filledPipeCount_++; /* another round w/o much growth */
-  if (filledPipeCount_ >= 3) {
-    filledPipe_ = true;
-  }
-}
-
 void Bbr2CongestionController::checkStartupHighLoss() {
   /*
   Our implementation differs from the spec a bit here. The conditions in the
   spec are:
-  1. The connection has been in fast recovery for at least one full round trip.
+  1. The connection has been in fast recovery for at least one full packet-timed
+  round trip.
   2. The loss rate over the time scale of a single full round trip exceeds
   BBRLossThresh (2%).
-  3. There are at least BBRStartupFullLossCnt=3 noncontiguous sequence ranges
-  lost in that round trip.
+  3. There are at least BBRStartupFullLossCnt=6
+  discontiguous sequence ranges lost in that round trip.
 
   For 1,2 we use the loss pct from the last loss round which means we could exit
-  before a full RTT. For 3, we check we received three separate loss events
-  which servers a similar purpose to discontiguous ranges but it's not exactly
-  the same.
+  before a full RTT.
   */
-  if (filledPipe_ || !roundStart_ || isAppLimited() ||
-      conn_.transportSettings.ccaConfig.ignoreLoss) {
-    // TODO: the appLimited condition means we could tolerate losses in startup
-    // if we haven't found the full bandwidth. This may need to be revisited.
-
+  if (fullBwReached_ || !roundStart_ || lastAckedPacketAppLimited_ ||
+      !conn_.transportSettings.ccaConfig.exitStartupOnLoss) {
     return; /* no need to check for a the loss exit condition now */
   }
-  if (lossPctInLastRound_ > kLossThreshold && lossEventsInLastRound_ >= 3) {
-    filledPipe_ = true;
+  if (lossPctInLastRound_ > kLossThreshold && lossEventsInLastRound_ >= 6) {
+    fullBwReached_ = true;
+    inflightHi_ = std::max(getBDPWithGain(), inflightLatest_);
   }
 }
 
+void Bbr2CongestionController::checkFullBwReached() {
+  if (fullBwNow_ || lastAckedPacketAppLimited_) {
+    return; /* no need to check for a full pipe now */
+  }
+  if (!roundStart_) {
+    return;
+  }
+  if (maxBwFilter_.GetBest() >= fullBw_ * 1.25) {
+    resetFullBw(); // bw still growing, reset tracking
+    fullBw_ = maxBwFilter_.GetBest(); /* record new baseline level */
+    return;
+  }
+  fullBwCount_++; /* another round w/o much growth */
+  fullBwNow_ = (fullBwCount_ >= 3);
+  if (fullBwNow_) {
+    fullBwReached_ = true;
+  }
+}
+
+void Bbr2CongestionController::resetFullBw() {
+  fullBw_ = Bandwidth();
+  fullBwNow_ = false;
+  fullBwCount_ = 0;
+}
+
 void Bbr2CongestionController::enterDrain() {
   state_ = State::Drain;
   updatePacingAndCwndGain();
@@ -523,15 +602,13 @@ void Bbr2CongestionController::checkDrain() {
     enterProbeBW(); /* BBR estimates the queue was drained */
   }
 }
-void Bbr2CongestionController::updateProbeBwCyclePhase(
-    uint64_t ackedBytes,
-    uint64_t inflightBytesAtLargestAckedPacket,
-    uint64_t lostBytes) {
+
+void Bbr2CongestionController::updateProbeBwCyclePhase() {
   /* The core state machine logic for ProbeBW: */
-  if (!filledPipe_) {
+  if (!fullBwReached_) {
     return; /* only handling steady-state behavior here */
   }
-  adaptUpperBounds(ackedBytes, inflightBytesAtLargestAckedPacket, lostBytes);
+  adaptUpperBounds();
   if (!isProbeBwState(state_)) {
     return; /* only handling ProbeBW states here: */
   }
@@ -553,13 +630,13 @@ void Bbr2CongestionController::updateProbeBwCyclePhase(
       /* After one round of REFILL, start UP */
       if (roundStart_) {
         // Enable one reaction to loss per probe bw cycle.
-        bwProbeShouldHandleLoss_ = true;
+        canUpdateLongtermLossModel_ = true;
         startProbeBwUp();
       }
       break;
     case State::ProbeBw_Up:
-      if (hasElapsedInPhase(minRtt_) &&
-          conn_.lossState.inflightBytes > getTargetInflightWithGain(1.25)) {
+      if (checkTimeToGoDown()) {
+        canUpdateLongtermLossModel_ = false;
         startProbeBwDown();
       }
       break;
@@ -571,28 +648,22 @@ void Bbr2CongestionController::updateProbeBwCyclePhase(
   }
 }
 
-void Bbr2CongestionController::adaptUpperBounds(
-    uint64_t ackedBytes,
-    uint64_t inflightBytesAtLargestAckedPacket,
-    uint64_t lostBytes) {
+void Bbr2CongestionController::adaptUpperBounds() {
   /* Update BBR.inflight_hi and BBR.bw_hi. */
 
-  if (!checkInflightTooHigh(inflightBytesAtLargestAckedPacket, lostBytes)) {
-    if (!inflightHi_.has_value() || !bandwidthHi_.has_value()) {
+  if (!checkInflightTooHigh()) {
+    if (!inflightHi_.has_value()) {
       // No loss has occurred yet so these values are not set and do not need to
       // be raised.
       return;
     }
     /* There is loss but it's at safe levels. The limits are populated so we
      * update them */
-    if (inflightBytesAtLargestAckedPacket > *inflightHi_) {
-      inflightHi_ = inflightBytesAtLargestAckedPacket;
-    }
-    if (bandwidthLatest_ > *bandwidthHi_) {
-      bandwidthHi_ = bandwidthLatest_;
+    if (inflightBytesAtLastAckedPacket_ > *inflightHi_) {
+      inflightHi_ = inflightBytesAtLastAckedPacket_;
     }
     if (state_ == State::ProbeBw_Up) {
-      probeInflightHiUpward(ackedBytes);
+      probeInflightHiUpward();
     }
   }
 }
@@ -616,18 +687,27 @@ bool Bbr2CongestionController::checkTimeToCruise() {
   return false;
 }
 
+bool Bbr2CongestionController::checkTimeToGoDown() {
+  if (cwndLimitedInRound_ && inflightHi_.has_value() &&
+      getTargetInflightWithGain(1.25) >= inflightHi_.value()) {
+    resetFullBw();
+    fullBw_ = maxBwFilter_.GetBest();
+  } else if (fullBwNow_) {
+    return true;
+  }
+  return false;
+}
+
 bool Bbr2CongestionController::hasElapsedInPhase(
     std::chrono::microseconds interval) {
   return Clock::now() > probeBWCycleStart_ + interval;
 }
 
 // Was the loss percent too high for the last ack received?
-bool Bbr2CongestionController::checkInflightTooHigh(
-    uint64_t inflightBytesAtLargestAckedPacket,
-    uint64_t lostBytes) {
-  if (isInflightTooHigh(inflightBytesAtLargestAckedPacket, lostBytes)) {
-    if (bwProbeShouldHandleLoss_) {
-      handleInFlightTooHigh(inflightBytesAtLargestAckedPacket);
+bool Bbr2CongestionController::checkInflightTooHigh() {
+  if (isInflightTooHigh()) {
+    if (canUpdateLongtermLossModel_) {
+      handleInFlightTooHigh();
     }
     return true;
   } else {
@@ -635,24 +715,20 @@ bool Bbr2CongestionController::checkInflightTooHigh(
   }
 }
 
-bool Bbr2CongestionController::isInflightTooHigh(
-    uint64_t inflightBytesAtLargestAckedPacket,
-    uint64_t lostBytes) {
-  return static_cast<float>(lostBytes) >
-      static_cast<float>(inflightBytesAtLargestAckedPacket) * kLossThreshold;
+bool Bbr2CongestionController::isInflightTooHigh() {
+  // TODO: The comparison should ideally use the  bytes lost since the
+  // largestAckedPacket was sent, but we don't track that.
+  return static_cast<float>(lossBytesInRound_) >
+      static_cast<float>(inflightBytesAtLastAckedPacket_) * kLossThreshold;
 }
 
-void Bbr2CongestionController::handleInFlightTooHigh(
-    uint64_t inflightBytesAtLargestAckedPacket) {
-  bwProbeShouldHandleLoss_ = false;
-  // TODO: Should this be the app limited state of the largest acknowledged
-  // packet?
-  if (!isAppLimited()) {
+void Bbr2CongestionController::handleInFlightTooHigh() {
+  canUpdateLongtermLossModel_ = false;
+  if (!lastAckedPacketAppLimited_) {
     inflightHi_ = std::max(
-        inflightBytesAtLargestAckedPacket,
+        inflightBytesAtLastAckedPacket_,
         static_cast<uint64_t>(
             static_cast<float>(getTargetInflightWithGain()) * kBeta));
-    bandwidthHi_ = maxBwFilter_.GetBest();
   }
   if (state_ == State::ProbeBw_Up) {
     startProbeBwDown();
@@ -675,13 +751,13 @@ uint64_t Bbr2CongestionController::getTargetInflightWithHeadroom() const {
   }
 }
 
-void Bbr2CongestionController::probeInflightHiUpward(uint64_t ackedBytes) {
+void Bbr2CongestionController::probeInflightHiUpward() {
   if (!inflightHi_.has_value() || !cwndLimitedInRound_ ||
       cwndBytes_ < *inflightHi_) {
     return; /* no inflight_hi set or not fully using inflight_hi, so don't grow
                it */
   }
-  probeUpAcks_ += ackedBytes;
+  probeUpAcks_ += currentAckEvent_->ackedBytes;
   if (probeUpAcks_ >= probeUpCount_) {
     auto delta = probeUpAcks_ / probeUpCount_;
     probeUpAcks_ -= delta * probeUpCount_;
@@ -715,7 +791,7 @@ void Bbr2CongestionController::updateMinRtt() {
   }
 }
 
-void Bbr2CongestionController::checkProbeRtt(uint64_t ackedBytes) {
+void Bbr2CongestionController::checkProbeRtt() {
   if (state_ != State::ProbeRTT && probeRttExpired_ && !idleRestart_) {
     enterProbeRtt();
     saveCwnd();
@@ -725,13 +801,14 @@ void Bbr2CongestionController::checkProbeRtt(uint64_t ackedBytes) {
   if (state_ == State::ProbeRTT) {
     handleProbeRtt();
   }
-  if (ackedBytes > 0) {
+  if (currentAckEvent_->ackedBytes > 0) {
     idleRestart_ = false;
   }
 }
 
 void Bbr2CongestionController::enterProbeRtt() {
   state_ = State::ProbeRTT;
+  canUpdateLongtermLossModel_ = false;
   updatePacingAndCwndGain();
 }
 
@@ -755,11 +832,10 @@ void Bbr2CongestionController::handleProbeRtt() {
   }
 }
 
-void Bbr2CongestionController::advanceLatestDeliverySignals(
-    const AckEvent& ackEvent) {
+void Bbr2CongestionController::advanceLatestDeliverySignals() {
   if (lossRoundStart_) {
-    bandwidthLatest_ = getBandwidthSampleFromAck(ackEvent);
-    inflightLatest_ = bandwidthLatest_.units;
+    bandwidthLatest_ = currentBwSample_;
+    inflightLatest_ = currentAckMaxInflightBytes_;
   }
 }
 
@@ -768,6 +844,7 @@ uint64_t Bbr2CongestionController::getProbeRTTCwnd() {
       getBDPWithGain(kProbeRttCwndGain),
       quic::kMinCwndInMssForBbr * conn_.udpSendPacketLen);
 }
+
 void Bbr2CongestionController::boundCwndForProbeRTT() {
   if (state_ == State::ProbeRTT) {
     cwndBytes_ = std::min(cwndBytes_, getProbeRTTCwnd());
@@ -782,10 +859,6 @@ void Bbr2CongestionController::boundBwForModel() {
         !conn_.transportSettings.ccaConfig.ignoreLoss) {
       bandwidth_ = std::min(bandwidth_, *bandwidthLo_);
     }
-    if (bandwidthHi_.has_value() &&
-        !conn_.transportSettings.ccaConfig.ignoreInflightHi) {
-      bandwidth_ = std::min(bandwidth_, *bandwidthHi_);
-    }
   }
   if (conn_.qLogger && previousBw != bandwidth_) {
     conn_.qLogger->addBandwidthEstUpdate(bandwidth_.units, bandwidth_.interval);
@@ -806,7 +879,7 @@ uint64_t Bbr2CongestionController::addQuantizationBudget(uint64_t input) const {
 }
 
 void Bbr2CongestionController::saveCwnd() {
-  if (!inLossRecovery_ && state_ != State::ProbeRTT) {
+  if (!isInRecovery() && state_ != State::ProbeRTT) {
     previousCwndBytes_ = cwndBytes_;
   } else {
     previousCwndBytes_ = std::max(cwndBytes_, previousCwndBytes_);
@@ -834,8 +907,9 @@ void Bbr2CongestionController::enterProbeBW() {
 void Bbr2CongestionController::startRound() {
   nextRoundDelivered_ = conn_.lossState.totalBytesAcked;
 }
-void Bbr2CongestionController::updateRound(const AckEvent& ackEvent) {
-  auto pkt = ackEvent.getLargestNewlyAckedPacket();
+
+void Bbr2CongestionController::updateRound() {
+  auto pkt = currentAckEvent_->getLargestNewlyAckedPacket();
   if (pkt && pkt->lastAckedPacketInfo &&
       pkt->lastAckedPacketInfo->totalBytesAcked >= nextRoundDelivered_) {
     startRound();
@@ -864,10 +938,11 @@ void Bbr2CongestionController::startProbeBwDown() {
 
   // This is a new ProbeBW cycle. Advance the max bw filter if we're not app
   // limited
-  if (!isAppLimited()) {
+  if (!lastAckedPacketAppLimited_) {
     cycleCount_++;
   }
 }
+
 void Bbr2CongestionController::startProbeBwCruise() {
   state_ = State::ProbeBw_Cruise;
   updatePacingAndCwndGain();
@@ -881,11 +956,13 @@ void Bbr2CongestionController::startProbeBwRefill() {
   updatePacingAndCwndGain();
   startRound();
 }
+
 void Bbr2CongestionController::startProbeBwUp() {
   probeBWCycleStart_ = Clock::now();
   state_ = State::ProbeBw_Up;
   updatePacingAndCwndGain();
   startRound();
+  resetFullBw();
   raiseInflightHiSlope();
 }
 
@@ -906,10 +983,19 @@ bool Bbr2CongestionController::isProbeBwState(
       state == Bbr2CongestionController::State::ProbeBw_Up);
 }
 
-Bandwidth Bbr2CongestionController::getBandwidthSampleFromAck(
+bool Bbr2CongestionController::isProbingBandwidth(
+    const Bbr2CongestionController::State state) {
+  return (
+      state == Bbr2CongestionController::State::ProbeBw_Up ||
+      state == Bbr2CongestionController::State::ProbeBw_Refill ||
+      state == Bbr2CongestionController::State::Startup);
+}
+
+void Bbr2CongestionController::updateBandwidthSampleFromAck(
     const AckEvent& ackEvent) {
   auto ackTime = ackEvent.adjustedAckTime;
-  auto bwSample = Bandwidth();
+  currentBwSample_ = Bandwidth();
+  currentAckMaxInflightBytes_ = 0;
   for (auto const& ackedPacket : ackEvent.ackedPackets) {
     auto pkt = &ackedPacket;
     if (ackedPacket.outstandingPacketMetadata.encodedSize == 0) {
@@ -926,20 +1012,21 @@ Bandwidth Bbr2CongestionController::getBandwidthSampleFromAck(
     auto ackElapsed = ackTime - lastAckTime;
     auto interval = std::max(ackElapsed, sendElapsed);
     if (interval == 0us) {
-      return Bandwidth();
+      continue;
     }
     auto lastBytesDelivered =
         lastAckedPacket ? lastAckedPacket->totalBytesAcked : 0;
     auto bytesDelivered = ackEvent.totalBytesAcked - lastBytesDelivered;
+    currentAckMaxInflightBytes_ =
+        std::max(currentAckMaxInflightBytes_, bytesDelivered);
     Bandwidth bw(
         bytesDelivered,
         std::chrono::duration_cast<std::chrono::microseconds>(interval),
         pkt->isAppLimited || lastSentTime < appLimitedLastSendTime_);
-    if (bw > bwSample) {
-      bwSample = bw;
+    if (bw > currentBwSample_) {
+      currentBwSample_ = bw;
     }
   }
-  return bwSample;
 }
 
 bool Bbr2CongestionController::isRenoCoexistenceProbeTime() {
@@ -953,6 +1040,10 @@ bool Bbr2CongestionController::isRenoCoexistenceProbeTime() {
   return roundsSinceBwProbe_ >= roundsBeforeRenoProbe;
 }
 
+bool Bbr2CongestionController::isInRecovery() const {
+  return recoveryState_ != RecoveryState::NOT_RECOVERY;
+}
+
 Bbr2CongestionController::State Bbr2CongestionController::getState()
     const noexcept {
   return state_;
@@ -978,11 +1069,11 @@ void Bbr2CongestionController::updatePacingAndCwndGain() {
       break;
     case State::ProbeBw_Up:
       pacingGain_ = kProbeBwUpPacingGain;
-      cwndGain_ = kProbeBwUpDownCwndGain;
+      cwndGain_ = kProbeBwUpCwndGain;
       break;
     case State::ProbeBw_Down:
       pacingGain_ = kProbeBwDownPacingGain;
-      cwndGain_ = kProbeBwUpDownCwndGain;
+      cwndGain_ = kProbeBwDownCwndGain;
       break;
     case State::ProbeBw_Cruise:
     case State::ProbeBw_Refill:
