@@ -16,7 +16,7 @@ class DSRStreamFrameScheduler {
  public:
   explicit DSRStreamFrameScheduler(QuicServerConnectionState& conn);
 
-  FOLLY_NODISCARD bool hasPendingData() const;
+  [[nodiscard]] bool hasPendingData() const;
 
   struct SchedulingResult {
     bool writeSuccess{false};
@@ -31,7 +31,8 @@ class DSRStreamFrameScheduler {
   };
 
   // Write a single stream's data into builder.
-  SchedulingResult writeStream(DSRPacketBuilderBase& builder);
+  [[nodiscard]] folly::Expected<SchedulingResult, QuicError> writeStream(
+      DSRPacketBuilderBase& builder);
 
  private:
   void enrichInstruction(
@@ -42,8 +43,8 @@ class DSRStreamFrameScheduler {
       SchedulingResult,
       DSRPacketBuilderBase&,
       SendInstruction::Builder&,
-      const PriorityQueue&,
-      const PriorityQueue::LevelItr&,
+      const deprecated::PriorityQueue&,
+      const deprecated::PriorityQueue::LevelItr&,
       QuicStreamState&);
 
  private:
