@@ -49,9 +49,15 @@ class TestingQuicClientTransport : public QuicClientTransport {
       std::shared_ptr<ClientHandshakeFactory> handshakeFactory,
       size_t connIdSize = kDefaultConnectionIdSize,
       bool useConnectionEndWithErrorCallback = false)
-      : QuicClientTransport(
+      : QuicTransportBaseLite(
             evb,
             std::move(socket),
+            useConnectionEndWithErrorCallback),
+        QuicClientTransport(
+            evb,
+            nullptr /* Initialized through the QuicTransportBaseLite constructor
+                     */
+            ,
             std::move(handshakeFactory),
             connIdSize,
             useConnectionEndWithErrorCallback) {}
@@ -344,8 +350,10 @@ class FakeOneRttHandshakeLayer : public FizzClientHandshake {
 
   std::unique_ptr<Aead> oneRttWriteCipher_;
   std::unique_ptr<PacketNumberCipher> oneRttWriteHeaderCipher_;
+  Optional<std::string> alpn_{folly::none};
 
   FizzCryptoFactory cryptoFactory_;
+
   const CryptoFactory& getCryptoFactory() const override {
     return cryptoFactory_;
   }
@@ -354,22 +362,28 @@ class FakeOneRttHandshakeLayer : public FizzClientHandshake {
   bool isTLSResumed() const override {
     throw std::runtime_error("isTLSResumed not implemented");
   }
+
   EncryptionLevel getReadRecordLayerEncryptionLevel() override {
     throw std::runtime_error(
         "getReadRecordLayerEncryptionLevel not implemented");
   }
+
   const Optional<std::string>& getApplicationProtocol() const override {
-    throw std::runtime_error("getApplicationProtocol not implemented");
+    return alpn_;
   }
+
   void processSocketData(folly::IOBufQueue&) override {
     throw std::runtime_error("processSocketData not implemented");
   }
+
   bool matchEarlyParameters() override {
     throw std::runtime_error("matchEarlyParameters not implemented");
   }
+
   std::unique_ptr<Aead> buildAead(CipherKind, folly::ByteRange) override {
     return createNoOpAead();
   }
+
   std::unique_ptr<PacketNumberCipher> buildHeaderCipher(
       folly::ByteRange) override {
     throw std::runtime_error("buildHeaderCipher not implemented");
@@ -501,10 +515,16 @@ class QuicClientTransportTestBase : public virtual testing::Test {
 
     setupCryptoLayer();
     start();
-    client->getNonConstConn().streamManager->setMaxLocalBidirectionalStreams(
-        std::numeric_limits<uint32_t>::max());
-    client->getNonConstConn().streamManager->setMaxLocalUnidirectionalStreams(
-        std::numeric_limits<uint32_t>::max());
+    CHECK(!client->getNonConstConn()
+               .streamManager
+               ->setMaxLocalBidirectionalStreams(
+                   std::numeric_limits<uint32_t>::max())
+               .hasError());
+    CHECK(!client->getNonConstConn()
+               .streamManager
+               ->setMaxLocalUnidirectionalStreams(
+                   std::numeric_limits<uint32_t>::max())
+               .hasError());
   }
 
   void destroyTransport() {
