@@ -12,6 +12,7 @@
 #include <quic/logging/QLogger.h>
 
 #include <quic/state/StreamData.h>
+#include <algorithm>
 #include <limits>
 
 namespace quic {
@@ -38,9 +39,17 @@ Optional<uint64_t> calculateNewWindowUpdate(
       enoughTimeElapsed && !transportSettings.autotuneReceiveConnFlowControl) {
     return nextAdvertisedOffset;
   }
-  bool enoughWindowElapsed = (curAdvertisedOffset - curReadOffset) *
-          transportSettings.flowControlWindowFrequency <
-      windowSize;
+  // The logic here is that we want to send updates when we have read
+  // windowSize / flowControlWindowFrequency bytes.
+  auto remaining = curAdvertisedOffset - curReadOffset;
+  bool enoughWindowElapsed = [&]() {
+    if (remaining > windowSize) {
+      return false;
+    }
+    return (windowSize - remaining) *
+        transportSettings.flowControlWindowFrequency >
+        windowSize;
+  }();
   if (enoughWindowElapsed) {
     return nextAdvertisedOffset;
   }
@@ -48,21 +57,27 @@ Optional<uint64_t> calculateNewWindowUpdate(
 }
 
 template <typename T>
-inline void incrementWithOverFlowCheck(T& num, T diff) {
+[[nodiscard]] inline folly::Expected<folly::Unit, QuicError>
+incrementWithOverFlowCheck(T& num, T diff) {
   if (num > std::numeric_limits<T>::max() - diff) {
-    throw QuicInternalException(
-        "flow control state overflow", LocalErrorCode::INTERNAL_ERROR);
+    return folly::makeUnexpected(QuicError(
+        QuicErrorCode(LocalErrorCode::INTERNAL_ERROR),
+        "flow control state overflow"));
   }
   num += diff;
+  return folly::unit;
 }
 
 template <typename T>
-inline void decrementWithOverFlowCheck(T& num, T diff) {
+[[nodiscard]] inline folly::Expected<folly::Unit, QuicError>
+decrementWithOverFlowCheck(T& num, T diff) {
   if (num < std::numeric_limits<T>::min() + diff) {
-    throw QuicInternalException(
-        "flow control state overflow", LocalErrorCode::INTERNAL_ERROR);
+    return folly::makeUnexpected(QuicError(
+        QuicErrorCode(LocalErrorCode::INTERNAL_ERROR),
+        "flow control state overflow"));
   }
   num -= diff;
+  return folly::unit;
 }
 
 inline uint64_t calculateMaximumData(const QuicStreamState& stream) {
@@ -171,40 +186,57 @@ bool maybeSendStreamWindowUpdate(
   return false;
 }
 
-void updateFlowControlOnStreamData(
+folly::Expected<folly::Unit, QuicError> updateFlowControlOnStreamData(
     QuicStreamState& stream,
     uint64_t previousMaxOffsetObserved,
     uint64_t bufferEndOffset) {
   if (stream.flowControlState.advertisedMaxOffset < bufferEndOffset) {
-    throw QuicTransportException(
+    return folly::makeUnexpected(QuicError(
+        TransportErrorCode::FLOW_CONTROL_ERROR,
         folly::to<std::string>(
-            "Stream flow control violation on stream ", stream.id),
-        TransportErrorCode::FLOW_CONTROL_ERROR);
+            "Stream flow control violation on stream ", stream.id)));
   }
   auto curMaxOffsetObserved =
       std::max(previousMaxOffsetObserved, bufferEndOffset);
   auto& connFlowControlState = stream.conn.flowControlState;
   uint64_t connMaxObservedOffset = connFlowControlState.sumMaxObservedOffset;
-  incrementWithOverFlowCheck(
+  auto incrementResult = incrementWithOverFlowCheck(
       connMaxObservedOffset, curMaxOffsetObserved - previousMaxOffsetObserved);
-  if (connMaxObservedOffset > connFlowControlState.advertisedMaxOffset) {
-    throw QuicTransportException(
-        "Connection flow control violation",
-        TransportErrorCode::FLOW_CONTROL_ERROR);
+  if (incrementResult.hasError()) {
+    return incrementResult;
   }
-  incrementWithOverFlowCheck(
+  if (connMaxObservedOffset > connFlowControlState.advertisedMaxOffset) {
+    return folly::makeUnexpected(QuicError(
+        TransportErrorCode::FLOW_CONTROL_ERROR,
+        "Connection flow control violation"));
+  }
+  return incrementWithOverFlowCheck(
       connFlowControlState.sumMaxObservedOffset,
       curMaxOffsetObserved - previousMaxOffsetObserved);
 }
 
-void updateFlowControlOnRead(
+folly::Expected<folly::Unit, QuicError> updateFlowControlOnRead(
     QuicStreamState& stream,
     uint64_t lastReadOffset,
     TimePoint readTime) {
   DCHECK_GE(stream.currentReadOffset, lastReadOffset);
-  auto diff = stream.currentReadOffset - lastReadOffset;
-  incrementWithOverFlowCheck(
+  uint64_t diff = 0;
+  if (stream.reliableSizeFromPeer &&
+      stream.currentReadOffset >= *stream.reliableSizeFromPeer) {
+    CHECK(stream.finalReadOffset.hasValue())
+        << "We got a reset from the peer, but the finalReadOffset is not set.";
+    // We've read all reliable bytes, so we can advance the currentReadOffset
+    // to the final size.
+    diff = *stream.finalReadOffset - lastReadOffset;
+    stream.currentReadOffset = *stream.finalReadOffset;
+  } else {
+    diff = stream.currentReadOffset - lastReadOffset;
+  }
+  auto incrementResult = incrementWithOverFlowCheck(
       stream.conn.flowControlState.sumCurReadOffset, diff);
+  if (incrementResult.hasError()) {
+    return incrementResult;
+  }
   if (maybeSendConnWindowUpdate(stream.conn, readTime)) {
     VLOG(4) << "Read trigger conn window update "
             << " readOffset=" << stream.conn.flowControlState.sumCurReadOffset
@@ -217,13 +249,48 @@ void updateFlowControlOnRead(
             << " maxOffset=" << stream.flowControlState.advertisedMaxOffset
             << " window=" << stream.flowControlState.windowSize;
   }
+  return folly::unit;
 }
 
-void updateFlowControlOnWriteToSocket(
+folly::Expected<folly::Unit, QuicError> updateFlowControlOnReceiveReset(
+    QuicStreamState& stream,
+    TimePoint resetTime) {
+  CHECK(stream.reliableSizeFromPeer.hasValue())
+      << "updateFlowControlOnReceiveReset has been called, "
+      << "but reliableSizeFromPeer has not been set";
+  CHECK(stream.finalReadOffset.hasValue())
+      << "updateFlowControlOnReceiveReset has been called, "
+      << "but finalReadOffset has not been set";
+  if (stream.currentReadOffset >= *stream.reliableSizeFromPeer) {
+    // We only advance the currentReadOffset to the final size if the
+    // application has read all of the reliable bytes. We don't do this
+    // earlier because we'll buffer additional data that arrives.
+    auto diff = *stream.finalReadOffset - stream.currentReadOffset;
+    stream.currentReadOffset = *stream.finalReadOffset;
+    auto incrementResult = incrementWithOverFlowCheck(
+        stream.conn.flowControlState.sumCurReadOffset, diff);
+    if (incrementResult.hasError()) {
+      return incrementResult;
+    }
+    if (maybeSendConnWindowUpdate(stream.conn, resetTime)) {
+      VLOG(4) << "Reset trigger conn window update "
+              << " readOffset=" << stream.conn.flowControlState.sumCurReadOffset
+              << " maxOffset="
+              << stream.conn.flowControlState.advertisedMaxOffset
+              << " window=" << stream.conn.flowControlState.windowSize;
+    }
+  }
+  return folly::unit;
+}
+
+folly::Expected<folly::Unit, QuicError> updateFlowControlOnWriteToSocket(
     QuicStreamState& stream,
     uint64_t length) {
-  incrementWithOverFlowCheck(
+  auto incrementResult = incrementWithOverFlowCheck(
       stream.conn.flowControlState.sumCurWriteOffset, length);
+  if (incrementResult.hasError()) {
+    return incrementResult;
+  }
   DCHECK_GE(stream.conn.flowControlState.sumCurStreamBufferLen, length);
   stream.conn.flowControlState.sumCurStreamBufferLen -= length;
   if (stream.conn.flowControlState.sumCurWriteOffset ==
@@ -234,20 +301,44 @@ void updateFlowControlOnWriteToSocket(
     }
     QUIC_STATS(stream.conn.statsCallback, onConnFlowControlBlocked);
   }
+  return folly::unit;
 }
 
-void updateFlowControlOnWriteToStream(
+folly::Expected<folly::Unit, QuicError> updateFlowControlOnWriteToStream(
     QuicStreamState& stream,
     uint64_t length) {
-  incrementWithOverFlowCheck(
+  return incrementWithOverFlowCheck(
       stream.conn.flowControlState.sumCurStreamBufferLen, length);
 }
 
-void updateFlowControlOnResetStream(QuicStreamState& stream) {
-  decrementWithOverFlowCheck(
-      stream.conn.flowControlState.sumCurStreamBufferLen,
-      static_cast<uint64_t>(
-          stream.pendingWrites.chainLength() + stream.writeBufMeta.length));
+folly::Expected<folly::Unit, QuicError> updateFlowControlOnResetStream(
+    QuicStreamState& stream,
+    folly::Optional<uint64_t> reliableSize) {
+  uint64_t decrementAmount = 0;
+  if (reliableSize && *reliableSize > 0) {
+    // This is the amount of pending data that we are "throwing away"
+    if (stream.pendingWrites.chainLength() + stream.currentWriteOffset >
+        *reliableSize) {
+      // Non-DSR case
+      decrementAmount +=
+          (stream.pendingWrites.chainLength() + stream.currentWriteOffset -
+           std::max(*reliableSize, stream.currentWriteOffset));
+    }
+
+    if (stream.writeBufMeta.offset + stream.writeBufMeta.length >
+        *reliableSize) {
+      // DSR case
+      decrementAmount += stream.writeBufMeta.length +
+          stream.writeBufMeta.offset -
+          std::max<uint64_t>(*reliableSize, stream.writeBufMeta.offset);
+    }
+  } else {
+    decrementAmount = static_cast<uint64_t>(
+        stream.pendingWrites.chainLength() + stream.writeBufMeta.length);
+  }
+
+  return decrementWithOverFlowCheck(
+      stream.conn.flowControlState.sumCurStreamBufferLen, decrementAmount);
 }
 
 void maybeWriteBlockAfterAPIWrite(QuicStreamState& stream) {
@@ -274,7 +365,10 @@ void maybeWriteDataBlockedAfterSocketWrite(QuicConnectionStateBase& conn) {
 void maybeWriteBlockAfterSocketWrite(QuicStreamState& stream) {
   // Only write blocked when the flow control bytes are used up and there are
   // still pending data
-  if (stream.streamWriteError) {
+  if (stream.streamWriteError && !stream.reliableSizeToPeer) {
+    // Note that we don't want to return prematurely if we've sent a reliable
+    // reset to the peer, because there could still be data that we want to
+    // write on the stream and we might be blocked on flow control.
     return;
   }
   if (stream.finalWriteOffset && stream.hasSentFIN()) {
