@@ -26,6 +26,7 @@ class QuicFlowControlTest : public Test {
         conn_, conn_.nodeType, conn_.transportSettings);
     conn_.statsCallback = quicStats_.get();
   }
+
   std::unique_ptr<MockQuicStats> quicStats_;
   QuicConnectionStateBase conn_{QuicNodeType::Client};
 };
@@ -47,6 +48,24 @@ TEST_F(QuicFlowControlTest, MaybeSendConnWindowUpdate) {
   EXPECT_TRUE(conn_.pendingEvents.connWindowUpdate);
 }
 
+TEST_F(QuicFlowControlTest, MaybeSendConnWindowUpdateWindowByFour) {
+  conn_.transportSettings.flowControlWindowFrequency = 4;
+  conn_.flowControlState.windowSize = 256 * 1024;
+  conn_.flowControlState.advertisedMaxOffset = 256 * 1024;
+  conn_.flowControlState.sumCurReadOffset = 64 * 1024;
+
+  // Should not send window update
+  EXPECT_CALL(*quicStats_, onConnFlowControlUpdate()).Times(0);
+  maybeSendConnWindowUpdate(conn_, Clock::now());
+  EXPECT_FALSE(conn_.pendingEvents.connWindowUpdate);
+
+  conn_.flowControlState.sumCurReadOffset += 64 * 1024 + 1;
+  // Should send window update
+  EXPECT_CALL(*quicStats_, onConnFlowControlUpdate()).Times(1);
+  maybeSendConnWindowUpdate(conn_, Clock::now());
+  EXPECT_TRUE(conn_.pendingEvents.connWindowUpdate);
+}
+
 TEST_F(QuicFlowControlTest, MaybeSendConnWindowUpdateAndIncrease) {
   conn_.flowControlState.windowSize = 500;
   conn_.flowControlState.advertisedMaxOffset = 400;
@@ -349,6 +368,7 @@ TEST_F(QuicFlowControlTest, MaybeWriteBlockedAfterAPIWrite) {
   maybeWriteBlockAfterAPIWrite(stream);
   EXPECT_TRUE(conn_.streamManager->hasBlocked());
 }
+
 TEST_F(QuicFlowControlTest, UpdateFlowControlClearsStreamBlockedFlag) {
   StreamId id = 3;
   QuicStreamState stream(id, conn_);
@@ -636,8 +656,9 @@ TEST_F(QuicFlowControlTest, UpdateFlowControlOnStreamData) {
 
   auto data1 = buildRandomInputData(10);
   uint64_t buffer1EndOffset = 200 + data1->computeChainDataLength();
-  updateFlowControlOnStreamData(
+  auto result = updateFlowControlOnStreamData(
       stream, stream.maxOffsetObserved, buffer1EndOffset);
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(conn_.flowControlState.sumMaxObservedOffset, 560);
 }
 
@@ -654,8 +675,9 @@ TEST_F(QuicFlowControlTest, UpdateFlowControlOnStreamDataUnchangedOffset) {
   stream.flowControlState.advertisedMaxOffset = 250;
 
   uint64_t buffer1EndOffset = 100;
-  updateFlowControlOnStreamData(
+  auto result = updateFlowControlOnStreamData(
       stream, stream.maxOffsetObserved, buffer1EndOffset);
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.maxOffsetObserved, 200);
   EXPECT_EQ(conn_.flowControlState.sumMaxObservedOffset, 550);
 }
@@ -675,26 +697,29 @@ TEST_F(QuicFlowControlTest, UpdateBadFlowControlOnStreamData) {
   auto data1 = buildRandomInputData(100);
   uint64_t buffer1EndOffset = 200 + data1->computeChainDataLength();
   // Stream flow control violation
-  EXPECT_THROW(
-      updateFlowControlOnStreamData(
-          stream, stream.maxOffsetObserved, buffer1EndOffset),
-      QuicTransportException);
+  auto result1 = updateFlowControlOnStreamData(
+      stream, stream.maxOffsetObserved, buffer1EndOffset);
+  EXPECT_TRUE(result1.hasError());
+  EXPECT_NE(result1.error().code.asTransportErrorCode(), nullptr);
+  EXPECT_FALSE(result1.error().message.empty());
 
   stream.currentReadOffset = 200;
   // Connection flow control violation
-  EXPECT_THROW(
-      updateFlowControlOnStreamData(
-          stream, stream.maxOffsetObserved, buffer1EndOffset),
-      QuicTransportException);
+  auto result2 = updateFlowControlOnStreamData(
+      stream, stream.maxOffsetObserved, buffer1EndOffset);
+  EXPECT_TRUE(result2.hasError());
+  EXPECT_NE(result2.error().code.asTransportErrorCode(), nullptr);
+  EXPECT_FALSE(result2.error().message.empty());
 
   auto data2 = buildRandomInputData(50);
   uint64_t buffer2EndOffset = 200 + data2->computeChainDataLength();
-  EXPECT_NO_THROW(updateFlowControlOnStreamData(
-      stream, stream.maxOffsetObserved, buffer2EndOffset));
+  auto result3 = updateFlowControlOnStreamData(
+      stream, stream.maxOffsetObserved, buffer2EndOffset);
+  ASSERT_FALSE(result3.hasError());
   EXPECT_EQ(conn_.flowControlState.sumMaxObservedOffset, 600);
 }
 
-TEST_F(QuicFlowControlTest, UpdateFlowControlOnRead) {
+TEST_F(QuicFlowControlTest, UpdateFlowControlOnReadBasic) {
   auto qLogger = std::make_shared<FileQLogger>(VantagePoint::Client);
   conn_.qLogger = qLogger;
 
@@ -709,7 +734,8 @@ TEST_F(QuicFlowControlTest, UpdateFlowControlOnRead) {
   conn_.flowControlState.sumCurReadOffset = 100;
   EXPECT_CALL(*quicStats_, onConnFlowControlUpdate());
   EXPECT_CALL(*quicStats_, onStreamFlowControlUpdate());
-  updateFlowControlOnRead(stream, 100, Clock::now());
+  auto result = updateFlowControlOnRead(stream, 100, Clock::now());
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(conn_.flowControlState.sumCurReadOffset, 200);
 
   EXPECT_TRUE(conn_.streamManager->pendingWindowUpdate(stream.id));
@@ -723,6 +749,118 @@ TEST_F(QuicFlowControlTest, UpdateFlowControlOnRead) {
   EXPECT_EQ(event->update, getFlowControlEvent(700));
 }
 
+// We've received a reliable reset. Now, we're receiving additional data,
+// but we haven't yet received all of the reliable data.
+TEST_F(QuicFlowControlTest, UpdateFlowControlOnReadReliableReset1) {
+  StreamId id = 3;
+  QuicStreamState stream(id, conn_);
+  stream.reliableSizeFromPeer = 100;
+  stream.finalReadOffset = 150;
+  stream.currentReadOffset = 10;
+  stream.flowControlState.windowSize = 20;
+  stream.flowControlState.advertisedMaxOffset = 10000;
+
+  conn_.flowControlState.windowSize = 1000;
+  conn_.flowControlState.advertisedMaxOffset = 10000;
+  conn_.flowControlState.sumCurReadOffset = 40;
+
+  // Simulate the reading of 10 bytes
+  stream.currentReadOffset = 20;
+  auto result = updateFlowControlOnRead(stream, 10, Clock::now());
+  ASSERT_FALSE(result.hasError());
+  EXPECT_EQ(conn_.flowControlState.sumCurReadOffset, 50);
+  EXPECT_EQ(stream.currentReadOffset, 20);
+}
+
+// We've received a reliable reset. Now, we're receiving additional data,
+// and we've received all of the reliable data.
+TEST_F(QuicFlowControlTest, UpdateFlowControlOnReadReliableReset2) {
+  StreamId id = 3;
+  QuicStreamState stream(id, conn_);
+  stream.reliableSizeFromPeer = 100;
+  stream.finalReadOffset = 150;
+  stream.currentReadOffset = 10;
+  stream.flowControlState.windowSize = 20;
+  stream.flowControlState.advertisedMaxOffset = 10000;
+
+  conn_.flowControlState.windowSize = 1000;
+  conn_.flowControlState.advertisedMaxOffset = 10000;
+  conn_.flowControlState.sumCurReadOffset = 40;
+
+  // Simulate the reading of 90 bytes
+  stream.currentReadOffset = 100;
+  auto result = updateFlowControlOnRead(stream, 10, Clock::now());
+  ASSERT_FALSE(result.hasError());
+
+  EXPECT_EQ(conn_.flowControlState.sumCurReadOffset, 180);
+  EXPECT_EQ(stream.currentReadOffset, 150);
+}
+
+// We're receiving a reliable reset with a reliable size > the amount
+// of data we've read so far.
+TEST_F(QuicFlowControlTest, UpdateFlowControlOnReceiveReset1) {
+  StreamId id = 3;
+  QuicStreamState stream(id, conn_);
+  stream.currentReadOffset = 10;
+  stream.flowControlState.windowSize = 20;
+  stream.flowControlState.advertisedMaxOffset = 10000;
+
+  conn_.flowControlState.windowSize = 1000;
+  conn_.flowControlState.advertisedMaxOffset = 10000;
+  conn_.flowControlState.sumCurReadOffset = 40;
+
+  // Simulate the receiving of a reliable reset
+  stream.reliableSizeFromPeer = 100;
+  stream.finalReadOffset = 150;
+  auto result = updateFlowControlOnReceiveReset(stream, Clock::now());
+  ASSERT_FALSE(result.hasError());
+  EXPECT_EQ(conn_.flowControlState.sumCurReadOffset, 40);
+  EXPECT_EQ(stream.currentReadOffset, 10);
+}
+
+// We're receiving a reliable reset with a reliable size <= the amount
+// of data we've read so far.
+TEST_F(QuicFlowControlTest, UpdateFlowControlOnReceiveReset2) {
+  StreamId id = 3;
+  QuicStreamState stream(id, conn_);
+  stream.currentReadOffset = 10;
+  stream.flowControlState.windowSize = 20;
+  stream.flowControlState.advertisedMaxOffset = 10000;
+
+  conn_.flowControlState.windowSize = 1000;
+  conn_.flowControlState.advertisedMaxOffset = 10000;
+  conn_.flowControlState.sumCurReadOffset = 40;
+
+  // Simulate the receiving of a reliable reset
+  stream.reliableSizeFromPeer = 10;
+  stream.finalReadOffset = 150;
+  auto result = updateFlowControlOnReceiveReset(stream, Clock::now());
+  ASSERT_FALSE(result.hasError());
+  EXPECT_EQ(conn_.flowControlState.sumCurReadOffset, 180);
+  EXPECT_EQ(stream.currentReadOffset, 150);
+}
+
+// We're receiving a non-reliable reset
+TEST_F(QuicFlowControlTest, UpdateFlowControlOnReceiveReset3) {
+  StreamId id = 3;
+  QuicStreamState stream(id, conn_);
+  stream.currentReadOffset = 10;
+  stream.flowControlState.windowSize = 20;
+  stream.flowControlState.advertisedMaxOffset = 10000;
+
+  conn_.flowControlState.windowSize = 1000;
+  conn_.flowControlState.advertisedMaxOffset = 10000;
+  conn_.flowControlState.sumCurReadOffset = 40;
+
+  // Simulate the receiving of a non-reliable reset
+  stream.reliableSizeFromPeer = 0;
+  stream.finalReadOffset = 150;
+  auto result = updateFlowControlOnReceiveReset(stream, Clock::now());
+  ASSERT_FALSE(result.hasError());
+  EXPECT_EQ(conn_.flowControlState.sumCurReadOffset, 180);
+  EXPECT_EQ(stream.currentReadOffset, 150);
+}
+
 TEST_F(QuicFlowControlTest, UpdateFlowControlOnWrite) {
   StreamId id = 3;
   QuicStreamState stream(id, conn_);
@@ -731,10 +869,12 @@ TEST_F(QuicFlowControlTest, UpdateFlowControlOnWrite) {
 
   conn_.flowControlState.sumCurWriteOffset = 200;
   EXPECT_CALL(*quicStats_, onConnFlowControlBlocked()).Times(0);
-  updateFlowControlOnWriteToStream(stream, 100);
+  auto result1 = updateFlowControlOnWriteToStream(stream, 100);
+  ASSERT_FALSE(result1.hasError());
   EXPECT_EQ(conn_.flowControlState.sumCurStreamBufferLen, 100);
   EXPECT_CALL(*quicStats_, onConnFlowControlBlocked()).Times(0);
-  updateFlowControlOnWriteToSocket(stream, 100);
+  auto result2 = updateFlowControlOnWriteToSocket(stream, 100);
+  ASSERT_FALSE(result2.hasError());
   EXPECT_EQ(conn_.flowControlState.sumCurWriteOffset, 300);
   EXPECT_EQ(conn_.flowControlState.sumCurStreamBufferLen, 0);
 
@@ -742,11 +882,13 @@ TEST_F(QuicFlowControlTest, UpdateFlowControlOnWrite) {
 
   stream.currentWriteOffset = 300;
   EXPECT_CALL(*quicStats_, onConnFlowControlBlocked()).Times(0);
-  updateFlowControlOnWriteToStream(stream, 100);
+  auto result3 = updateFlowControlOnWriteToStream(stream, 100);
+  ASSERT_FALSE(result3.hasError());
   EXPECT_EQ(conn_.flowControlState.sumCurStreamBufferLen, 100);
 
   EXPECT_CALL(*quicStats_, onConnFlowControlBlocked()).Times(0);
-  updateFlowControlOnWriteToSocket(stream, 100);
+  auto result4 = updateFlowControlOnWriteToSocket(stream, 100);
+  ASSERT_FALSE(result4.hasError());
   EXPECT_EQ(conn_.flowControlState.sumCurStreamBufferLen, 0);
   EXPECT_EQ(conn_.flowControlState.sumCurWriteOffset, 400);
   EXPECT_FALSE(conn_.streamManager->flowControlUpdatedContains(id));
@@ -755,7 +897,8 @@ TEST_F(QuicFlowControlTest, UpdateFlowControlOnWrite) {
   conn_.flowControlState.sumCurStreamBufferLen = 100;
   stream.flowControlState.peerAdvertisedMaxOffset = 600;
   EXPECT_CALL(*quicStats_, onConnFlowControlBlocked()).Times(1);
-  updateFlowControlOnWriteToSocket(stream, 100);
+  auto result5 = updateFlowControlOnWriteToSocket(stream, 100);
+  ASSERT_FALSE(result5.hasError());
 }
 
 TEST_F(QuicFlowControlTest, UpdateFlowControlOnWriteToStream) {
@@ -766,13 +909,16 @@ TEST_F(QuicFlowControlTest, UpdateFlowControlOnWriteToStream) {
   conn_.flowControlState.sumCurStreamBufferLen = 100;
   stream.flowControlState.peerAdvertisedMaxOffset = 300;
 
-  updateFlowControlOnWriteToStream(stream, 100);
+  auto result1 = updateFlowControlOnWriteToStream(stream, 100);
+  ASSERT_FALSE(result1.hasError());
   EXPECT_EQ(conn_.flowControlState.sumCurStreamBufferLen, 200);
 
-  updateFlowControlOnWriteToSocket(stream, 150);
+  auto result2 = updateFlowControlOnWriteToSocket(stream, 150);
+  ASSERT_FALSE(result2.hasError());
   EXPECT_EQ(conn_.flowControlState.sumCurStreamBufferLen, 50);
 
-  updateFlowControlOnWriteToStream(stream, 100);
+  auto result3 = updateFlowControlOnWriteToStream(stream, 100);
+  ASSERT_FALSE(result3.hasError());
   EXPECT_EQ(conn_.flowControlState.sumCurStreamBufferLen, 150);
 }
 
@@ -830,7 +976,7 @@ TEST_F(QuicFlowControlTest, WritableList) {
 
   auto buf = IOBuf::create(100);
   buf->append(100);
-  writeDataToQuicStream(stream, std::move(buf), false);
+  ASSERT_FALSE(writeDataToQuicStream(stream, std::move(buf), false).hasError());
   conn_.streamManager->updateWritableStreams(stream);
   EXPECT_TRUE(writableContains(*conn_.streamManager, id));
 
@@ -840,7 +986,7 @@ TEST_F(QuicFlowControlTest, WritableList) {
   EXPECT_FALSE(writableContains(*conn_.streamManager, id));
 
   // Fin
-  writeDataToQuicStream(stream, nullptr, true);
+  ASSERT_FALSE(writeDataToQuicStream(stream, nullptr, true).hasError());
   stream.writeBuffer.move();
   ChainedByteRangeHead(std::move(stream.pendingWrites));
   stream.currentWriteOffset += 100;
@@ -990,4 +1136,114 @@ TEST_F(QuicFlowControlTest, StreamFlowControlWithBufMeta) {
   EXPECT_EQ(400, getSendStreamFlowControlBytesAPI(stream));
 }
 
+// This tests the non-DSR case where
+// currentWriteOffset < reliableSize < (currentWriteOffset +
+// pendingWrites.chainLength())
+TEST_F(QuicFlowControlTest, ReliableSizeNonDsrReset1) {
+  StreamId id = 0;
+  QuicStreamState stream(id, conn_);
+  stream.currentWriteOffset = 20;
+  auto inputData = buildRandomInputData(5);
+  stream.pendingWrites.append(inputData);
+  stream.writeBuffer.append(std::move(inputData));
+  stream.conn.flowControlState.sumCurStreamBufferLen = 5;
+
+  auto result = updateFlowControlOnResetStream(stream, 22);
+  ASSERT_FALSE(result.hasError());
+
+  // We threw away 3 bytes due to the reliable reset
+  EXPECT_EQ(stream.conn.flowControlState.sumCurStreamBufferLen, 2);
+}
+
+// This tests the non-DSR case where
+// reliableSize < currentWriteOffset < (currentWriteOffset +
+// pendingWrites.chainLength())
+TEST_F(QuicFlowControlTest, ReliableSizeNonDsrReset2) {
+  StreamId id = 0;
+  QuicStreamState stream(id, conn_);
+  stream.currentWriteOffset = 20;
+  auto inputData = buildRandomInputData(5);
+  stream.pendingWrites.append(inputData);
+  stream.writeBuffer.append(std::move(inputData));
+  stream.conn.flowControlState.sumCurStreamBufferLen = 5;
+
+  auto result = updateFlowControlOnResetStream(stream, 10);
+  ASSERT_FALSE(result.hasError());
+
+  // We threw away all 5 bytes due to the reliable reset
+  EXPECT_EQ(stream.conn.flowControlState.sumCurStreamBufferLen, 0);
+}
+
+// This tests the non-DSR case where
+// currentWriteOffset < (currentWriteOffset +
+// pendingWrites.chainLength()) < reliableSize
+TEST_F(QuicFlowControlTest, ReliableSizeNonDsrReset3) {
+  StreamId id = 0;
+  QuicStreamState stream(id, conn_);
+  stream.currentWriteOffset = 20;
+  auto inputData = buildRandomInputData(5);
+  stream.pendingWrites.append(inputData);
+  stream.writeBuffer.append(std::move(inputData));
+  stream.conn.flowControlState.sumCurStreamBufferLen = 5;
+
+  auto result = updateFlowControlOnResetStream(stream, 30);
+  ASSERT_FALSE(result.hasError());
+
+  // We didn't throw away any bytes after the reliable reset
+  EXPECT_EQ(stream.conn.flowControlState.sumCurStreamBufferLen, 5);
+}
+
+// This tests the DSR case where
+// writeBufMeta.offset < reliableSize < (writeBufMeta.offset +
+// writeBufMeta.length)
+TEST_F(QuicFlowControlTest, ReliableSizeDsrReset1) {
+  StreamId id = 0;
+  QuicStreamState stream(id, conn_);
+  stream.writeBufMeta.offset = 20;
+  stream.writeBufMeta.length = 5;
+
+  stream.conn.flowControlState.sumCurStreamBufferLen = 5;
+  auto result = updateFlowControlOnResetStream(stream, 22);
+  ASSERT_FALSE(result.hasError());
+
+  // We threw away 3 bytes due to the reliable reset
+  EXPECT_EQ(stream.conn.flowControlState.sumCurStreamBufferLen, 2);
+}
+
+// This tests the DSR case where
+// reliableSize < writeBufMeta.offset < (writeBufMeta.offset +
+// writeBufMeta.length)
+TEST_F(QuicFlowControlTest, ReliableSizeDsrReset2) {
+  StreamId id = 0;
+  QuicStreamState stream(id, conn_);
+  stream.writeBufMeta.offset = 20;
+  stream.writeBufMeta.length = 5;
+
+  stream.conn.flowControlState.sumCurStreamBufferLen = 5;
+
+  auto result = updateFlowControlOnResetStream(stream, 10);
+  ASSERT_FALSE(result.hasError());
+
+  // We threw away all 5 bytes due to the reliable reset
+  EXPECT_EQ(stream.conn.flowControlState.sumCurStreamBufferLen, 0);
+}
+
+// This tests the DSR case where
+// writeBufMeta.offset < (writeBufMeta.offset +
+// writeBufMeta.length) < reliableSize
+TEST_F(QuicFlowControlTest, ReliableSizeDsrReset3) {
+  StreamId id = 0;
+  QuicStreamState stream(id, conn_);
+  stream.writeBufMeta.offset = 20;
+  stream.writeBufMeta.length = 5;
+
+  stream.conn.flowControlState.sumCurStreamBufferLen = 5;
+
+  auto result = updateFlowControlOnResetStream(stream, 30);
+  ASSERT_FALSE(result.hasError());
+
+  // We didn't throw away any bytes after the reliable reset
+  EXPECT_EQ(stream.conn.flowControlState.sumCurStreamBufferLen, 5);
+}
+
 } // namespace quic::test
