@@ -9,7 +9,7 @@
 
 #include <fstream>
 
-#include <folly/json/json.h>
+#include <folly/json/json.h> // @manual=//folly:dynamic
 
 namespace quic {
 
@@ -21,6 +21,7 @@ void FileQLogger::setDcid(Optional<ConnectionId> connID) {
     }
   }
 }
+
 void FileQLogger::setScid(Optional<ConnectionId> connID) {
   if (connID.hasValue()) {
     scid = connID.value();
@@ -44,7 +45,8 @@ void FileQLogger::setupStream() {
     return;
   }
   if (compress_) {
-    compressionCodec_ = folly::io::getStreamCodec(folly::io::CodecType::GZIP);
+    compressionCodec_ =
+        folly::compression::getStreamCodec(folly::compression::CodecType::GZIP);
     compressionBuffer_ = folly::IOBuf::createCombined(kCompressionBufferSize);
   }
 
@@ -150,7 +152,9 @@ void FileQLogger::finishStream() {
       auto outputRange = folly::MutableByteRange(
           compressionBuffer_->writableData(), compressionBuffer_->capacity());
       ended = compressionCodec_->compressStream(
-          inputRange, outputRange, folly::io::StreamCodec::FlushOp::END);
+          inputRange,
+          outputRange,
+          folly::compression::StreamCodec::FlushOp::END);
       auto outputLen = compressionBuffer_->capacity() - outputRange.size();
       writer_->writeMessage(folly::StringPiece(
           (const char*)compressionBuffer_->data(), outputLen));
@@ -250,6 +254,13 @@ void FileQLogger::addTransportSummary(const TransportSummaryArgs& args) {
       args.usedZeroRtt,
       args.quicVersion,
       args.dsrPacketCount,
+      args.initialPacketsReceived,
+      args.uniqueInitialCryptoFramesReceived,
+      args.timeUntilLastInitialCryptoFrameReceived,
+      args.alpn,
+      args.namedGroup,
+      args.pskType,
+      args.echStatus,
       refTime));
 }
 
@@ -294,6 +305,7 @@ void FileQLogger::addAppUnlimitedUpdate() {
       std::chrono::duration_cast<std::chrono::microseconds>(
           std::chrono::steady_clock::now().time_since_epoch())));
 }
+
 void FileQLogger::addPacingMetricUpdate(
     uint64_t pacingBurstSizeIn,
     std::chrono::microseconds pacingIntervalIn) {
@@ -491,12 +503,11 @@ void FileQLogger::addPathValidationEvent(bool success) {
 
 void FileQLogger::addPriorityUpdate(
     quic::StreamId streamId,
-    uint8_t urgency,
-    bool incremental) {
+    PriorityQueue::PriorityLogFields priority) {
   auto refTime = std::chrono::duration_cast<std::chrono::microseconds>(
       std::chrono::steady_clock::now().time_since_epoch());
   handleEvent(std::make_unique<quic::QLogPriorityUpdateEvent>(
-      streamId, urgency, incremental, refTime));
+      streamId, std::move(priority), refTime));
 }
 
 void FileQLogger::addL4sWeightUpdate(
@@ -546,7 +557,8 @@ void FileQLogger::outputLogsToFile(const std::string& path, bool prettyJson) {
                            : folly::toJson(toDynamic());
     if (compress_) {
       try {
-        auto gzipCodec = folly::io::getCodec(folly::io::CodecType::GZIP);
+        auto gzipCodec =
+            folly::compression::getCodec(folly::compression::CodecType::GZIP);
         auto compressed = gzipCodec->compress(qLog);
         fileObj << compressed;
       } catch (std::invalid_argument& ex) {
