@@ -28,10 +28,14 @@ folly::dynamic NewTokenFrameLog::toDynamic() const {
 
 folly::dynamic RstStreamFrameLog::toDynamic() const {
   folly::dynamic d = folly::dynamic::object();
-  d["frame_type"] = toQlogString(FrameType::RST_STREAM);
+  d["frame_type"] = toQlogString(
+      reliableOffset ? FrameType::RST_STREAM_AT : FrameType::RST_STREAM);
   d["stream_id"] = streamId;
   d["error_code"] = errorCode;
   d["offset"] = offset;
+  if (reliableOffset) {
+    d["reliable_offset"] = reliableOffset.value();
+  }
   return d;
 }
 
@@ -220,11 +224,13 @@ folly::dynamic ReadAckFrameLog::toDynamic() const {
   }
   d["acked_ranges"] = ackRangeDynamic;
   d["frame_type"] = toQlogString(frameType);
-  if (frameType == FrameType::ACK_ECN) {
+  if (frameType == FrameType::ACK_EXTENDED || frameType == FrameType::ACK_ECN) {
     d["ecn_ect0"] = ecnECT0Count;
     d["ecn_ect1"] = ecnECT1Count;
     d["ecn_ce"] = ecnCECount;
-  } else if (frameType == FrameType::ACK_RECEIVE_TIMESTAMPS) {
+  }
+  if (frameType == FrameType::ACK_EXTENDED ||
+      frameType == FrameType::ACK_RECEIVE_TIMESTAMPS) {
     if (maybeLatestRecvdPacketTime.has_value()) {
       d["latest_recvd_packet_time"] =
           maybeLatestRecvdPacketTime.value().count();
@@ -261,11 +267,13 @@ folly::dynamic WriteAckFrameLog::toDynamic() const {
   }
   d["acked_ranges"] = ackRangeDynamic;
   d["frame_type"] = toQlogString(frameType);
-  if (frameType == FrameType::ACK_ECN) {
+  if (frameType == FrameType::ACK_EXTENDED || frameType == FrameType::ACK_ECN) {
     d["ecn_ect0"] = ecnECT0Count;
     d["ecn_ect1"] = ecnECT1Count;
     d["ecn_ce"] = ecnCECount;
-  } else if (frameType == FrameType::ACK_RECEIVE_TIMESTAMPS) {
+  }
+  if (frameType == FrameType::ACK_EXTENDED ||
+      frameType == FrameType::ACK_RECEIVE_TIMESTAMPS) {
     if (maybeLatestRecvdPacketTime.has_value()) {
       d["latest_recvd_packet_time"] =
           maybeLatestRecvdPacketTime.value().count();
@@ -423,6 +431,13 @@ QLogTransportSummaryEvent::QLogTransportSummaryEvent(
     bool usedZeroRttIn,
     QuicVersion quicVersionIn,
     uint64_t dsrPacketCountIn,
+    uint16_t initialPacketsReceivedIn,
+    uint16_t uniqueInitialCryptoFramesReceivedIn,
+    std::chrono::milliseconds timeUntilLastInitialCryptoFrameReceivedIn,
+    std::string alpnIn,
+    std::string namedGroupIn,
+    std::string pskTypeIn,
+    std::string echStatusIn,
     std::chrono::microseconds refTimeIn)
     : totalBytesSent{totalBytesSentIn},
       totalBytesRecvd{totalBytesRecvdIn},
@@ -442,7 +457,15 @@ QLogTransportSummaryEvent::QLogTransportSummaryEvent(
           finalPacketLossTimeReorderingThreshDividend},
       usedZeroRtt{usedZeroRttIn},
       quicVersion{quicVersionIn},
-      dsrPacketCount{dsrPacketCountIn} {
+      dsrPacketCount{dsrPacketCountIn},
+      initialPacketsReceived{initialPacketsReceivedIn},
+      uniqueInitialCryptoFramesReceived{uniqueInitialCryptoFramesReceivedIn},
+      timeUntilLastInitialCryptoFrameReceived{
+          timeUntilLastInitialCryptoFrameReceivedIn},
+      alpn{std::move(alpnIn)},
+      namedGroup{std::move(namedGroupIn)},
+      pskType{std::move(pskTypeIn)},
+      echStatus{std::move(echStatusIn)} {
   eventType = QLogEventType::TransportSummary;
   refTime = refTimeIn;
 }
@@ -479,6 +502,16 @@ folly::dynamic QLogTransportSummaryEvent::toDynamic() const {
       static_cast<std::underlying_type<decltype(quicVersion)>::type>(
           quicVersion);
   data["dsr_packet_count"] = dsrPacketCount;
+  data["initial_packets_received"] = initialPacketsReceived;
+  data["unique_initial_crypto_frames_received"] =
+      uniqueInitialCryptoFramesReceived;
+  data["time_until_last_initial_crypto_frame_received"] =
+      timeUntilLastInitialCryptoFrameReceived.count();
+  data["alpn"] = alpn;
+  data["named_group"] = namedGroup;
+  data["psk_type"] = pskType;
+  data["ech_status"] = echStatus;
+  ;
 
   d.push_back(std::move(data));
   return d;
@@ -937,10 +970,9 @@ folly::dynamic QLogPathValidationEvent::toDynamic() const {
 
 QLogPriorityUpdateEvent::QLogPriorityUpdateEvent(
     StreamId streamId,
-    uint8_t urgency,
-    bool incremental,
+    PriorityQueue::PriorityLogFields priority,
     std::chrono::microseconds refTimeIn)
-    : streamId_(streamId), urgency_(urgency), incremental_(incremental) {
+    : streamId_(streamId), priority_(std::move(priority)) {
   eventType = QLogEventType::PriorityUpdate;
   refTime = refTimeIn;
 }
@@ -951,8 +983,9 @@ folly::dynamic QLogPriorityUpdateEvent::toDynamic() const {
   folly::dynamic data = folly::dynamic::object();
 
   data["id"] = streamId_;
-  data["urgency"] = urgency_;
-  data["incremental"] = incremental_;
+  for (const auto& entry : priority_) {
+    data[entry.first] = entry.second;
+  }
   d.push_back(std::move(data));
   return d;
 }
