@@ -9,7 +9,7 @@
 
 #include <folly/FileUtil.h>
 #include <folly/Random.h>
-#include <folly/json/json.h>
+#include <folly/json/json.h> // @manual=//folly:dynamic
 #include <folly/portability/Filesystem.h>
 #include <gtest/gtest.h>
 #include <quic/common/test/TestUtils.h>
@@ -116,7 +116,15 @@ TEST_F(QLoggerTest, TransportSummaryEvent) {
        47,
        false,
        QuicVersion::MVFST,
-       37});
+       37,
+       /* numQuicInitialPacketReceived = */ 1,
+       /* numUniqueInitialCryptoFrameReceived = */ 1,
+       /* timeUntilLastInitialCryptoFrameReceived = */
+       std::chrono::milliseconds(1),
+       /* alpn = */ "blah",
+       /* namedGroup = */ "group",
+       /* pskType = */ "resumed",
+       /* echStatus = */ "accepted"});
 
   std::unique_ptr<QLogEvent> p = std::move(q.logs[0]);
   auto gotEvent = dynamic_cast<QLogTransportSummaryEvent*>(p.get());
@@ -138,6 +146,13 @@ TEST_F(QLoggerTest, TransportSummaryEvent) {
   EXPECT_EQ(gotEvent->finalPacketLossTimeReorderingThreshDividend, 47);
   EXPECT_EQ(gotEvent->usedZeroRtt, false);
   EXPECT_EQ(gotEvent->dsrPacketCount, 37);
+  EXPECT_EQ(gotEvent->initialPacketsReceived, 1);
+  EXPECT_EQ(gotEvent->uniqueInitialCryptoFramesReceived, 1);
+  EXPECT_EQ(gotEvent->timeUntilLastInitialCryptoFrameReceived.count(), 1);
+  EXPECT_EQ(gotEvent->alpn, "blah");
+  EXPECT_EQ(gotEvent->namedGroup, "group");
+  EXPECT_EQ(gotEvent->pskType, "resumed");
+  EXPECT_EQ(gotEvent->echStatus, "accepted");
 }
 
 TEST_F(QLoggerTest, CongestionMetricUpdateEvent) {
@@ -856,7 +871,14 @@ TEST_F(QLoggerTest, TransportSummaryFollyDynamic) {
        "final_packet_loss_time_reordering_threshold_dividend": 15,
        "used_zero_rtt": true,
        "quic_version": 4207849474,
-       "dsr_packet_count": 37
+       "dsr_packet_count": 37,
+       "initial_packets_received": 1,
+       "unique_initial_crypto_frames_received": 1,
+       "time_until_last_initial_crypto_frame_received": 1,
+       "alpn": "blah",
+       "named_group": "group",
+       "psk_type": "resumed",
+       "ech_status": "accepted"
      }
    ]
  ])");
@@ -880,7 +902,15 @@ TEST_F(QLoggerTest, TransportSummaryFollyDynamic) {
        15,
        true,
        QuicVersion::MVFST,
-       37});
+       37,
+       /* numQuicInitialPacketReceived = */ 1,
+       /* numUniqueInitialCryptoFrameReceived = */ 1,
+       /* timeUntilLastInitialCryptoFrameReceived = */
+       std::chrono::milliseconds(1),
+       /* alpn = */ "blah",
+       /* namedGroup = */ "group",
+       /* pskType = */ "resumed",
+       /* echStatus = */ "accepted"});
   folly::dynamic gotDynamic = q.toDynamic();
   gotDynamic["traces"][0]["events"][0][0] = "0"; // hardcode reference time
   folly::dynamic gotEvents = gotDynamic["traces"][0]["events"];
@@ -1543,8 +1573,9 @@ TEST_F(QLoggerTest, CompressedStream) {
   auto success = folly::readFile(outputPath.c_str(), compressedData);
   ASSERT_TRUE(success);
 
-  std::string str = folly::io::getCodec(folly::io::CodecType::GZIP)
-                        ->uncompress(compressedData);
+  std::string str =
+      folly::compression::getCodec(folly::compression::CodecType::GZIP)
+          ->uncompress(compressedData);
   folly::dynamic parsed = folly::parseJson(str);
 
   parsed["traces"][0]["events"][0][0] = "31"; // hardcode reference time
@@ -1587,8 +1618,9 @@ TEST_F(QLoggerTest, CompressedNonStream) {
   auto success = folly::readFile(outputPath.c_str(), compressedData);
   ASSERT_TRUE(success);
 
-  std::string str = folly::io::getCodec(folly::io::CodecType::GZIP)
-                        ->uncompress(compressedData);
+  std::string str =
+      folly::compression::getCodec(folly::compression::CodecType::GZIP)
+          ->uncompress(compressedData);
   folly::dynamic parsed = folly::parseJson(str);
 
   parsed["traces"][0]["events"][0][0] = "31"; // hardcode reference time
