@@ -32,9 +32,7 @@ class QuicServer : public QuicServerWorker::WorkerCallback,
                    public std::enable_shared_from_this<QuicServer> {
  public:
   using TransportSettingsOverrideFn =
-      std::function<Optional<quic::TransportSettings>(
-          const quic::TransportSettings&,
-          const folly::IPAddress&)>;
+      std::function<void(quic::TransportSettings&, const folly::IPAddress&)>;
 
   static std::shared_ptr<QuicServer> createQuicServer(
       TransportSettings transportSettings = TransportSettings()) {
@@ -379,6 +377,7 @@ class QuicServer : public QuicServerWorker::WorkerCallback,
     std::unique_ptr<folly::EventBaseBackendBase> (*factory)();
     bool supportsRecvmsgMultishot = false;
   };
+
   static EventBaseBackendDetails getEventBaseBackendDetails();
 
   // helper function to initialize workers
@@ -452,13 +451,16 @@ class QuicServer : public QuicServerWorker::WorkerCallback,
   // address that the server is bound to
   folly::SocketAddress boundAddress_;
   folly::SocketOptionMap socketOptions_;
+
   // Rate limits
   struct RateLimit {
     RateLimit(std::function<uint64_t()> c, std::chrono::seconds w)
         : count(std::move(c)), window(w) {}
+
     std::function<uint64_t()> count;
     std::chrono::seconds window;
   };
+
   Optional<RateLimit> rateLimit_;
 
   std::function<int()> unfinishedHandshakeLimitFn_{[]() { return 1048576; }};
