@@ -9,16 +9,15 @@
 #include <quic/congestion_control/ServerCongestionControllerFactory.h>
 #include <quic/dsr/frontend/WriteFunctions.h>
 #include <quic/fizz/server/handshake/FizzServerQuicHandshakeContext.h>
+#include <quic/priority/HTTPPriorityQueue.h>
 #include <quic/server/QuicServerTransport.h>
 #include <quic/server/handshake/AppToken.h>
 #include <quic/server/handshake/DefaultAppTokenValidator.h>
-#include <quic/server/handshake/StatelessResetGenerator.h>
 #include <quic/state/QuicStreamUtilities.h>
 #include <quic/state/TransportSettingsFunctions.h>
 
 #include <quic/common/Optional.h>
 #include <quic/common/TransportKnobs.h>
-#include <algorithm>
 #include <chrono>
 #include <memory>
 #include <stdexcept>
@@ -51,10 +50,16 @@ QuicServerTransport::QuicServerTransport(
     std::shared_ptr<const fizz::server::FizzServerContext> ctx,
     std::unique_ptr<CryptoFactory> cryptoFactory,
     bool useConnectionEndWithErrorCallback)
-    : QuicTransportBase(
-          std::move(evb),
+    : QuicTransportBaseLite(
+          evb,
           std::move(sock),
           useConnectionEndWithErrorCallback),
+      QuicTransportBase(
+          std::move(evb),
+          nullptr /* Initialized through the QuicTransportBaseLite constructor
+                   */
+          ,
+          useConnectionEndWithErrorCallback),
       ctx_(std::move(ctx)),
       wrappedObserverContainer_(this) {
   auto tempConn = std::make_unique<QuicServerConnectionState>(
@@ -150,7 +155,7 @@ void QuicServerTransport::setServerConnectionIdRejector(
   }
 }
 
-void QuicServerTransport::onReadData(
+folly::Expected<folly::Unit, QuicError> QuicServerTransport::onReadData(
     const folly::SocketAddress& peer,
     ReceivedUdpPacket&& udpPacket) {
   ServerEvents::ReadData readData;
@@ -164,7 +169,7 @@ void QuicServerTransport::onReadData(
   processPendingData(true);
 
   if (closeState_ == CloseState::CLOSED) {
-    return;
+    return folly::unit;
   }
   if (!notifiedRouting_ && routingCb_ && conn_->serverConnectionId) {
     notifiedRouting_ = true;
@@ -185,7 +190,10 @@ void QuicServerTransport::onReadData(
   if (serverConn_->transportSettings.enableWritableBytesLimit &&
       serverConn_->numProbesWritableBytesLimited &&
       prevWritableBytes < curWritableBytes) {
-    onPTOAlarm(*serverConn_);
+    auto ptoAlarmResult = onPTOAlarm(*serverConn_);
+    if (ptoAlarmResult.hasError()) {
+      return ptoAlarmResult;
+    }
     serverConn_->numProbesWritableBytesLimited = 0;
   }
 
@@ -195,7 +203,8 @@ void QuicServerTransport::onReadData(
   maybeNotifyConnectionIdRetired();
   maybeIssueConnectionIds();
   maybeNotifyTransportReady();
-  maybeUpdateCongestionControllerFromTicket();
+
+  return folly::unit;
 }
 
 void QuicServerTransport::accept() {
@@ -208,9 +217,9 @@ void QuicServerTransport::accept() {
       std::make_unique<DefaultAppTokenValidator>(serverConn_));
 }
 
-void QuicServerTransport::writeData() {
+folly::Expected<folly::Unit, QuicError> QuicServerTransport::writeData() {
   if (!conn_->clientConnectionId || !conn_->serverConnectionId) {
-    return;
+    return folly::unit;
   }
   auto version = conn_->version.value_or(*(conn_->originalVersion));
   const ConnectionId& srcConnId = *conn_->serverConnectionId;
@@ -219,12 +228,12 @@ void QuicServerTransport::writeData() {
     if (conn_->peerConnectionError &&
         hasReceivedUdpPacketsAtLastCloseSent(*conn_)) {
       // The peer sent us an error, we are in draining state now.
-      return;
+      return folly::unit;
     }
     if (hasReceivedUdpPacketsAtLastCloseSent(*conn_) &&
         hasNotReceivedNewPacketsSinceLastCloseSent(*conn_)) {
       // We did not receive any new packets, do not sent a new close frame.
-      return;
+      return folly::unit;
     }
     updateLargestReceivedUdpPacketsAtLastCloseSent(*conn_);
     if (conn_->oneRttWriteCipher) {
@@ -263,7 +272,7 @@ void QuicServerTransport::writeData() {
           *conn_->initialHeaderCipher,
           version);
     }
-    return;
+    return folly::unit;
   }
   uint64_t packetLimit =
       (isConnectionPaced(*conn_)
@@ -277,26 +286,33 @@ void QuicServerTransport::writeData() {
   };
   if (conn_->initialWriteCipher) {
     auto res = handleInitialWriteDataCommon(srcConnId, destConnId, packetLimit);
-    packetLimit -= res.packetsWritten;
-    serverConn_->numHandshakeBytesSent += res.bytesWritten;
+    if (res.hasError()) {
+      return folly::makeUnexpected(res.error());
+    }
+    packetLimit -= res->packetsWritten;
+    serverConn_->numHandshakeBytesSent += res->bytesWritten;
     if (!packetLimit && !conn_->pendingEvents.anyProbePackets()) {
-      return;
+      return folly::unit;
     }
   }
   if (conn_->handshakeWriteCipher) {
     auto res =
         handleHandshakeWriteDataCommon(srcConnId, destConnId, packetLimit);
-    packetLimit -= res.packetsWritten;
-    serverConn_->numHandshakeBytesSent += res.bytesWritten;
+    if (res.hasError()) {
+      return folly::makeUnexpected(res.error());
+    }
+    packetLimit -= res->packetsWritten;
+    serverConn_->numHandshakeBytesSent += res->bytesWritten;
     if (!packetLimit && !conn_->pendingEvents.anyProbePackets()) {
-      return;
+      return folly::unit;
     }
   }
   if (conn_->oneRttWriteCipher) {
     CHECK(conn_->oneRttWriteHeaderCipher);
     auto writeLoopBeginTime = Clock::now();
-    auto nonDsrPath = [&](auto limit) {
-      return writeQuicDataToSocket(
+    auto nonDsrPath =
+        [&](auto limit) -> folly::Expected<WriteQuicDataResult, QuicError> {
+      auto result = writeQuicDataToSocket(
           *socket_,
           *conn_,
           srcConnId /* src */,
@@ -306,20 +322,28 @@ void QuicServerTransport::writeData() {
           version,
           limit,
           writeLoopBeginTime);
+      if (result.hasError()) {
+        return folly::makeUnexpected(result.error());
+      }
+      return result.value();
     };
-    auto dsrPath = [&](auto limit) {
+    auto dsrPath =
+        [&](auto limit) -> folly::Expected<WriteQuicDataResult, QuicError> {
       auto bytesBefore = conn_->lossState.totalBytesSent;
       // The DSR path can't write probes.
       // This is packetsWritte, probesWritten, bytesWritten.
-      return WriteQuicDataResult{
-          writePacketizationRequest(
-              *serverConn_,
-              destConnId,
-              limit,
-              *conn_->oneRttWriteCipher,
-              writeLoopBeginTime),
-          0,
-          conn_->lossState.totalBytesSent - bytesBefore};
+      auto dsrResult = writePacketizationRequest(
+          *serverConn_,
+          destConnId,
+          limit,
+          *conn_->oneRttWriteCipher,
+          writeLoopBeginTime);
+      if (dsrResult.hasError()) {
+        return folly::makeUnexpected(dsrResult.error());
+      }
+      auto result = WriteQuicDataResult{
+          dsrResult.value(), 0, conn_->lossState.totalBytesSent - bytesBefore};
+      return result;
     };
     // We need a while loop because both paths write streams from the same
     // queue, which can result in empty writes.
@@ -328,14 +352,20 @@ void QuicServerTransport::writeData() {
       // Give the non-DSR path a chance first for things like ACKs and flow
       // control.
       auto written = nonDsrPath(packetLimit);
+      if (written.hasError()) {
+        return folly::makeUnexpected(written.error());
+      }
       // For both paths we only consider full packets against the packet
       // limit. While this is slightly more aggressive than the intended
       // packet limit it also helps ensure that small packets don't cause
       // us to underutilize the link when mixing between DSR and non-DSR.
-      packetLimit -= written.bytesWritten / conn_->udpSendPacketLen;
+      packetLimit -= written->bytesWritten / conn_->udpSendPacketLen;
       if (packetLimit && congestionControlWritableBytes(*serverConn_)) {
-        written = dsrPath(packetLimit);
-        packetLimit -= written.bytesWritten / conn_->udpSendPacketLen;
+        auto dsrWritten = dsrPath(packetLimit);
+        if (dsrWritten.hasError()) {
+          return folly::makeUnexpected(dsrWritten.error());
+        }
+        packetLimit -= dsrWritten->bytesWritten / conn_->udpSendPacketLen;
       }
       if (totalSentBefore == conn_->lossState.totalBytesSent) {
         // We haven't written anything with either path, so we're done.
@@ -343,6 +373,7 @@ void QuicServerTransport::writeData() {
       }
     }
   }
+  return folly::unit;
 }
 
 void QuicServerTransport::closeTransport() {
@@ -421,7 +452,11 @@ void QuicServerTransport::onCryptoEventAvailable() noexcept {
     maybeNotifyConnectionIdBound();
     maybeNotifyHandshakeFinished();
     maybeIssueConnectionIds();
-    writeSocketData();
+    auto writeResult = writeSocketData();
+    if (writeResult.hasError()) {
+      closeImpl(writeResult.error());
+      return;
+    }
     maybeNotifyTransportReady();
   } catch (const QuicTransportException& ex) {
     VLOG(4) << "onCryptoEventAvailable() error " << ex.what() << " " << *this;
@@ -476,13 +511,12 @@ void QuicServerTransport::processPendingData(bool async) {
     VLOG_IF(10, !pendingData->empty())
         << "Processing pending data size=" << pendingData->size() << " "
         << *this;
-    auto func = [pendingData = std::move(pendingData)](auto self) {
-      auto serverPtr = static_cast<QuicServerTransport*>(self.get());
+    auto func = [pendingData = std::move(pendingData), this](auto) {
       for (auto& pendingPacket : *pendingData) {
-        serverPtr->onNetworkData(
+        onNetworkData(
             pendingPacket.peer,
             NetworkData(std::move(pendingPacket.udpPacket)));
-        if (serverPtr->closeState_ == CloseState::CLOSED) {
+        if (closeState_ == CloseState::CLOSED) {
           // The pending data could potentially contain a connection close, or
           // the app could have triggered a connection close with an error. It
           // is not useful to continue the handshake.
@@ -509,22 +543,21 @@ bool QuicServerTransport::shouldWriteNewSessionTicket() {
   }
   // Conditions for writing more session tickets after the first one:
   // 1. includeCwndHintsInSessionTicket transport setting is set
-  // 2. The current congestion window is either smaller than or more than twice
+  // 2. The current BDP is either smaller than or more than twice
   // the last one we sent in a session ticket
   // 3. We haven't sent any session ticket in the last
   // kMinIntervalBetweenSessionTickets
 
-  if (serverConn_->congestionController &&
-      conn_->transportSettings.includeCwndHintsInSessionTicket &&
+  if (conn_->transportSettings.includeCwndHintsInSessionTicket &&
+      conn_->congestionController &&
       Clock::now() - newSessionTicketWrittenTimestamp_.value() >
           kMinIntervalBetweenSessionTickets) {
-    bool cwndChangedSinceLastHint =
+    const auto& targetBDP = conn_->congestionController->getBDP();
+    bool bdpChangedSinceLastHint =
         !newSessionTicketWrittenCwndHint_.has_value() ||
-        conn_->congestionController->getCongestionWindow() / 2 >
-            *newSessionTicketWrittenCwndHint_ ||
-        conn_->congestionController->getCongestionWindow() <
-            *newSessionTicketWrittenCwndHint_;
-    if (cwndChangedSinceLastHint) {
+        targetBDP / 2 > *newSessionTicketWrittenCwndHint_ ||
+        targetBDP < *newSessionTicketWrittenCwndHint_;
+    if (bdpChangedSinceLastHint) {
       return true;
     }
   }
@@ -541,9 +574,9 @@ void QuicServerTransport::maybeWriteNewSessionTicket() {
     Optional<uint64_t> cwndHint = none;
     if (conn_->transportSettings.includeCwndHintsInSessionTicket &&
         conn_->congestionController) {
-      VLOG(7) << "Writing a new session ticket with cwnd="
-              << conn_->congestionController->getCongestionWindow();
-      cwndHint = conn_->congestionController->getCongestionWindow();
+      const auto& bdp = conn_->congestionController->getBDP();
+      VLOG(7) << "Writing a new session ticket with cwnd hint=" << bdp;
+      cwndHint = bdp;
       newSessionTicketWrittenCwndHint_ = cwndHint;
     }
     AppToken appToken;
@@ -558,6 +591,7 @@ void QuicServerTransport::maybeWriteNewSessionTicket() {
         conn_->transportSettings.advertisedInitialUniStreamFlowControlWindow,
         conn_->transportSettings.advertisedInitialMaxStreamsBidi,
         conn_->transportSettings.advertisedInitialMaxStreamsUni,
+        conn_->transportSettings.advertisedExtendedAckFeatures,
         cwndHint);
     appToken.sourceAddresses = serverConn_->tokenSourceAddresses;
     appToken.version = conn_->version.value();
@@ -655,43 +689,6 @@ void QuicServerTransport::maybeNotifyTransportReady() {
   }
 }
 
-namespace {
-uint64_t determineCwndFromHint(const TransportSettings& tp, uint64_t cwndHint) {
-  if (cwndHint <= tp.cwndModerateJumpstart) {
-    // Weak Connectivity History
-    return tp.cwndWeakJumpstart;
-  } else if (cwndHint <= tp.cwndStrongJumpstart) {
-    // Moderate Connectivity History
-    return tp.cwndModerateJumpstart;
-  } else {
-    // Strong Connectivity History
-    return tp.cwndStrongJumpstart;
-  }
-}
-} // namespace
-
-void QuicServerTransport::maybeUpdateCongestionControllerFromTicket() {
-  if (serverConn_->transportSettings.useCwndHintsInSessionTicket &&
-      serverConn_->maybeCwndHintBytes.has_value() &&
-      serverConn_->lossState.maybeLrtt.has_value()) {
-    auto newCwnd = determineCwndFromHint(
-        serverConn_->transportSettings,
-        serverConn_->maybeCwndHintBytes.value());
-    serverConn_->maybeCwndHintBytes.reset();
-    if (newCwnd > conn_->congestionController->getCongestionWindow()) {
-      conn_->transportSettings.initCwndInMss =
-          newCwnd / conn_->udpSendPacketLen;
-      if (conn_->pacer) {
-        conn_->pacer.get()->refreshPacingRate(
-            newCwnd, serverConn_->lossState.lrtt);
-      }
-      conn_->congestionController =
-          conn_->congestionControllerFactory.get()->makeCongestionController(
-              *conn_, conn_->congestionController.get()->type());
-    }
-  }
-}
-
 void QuicServerTransport::registerTransportKnobParamHandler(
     uint64_t paramId,
     std::function<void(QuicServerTransport*, TransportKnobParam::Val)>&&
@@ -933,6 +930,18 @@ void QuicServerTransport::registerAllTransportKnobParamHandlers() {
             "SHORT_HEADER_PADDING_KNOB KnobParam received, setting paddingModulo={}",
             val);
       });
+  registerTransportKnobParamHandler(
+      static_cast<uint64_t>(
+          TransportKnobParamId::FIXED_SHORT_HEADER_PADDING_KNOB),
+      [](QuicServerTransport* serverTransport, TransportKnobParam::Val value) {
+        CHECK(serverTransport);
+        auto val = std::get<uint64_t>(value);
+        serverTransport->serverConn_->transportSettings
+            .fixedShortHeaderPadding = val;
+        VLOG(3) << fmt::format(
+            "FIXED_SHORT_HEADER_PADDING_KNOB KnobParam received, setting fixedShortHeaderPadding={}",
+            val);
+      });
 
   registerTransportKnobParamHandler(
       static_cast<uint64_t>(TransportKnobParamId::ADAPTIVE_LOSS_DETECTION),
@@ -1059,7 +1068,7 @@ void QuicServerTransport::registerAllTransportKnobParamHandlers() {
         } catch (std::exception&) {
           parseSuccess = false;
         }
-        if (!parseSuccess) {
+        if (!parseSuccess || level > 7) {
           auto errMsg = fmt::format(
               "Received invalid KnobParam for DEFAULT_STREAM_PRIORITY: {}",
               val);
@@ -1067,7 +1076,7 @@ void QuicServerTransport::registerAllTransportKnobParamHandlers() {
           throw std::runtime_error(errMsg);
         }
         serverConn->transportSettings.defaultPriority =
-            Priority(level, incremental);
+            HTTPPriorityQueue::Priority(level, incremental);
         VLOG(3) << "DEFAULT_STREAM_PRIORITY KnobParam received: " << val;
       });
   registerTransportKnobParamHandler(
@@ -1086,7 +1095,8 @@ void QuicServerTransport::registerAllTransportKnobParamHandlers() {
         auto val = std::get<uint64_t>(value);
         auto serverConn = serverTransport->serverConn_;
         serverConn->transportSettings.priorityQueueWritesPerStream = val;
-        serverConn->streamManager->writeQueue().setMaxNextsPerStream(val);
+        serverConn->streamManager->setWriteQueueMaxNextsPerStream(
+            serverConn->transportSettings.priorityQueueWritesPerStream);
         VLOG(3) << "WRITES_PER_STREAM KnobParam received: " << val;
       });
   registerTransportKnobParamHandler(
@@ -1164,6 +1174,29 @@ void QuicServerTransport::registerAllTransportKnobParamHandlers() {
         VLOG(3) << "INFLIGHT_REORDERING_THRESHOLD KnobParam received: "
                 << inflightReorderingThreshold;
       });
+  registerTransportKnobParamHandler(
+      static_cast<uint64_t>(TransportKnobParamId::PACER_MIN_BURST_PACKETS),
+      [](QuicServerTransport* serverTransport, TransportKnobParam::Val value) {
+        CHECK(serverTransport);
+        auto val = std::get<uint64_t>(value);
+        auto serverConn = serverTransport->serverConn_;
+        serverConn->transportSettings.minBurstPackets = val;
+        VLOG(3) << "PACER_MIN_BURST_PACKETS KnobParam received: " << val;
+      });
+  registerTransportKnobParamHandler(
+      static_cast<uint64_t>(TransportKnobParamId::MAX_BATCH_PACKETS),
+      [](QuicServerTransport* serverTransport, TransportKnobParam::Val value) {
+        CHECK(serverTransport);
+        auto val = std::get<uint64_t>(value);
+        auto serverConn = serverTransport->serverConn_;
+        serverConn->transportSettings.writeConnectionDataPacketsLimit =
+            val <= kMaxWriteConnectionDataPacketLimit
+            ? val
+            : kMaxWriteConnectionDataPacketLimit;
+        serverConn->transportSettings.maxBatchSize =
+            val <= kQuicMaxBatchSizeLimit ? val : kQuicMaxBatchSizeLimit;
+        VLOG(3) << "MAX_BATCH_PACKETS KnobParam received: " << val;
+      });
 }
 
 QuicConnectionStats QuicServerTransport::getConnectionsStats() const {
@@ -1192,7 +1225,8 @@ QuicSocket::WriteResult QuicServerTransport::writeBufMeta(
     if (!conn_->streamManager->streamExists(id)) {
       return folly::makeUnexpected(LocalErrorCode::STREAM_NOT_EXISTS);
     }
-    auto stream = CHECK_NOTNULL(conn_->streamManager->getStream(id));
+    auto stream =
+        CHECK_NOTNULL(conn_->streamManager->getStream(id).value_or(nullptr));
     if (!stream->writable()) {
       return folly::makeUnexpected(LocalErrorCode::STREAM_CLOSED);
     }
@@ -1217,7 +1251,12 @@ QuicSocket::WriteResult QuicServerTransport::writeBufMeta(
       wasAppLimitedOrIdle = conn_->congestionController->isAppLimited();
       wasAppLimitedOrIdle |= conn_->streamManager->isAppIdle();
     }
-    writeBufMetaToQuicStream(*stream, data, eof);
+    auto writeResult = writeBufMetaToQuicStream(*stream, data, eof);
+    if (writeResult.hasError()) {
+      throw QuicTransportException(
+          writeResult.error().message,
+          *writeResult.error().code.asTransportErrorCode());
+    }
     // If we were previously app limited restart pacing with the current rate.
     if (wasAppLimitedOrIdle && conn_->pacer) {
       conn_->pacer->reset();
@@ -1265,7 +1304,8 @@ QuicSocket::WriteResult QuicServerTransport::setDSRPacketizationRequestSender(
     if (!conn_->streamManager->streamExists(id)) {
       return folly::makeUnexpected(LocalErrorCode::STREAM_NOT_EXISTS);
     }
-    auto stream = CHECK_NOTNULL(conn_->streamManager->getStream(id));
+    auto stream =
+        CHECK_NOTNULL(conn_->streamManager->getStream(id).value_or(nullptr));
     // Only allow resetting it back to nullptr once set.
     if (stream->dsrSender && sender != nullptr) {
       return folly::makeUnexpected(LocalErrorCode::INVALID_OPERATION);
@@ -1290,7 +1330,7 @@ QuicSocket::WriteResult QuicServerTransport::setDSRPacketizationRequestSender(
     // shown good results.
     if (conn_->transportSettings.priorityQueueWritesPerStream == 1) {
       conn_->transportSettings.priorityQueueWritesPerStream = 5;
-      conn_->streamManager->writeQueue().setMaxNextsPerStream(5);
+      conn_->streamManager->setWriteQueueMaxNextsPerStream(5);
     }
 
     // Fow now, no appLimited or appIdle update here since we are not writing
