@@ -51,6 +51,7 @@ bool isValidConnIdLength(const quic::ConnectionId& connId) {
 namespace quic {
 
 std::atomic_int globalUnfinishedHandshakes{0};
+
 int QuicServerWorker::getUnfinishedHandshakeCount() {
   return globalUnfinishedHandshakes.load(std::memory_order_relaxed);
 }
@@ -134,6 +135,10 @@ void QuicServerWorker::bind(
   socket_->setTimestamping(SOF_TIMESTAMPING_SOFTWARE);
   socket_->setTXTime({CLOCK_MONOTONIC, /*deadline=*/false});
 
+  socket_->setMaxReadsPerEvent(transportSettings_.maxServerRecvPacketsPerLoop);
+  VLOG(3) << "Socket max reads per event set to "
+          << socket_->getMaxReadsPerEvent();
+
   if (mvfst_hook_on_socket_create) {
     mvfst_hook_on_socket_create(socket_->getNetworkSocket().toFd());
   }
@@ -688,23 +693,31 @@ QuicServerTransport::Ptr QuicServerWorker::makeTransport(
     trans->setCongestionControllerFactory(ccFactory_);
     trans->setTransportStatsCallback(statsCallback_.get()); // ok if nullptr
 
-    auto transportSettings = transportSettingsOverrideFn_
-        ? transportSettingsOverrideFn_(
-              transportSettings_, client.getIPAddress())
-              .value_or(transportSettings_)
-        : transportSettings_;
+    auto transportSettingsCopy = transportSettings_;
+    if (quicVersion == QuicVersion::MVFST_EXPERIMENTAL ||
+        quicVersion == QuicVersion::QUIC_V1_ALIAS) {
+      // Override BBRv1 with BBRv2
+      if (transportSettingsCopy.defaultCongestionController ==
+          CongestionControlType::BBR) {
+        transportSettingsCopy.defaultCongestionController =
+            CongestionControlType::BBR2;
+      }
+    }
+
+    // Call the override function last so it can override any previous config.
+    if (transportSettingsOverrideFn_) {
+      transportSettingsOverrideFn_(
+          transportSettingsCopy, client.getIPAddress());
+    }
     LOG_IF(
         ERROR,
-        transportSettings.dataPathType != transportSettings_.dataPathType)
+        transportSettingsCopy.dataPathType != transportSettings_.dataPathType)
         << "Overriding DataPathType isn't supported. Requested datapath="
-        << (transportSettings.dataPathType == DataPathType::ContinuousMemory
+        << (transportSettingsCopy.dataPathType == DataPathType::ContinuousMemory
                 ? "ContinuousMemory"
                 : "ChainedMemory");
-    if (quicVersion == QuicVersion::MVFST_EXPERIMENTAL) {
-      transportSettings.includeCwndHintsInSessionTicket = true;
-      transportSettings.useCwndHintsInSessionTicket = true;
-    }
-    trans->setTransportSettings(transportSettings);
+
+    trans->setTransportSettings(transportSettingsCopy);
     trans->setConnectionIdAlgo(connIdAlgo_.get());
     trans->setServerConnectionIdRejector(this);
     if (srcConnId) {
@@ -1252,9 +1265,12 @@ void QuicServerWorker::setFizzContext(
     std::shared_ptr<const fizz::server::FizzServerContext> ctx) {
   ctx_ = ctx;
 }
+
 TransportSettings QuicServerWorker::validateTransportSettings(
     TransportSettings transportSettings) {
-  if (transportSettings.batchingMode != QuicBatchingMode::BATCHING_MODE_GSO) {
+  if (transportSettings.batchingMode != QuicBatchingMode::BATCHING_MODE_GSO &&
+      transportSettings.batchingMode !=
+          QuicBatchingMode::BATCHING_MODE_SENDMMSG_GSO) {
     if (transportSettings.dataPathType == DataPathType::ContinuousMemory) {
       LOG(ERROR) << "Unsupported data path type and batching mode combination";
     }
@@ -1574,6 +1590,7 @@ size_t QuicServerWorker::SourceIdentityHash::operator()(
   std::array<unsigned char, kKeySize> key{};
 
   struct sockaddr_storage storage {};
+
   const auto& sockaddr = sid.first;
   sockaddr.getAddress(&storage);
   memcpy(key.data(), &storage, sizeof(struct sockaddr_storage));
