@@ -8,18 +8,13 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
-#include <condition_variable>
-#include <mutex>
-
 #include <fizz/client/test/Mocks.h>
 #include <fizz/crypto/test/TestUtil.h>
-#include <fizz/protocol/clock/test/Mocks.h>
 #include <fizz/protocol/test/Mocks.h>
 #include <fizz/server/test/Mocks.h>
 
 #include <folly/io/async/SSLContext.h>
 #include <folly/io/async/ScopedEventBaseThread.h>
-#include <folly/io/async/test/MockAsyncTransport.h>
 
 #include <quic/QuicConstants.h>
 #include <quic/common/test/TestUtils.h>
@@ -29,7 +24,6 @@
 #include <quic/fizz/server/handshake/AppToken.h>
 #include <quic/fizz/server/handshake/FizzServerHandshake.h>
 #include <quic/fizz/server/handshake/FizzServerQuicHandshakeContext.h>
-#include <quic/handshake/HandshakeLayer.h>
 #include <quic/server/handshake/AppToken.h>
 #include <quic/server/handshake/ServerHandshake.h>
 #include <quic/state/StateData.h>
@@ -143,7 +137,7 @@ class ServerHandshakeTest : public Test {
         hostname,
         cachedPsk,
         Optional<std::vector<fizz::ech::ECHConfig>>(none),
-        std::make_shared<FizzClientExtensions>(clientExtensions));
+        std::make_shared<FizzClientExtensions>(clientExtensions, 0));
   }
 
   void processCryptoEvents() {
@@ -292,31 +286,40 @@ class ServerHandshakeTest : public Test {
   }
 
   void operator()(fizz::DeliverAppData&) {}
+
   void operator()(fizz::WriteToSocket& write) {
     clientWrites.push_back(std::move(write));
   }
+
   void operator()(fizz::client::ReportEarlyHandshakeSuccess&) {
     earlyHandshakeSuccess = true;
   }
+
   void operator()(fizz::client::ReportHandshakeSuccess&) {
     handshakeSuccess = true;
   }
+
   void operator()(fizz::client::ReportEarlyWriteFailed&) {
     earlyWriteFailed = true;
   }
+
   void operator()(fizz::ReportError&) {
     error = true;
   }
+
   void operator()(fizz::WaitForData&) {
     waitForData = true;
     fizzClient->waitForData();
   }
+
   void operator()(fizz::client::MutateState& mutator) {
     mutator(clientState);
   }
+
   void operator()(fizz::client::NewCachedPsk& newCachedPsk) {
     clientCtx->putPsk(hostname, std::move(newCachedPsk.psk));
   }
+
   void operator()(fizz::SecretAvailable&) {}
 
   void operator()(fizz::EndOfData&) {}
