@@ -93,12 +93,8 @@ void recoverOrResetCongestionAndRttState(
 void maybeSetExperimentalSettings(QuicServerConnectionState& conn) {
   // no-op versions
   if (conn.version == QuicVersion::MVFST_EXPERIMENTAL) {
-    // MVFST_EXPERIMENTAL is used to jumpstart the cwnd for clients based on a
-    // cwndHint from a previous connection in QuicServerTransport.cpp
   } else if (conn.version == QuicVersion::MVFST_EXPERIMENTAL2) {
   } else if (conn.version == QuicVersion::MVFST_EXPERIMENTAL3) {
-    // MVFST_EXPERIMENTAL3 is used to apply a 2x pace scaling for BBRv2
-    // QuicServerWorker.cpp before CC is initialized.
   } else if (conn.version == QuicVersion::MVFST_EXPERIMENTAL4) {
   }
 }
@@ -254,6 +250,26 @@ void processClientInitialParams(
           TransportParameterId::knob_frames_supported),
       clientParams.parameters);
 
+  auto extendedAckFeatures = getIntegerParameter(
+      static_cast<TransportParameterId>(
+          TransportParameterId::extended_ack_features),
+      clientParams.parameters);
+
+  auto reliableResetTpIter = findParameter(
+      clientParams.parameters,
+      static_cast<TransportParameterId>(
+          TransportParameterId::reliable_stream_reset));
+  if (reliableResetTpIter != clientParams.parameters.end()) {
+    if (!reliableResetTpIter->value->empty()) {
+      throw QuicTransportException(
+          "Reliable reset transport parameter must be empty",
+          TransportErrorCode::TRANSPORT_PARAMETER_ERROR);
+    }
+    conn.peerAdvertisedReliableStreamResetSupport = true;
+  } else {
+    conn.peerAdvertisedReliableStreamResetSupport = false;
+  }
+
   // validate that we didn't receive original connection ID, stateless
   // reset token, or preferred address.
   if (preferredAddress && *preferredAddress != 0) {
@@ -279,7 +295,6 @@ void processClientInitialParams(
         "Retry Source Connection ID is received by server",
         TransportErrorCode::TRANSPORT_PARAMETER_ERROR);
   }
-
   if (maxAckDelay && *maxAckDelay >= kMaxAckDelay) {
     throw QuicTransportException(
         "Max Ack Delay is greater than 2^14 ",
@@ -310,10 +325,25 @@ void processClientInitialParams(
       maxStreamDataBidiRemote.value_or(0);
   conn.flowControlState.peerAdvertisedInitialMaxStreamOffsetUni =
       maxStreamDataUni.value_or(0);
-  conn.streamManager->setMaxLocalBidirectionalStreams(
-      maxStreamsBidi.value_or(0));
-  conn.streamManager->setMaxLocalUnidirectionalStreams(
-      maxStreamsUni.value_or(0));
+
+  auto maxBidiStreamsResult =
+      conn.streamManager->setMaxLocalBidirectionalStreams(
+          maxStreamsBidi.value_or(0));
+  if (!maxBidiStreamsResult) {
+    throw QuicTransportException(
+        "Failed to set max local bidirectional streams",
+        TransportErrorCode::TRANSPORT_PARAMETER_ERROR);
+  }
+
+  auto maxUniStreamsResult =
+      conn.streamManager->setMaxLocalUnidirectionalStreams(
+          maxStreamsUni.value_or(0));
+  if (!maxUniStreamsResult) {
+    throw QuicTransportException(
+        "Failed to set max local unidirectional streams",
+        TransportErrorCode::TRANSPORT_PARAMETER_ERROR);
+  }
+
   conn.peerIdleTimeout = std::chrono::milliseconds(idleTimeout.value_or(0));
   conn.peerIdleTimeout = timeMin(conn.peerIdleTimeout, kMaxIdleTimeout);
   if (ackDelayExponent && *ackDelayExponent > kMaxAckDelayExponent) {
@@ -370,6 +400,7 @@ void processClientInitialParams(
   }
 
   conn.peerAdvertisedKnobFrameSupport = knobFrameSupported.value_or(0) > 0;
+  conn.peerAdvertisedExtendedAckFeatures = extendedAckFeatures.value_or(0);
 }
 
 void updateHandshakeState(QuicServerConnectionState& conn) {
@@ -428,6 +459,7 @@ void updateHandshakeState(QuicServerConnectionState& conn) {
           TransportErrorCode::TRANSPORT_PARAMETER_ERROR);
     }
     processClientInitialParams(conn, std::move(*clientParams));
+    updateNegotiatedAckFeatures(conn);
   }
   if (oneRttReadCipher) {
     if (conn.qLogger) {
@@ -840,7 +872,8 @@ void onServerReadDataFromOpen(
     conn.readCodec->setCodecParameters(CodecParameters(
         conn.peerAckDelayExponent,
         version,
-        conn.transportSettings.maybeAckReceiveTimestampsConfigSentToPeer));
+        conn.transportSettings.maybeAckReceiveTimestampsConfigSentToPeer,
+        conn.transportSettings.advertisedExtendedAckFeatures));
     conn.initialWriteCipher = cryptoFactory.getServerInitialCipher(
         initialDestinationConnectionId, version);
 
@@ -890,14 +923,16 @@ void onServerReadDataFromOpen(
         break;
       }
       case CodecResult::Type::NOTHING: {
-        VLOG(10) << "drop cipher unavailable, no data " << conn;
+        VLOG(10) << "drop no data, reason: "
+                 << parsedPacket.nothing()->reason._to_string() << " " << conn;
         if (conn.qLogger) {
-          conn.qLogger->addPacketDrop(packetSize, kCipherUnavailable);
+          conn.qLogger->addPacketDrop(
+              packetSize, parsedPacket.nothing()->reason._to_string());
         }
         QUIC_STATS(
             conn.statsCallback,
             onPacketDropped,
-            PacketDropReason::UNEXPECTED_NOTHING);
+            parsedPacket.nothing()->reason);
         if (firstPacketFromPeer) {
           throw QuicInternalException(
               "Failed to decrypt first packet from peer",
@@ -905,6 +940,11 @@ void onServerReadDataFromOpen(
         }
         break;
       }
+      case CodecResult::Type::CODEC_ERROR: {
+        throw QuicTransportException(
+            parsedPacket.codecError()->error.message,
+            *parsedPacket.codecError()->error.code.asTransportErrorCode());
+      }
       case CodecResult::Type::REGULAR_PACKET:
         break;
     }
@@ -950,6 +990,15 @@ void onServerReadDataFromOpen(
 
     bool isZeroRttPacket = protectionLevel == ProtectionType::ZeroRtt;
 
+    bool isQuicInitialPacket =
+        regularPacket.header.getHeaderForm() == HeaderForm::Long &&
+        regularPacket.header.asLong()->getHeaderType() ==
+            LongHeader::Types::Initial;
+
+    if (isQuicInitialPacket) {
+      ++conn.initialPacketsReceived;
+    }
+
     if (!isProtectedPacket || isZeroRttPacket) {
       // there are some frame restrictions
       auto isFrameInvalidFn = !isProtectedPacket
@@ -1031,67 +1080,80 @@ void onServerReadDataFromOpen(
 
     AckedPacketVisitor ackedPacketVisitor =
         [&](const OutstandingPacketWrapper& outstandingPacket) {
-          maybeVerifyPendingKeyUpdate(conn, outstandingPacket, regularPacket);
+          return maybeVerifyPendingKeyUpdate(
+              conn, outstandingPacket, regularPacket);
         };
-
-    AckedFrameVisitor ackedFrameVisitor =
-        [&](const OutstandingPacketWrapper&,
-            const QuicWriteFrame& packetFrame) {
-          switch (packetFrame.type()) {
-            case QuicWriteFrame::Type::WriteStreamFrame: {
-              const WriteStreamFrame& frame = *packetFrame.asWriteStreamFrame();
-              VLOG(4) << "Server received ack for stream=" << frame.streamId
-                      << " offset=" << frame.offset << " fin=" << frame.fin
-                      << " len=" << frame.len << " " << conn;
-              auto ackedStream = conn.streamManager->getStream(frame.streamId);
-              if (ackedStream) {
-                sendAckSMHandler(*ackedStream, frame);
-              }
-              break;
-            }
-            case QuicWriteFrame::Type::WriteCryptoFrame: {
-              const WriteCryptoFrame& frame = *packetFrame.asWriteCryptoFrame();
-              auto cryptoStream =
-                  getCryptoStream(*conn.cryptoState, encryptionLevel);
-              processCryptoStreamAck(*cryptoStream, frame.offset, frame.len);
-              break;
-            }
-            case QuicWriteFrame::Type::RstStreamFrame: {
-              const RstStreamFrame& frame = *packetFrame.asRstStreamFrame();
-              VLOG(4) << "Server received ack for reset stream="
-                      << frame.streamId << " " << conn;
-              auto stream = conn.streamManager->getStream(frame.streamId);
-              if (stream) {
-                sendRstAckSMHandler(*stream);
-              }
-              break;
-            }
-            case QuicWriteFrame::Type::WriteAckFrame: {
-              const WriteAckFrame& frame = *packetFrame.asWriteAckFrame();
-              DCHECK(!frame.ackBlocks.empty());
-              VLOG(4) << "Server received ack for largestAcked="
-                      << frame.ackBlocks.front().end << " " << conn;
-              commonAckVisitorForAckFrame(ackState, frame);
-              break;
-            }
-            case QuicWriteFrame::Type::PingFrame:
-              conn.pendingEvents.cancelPingTimeout = true;
-              return;
-            case QuicWriteFrame::Type::QuicSimpleFrame: {
-              const QuicSimpleFrame& frame = *packetFrame.asQuicSimpleFrame();
-              // ACK of HandshakeDone is a server-specific behavior.
-              if (frame.asHandshakeDoneFrame()) {
-                // Call handshakeConfirmed outside of the packet
-                // processing loop to avoid a re-entrancy.
-                handshakeConfirmedThisLoop = true;
-              }
-              break;
-            }
-            default: {
-              break;
+    AckedFrameVisitor ackedFrameVisitor = [&](const OutstandingPacketWrapper&,
+                                              const QuicWriteFrame& packetFrame)
+        -> folly::Expected<folly::Unit, QuicError> {
+      switch (packetFrame.type()) {
+        case QuicWriteFrame::Type::WriteStreamFrame: {
+          const WriteStreamFrame& frame = *packetFrame.asWriteStreamFrame();
+          VLOG(4) << "Server received ack for stream=" << frame.streamId
+                  << " offset=" << frame.offset << " fin=" << frame.fin
+                  << " len=" << frame.len << " " << conn;
+          auto ackedStream =
+              conn.streamManager->getStream(frame.streamId).value_or(nullptr);
+          if (ackedStream) {
+            auto result = sendAckSMHandler(*ackedStream, frame);
+            if (result.hasError()) {
+              throw QuicTransportException(
+                  result.error().message,
+                  *result.error().code.asTransportErrorCode());
             }
           }
-        };
+          break;
+        }
+        case QuicWriteFrame::Type::WriteCryptoFrame: {
+          const WriteCryptoFrame& frame = *packetFrame.asWriteCryptoFrame();
+          auto cryptoStream =
+              getCryptoStream(*conn.cryptoState, encryptionLevel);
+          processCryptoStreamAck(*cryptoStream, frame.offset, frame.len);
+          break;
+        }
+        case QuicWriteFrame::Type::RstStreamFrame: {
+          const RstStreamFrame& frame = *packetFrame.asRstStreamFrame();
+          VLOG(4) << "Server received ack for reset stream=" << frame.streamId
+                  << " " << conn;
+          auto stream =
+              conn.streamManager->getStream(frame.streamId).value_or(nullptr);
+          if (stream) {
+            auto result = sendRstAckSMHandler(*stream, frame.reliableSize);
+            if (result.hasError()) {
+              throw QuicTransportException(
+                  result.error().message,
+                  *result.error().code.asTransportErrorCode());
+            }
+          }
+          break;
+        }
+        case QuicWriteFrame::Type::WriteAckFrame: {
+          const WriteAckFrame& frame = *packetFrame.asWriteAckFrame();
+          DCHECK(!frame.ackBlocks.empty());
+          VLOG(4) << "Server received ack for largestAcked="
+                  << frame.ackBlocks.front().end << " " << conn;
+          commonAckVisitorForAckFrame(ackState, frame);
+          break;
+        }
+        case QuicWriteFrame::Type::PingFrame:
+          conn.pendingEvents.cancelPingTimeout = true;
+          break;
+        case QuicWriteFrame::Type::QuicSimpleFrame: {
+          const QuicSimpleFrame& frame = *packetFrame.asQuicSimpleFrame();
+          // ACK of HandshakeDone is a server-specific behavior.
+          if (frame.asHandshakeDoneFrame()) {
+            // Call handshakeConfirmed outside of the packet
+            // processing loop to avoid a re-entrancy.
+            handshakeConfirmedThisLoop = true;
+          }
+          break;
+        }
+        default: {
+          break;
+        }
+      }
+      return folly::unit;
+    };
 
     for (auto& quicFrame : regularPacket.frames) {
       switch (quicFrame.type()) {
@@ -1100,27 +1162,67 @@ void onServerReadDataFromOpen(
                    << conn;
           isNonProbingPacket = true;
           ReadAckFrame& ackFrame = *quicFrame.asReadAckFrame();
-          conn.lastProcessedAckEvents.emplace_back(processAckFrame(
+
+          if (ackFrame.frameType == FrameType::ACK_EXTENDED &&
+              !conn.transportSettings.advertisedExtendedAckFeatures) {
+            throw QuicTransportException(
+                "Received unexpected ACK_EXTENDED frame",
+                TransportErrorCode::PROTOCOL_VIOLATION);
+          } else if (
+              ackFrame.frameType == FrameType::ACK_RECEIVE_TIMESTAMPS &&
+              !conn.transportSettings
+                   .maybeAckReceiveTimestampsConfigSentToPeer) {
+            throw QuicTransportException(
+                "Received unexpected ACK_RECEIVE_TIMESTAMPS frame",
+                TransportErrorCode::PROTOCOL_VIOLATION);
+          }
+
+          auto result = processAckFrame(
               conn,
               packetNumberSpace,
               ackFrame,
               ackedPacketVisitor,
               ackedFrameVisitor,
               markPacketLoss,
-              readData.udpPacket.timings.receiveTimePoint));
+              readData.udpPacket.timings.receiveTimePoint);
+          if (result.hasError()) {
+            throw QuicTransportException(
+                result.error().message,
+                *result.error().code.asTransportErrorCode());
+          }
+          conn.lastProcessedAckEvents.emplace_back(std::move(result.value()));
           break;
         }
         case QuicFrame::Type::RstStreamFrame: {
           RstStreamFrame& frame = *quicFrame.asRstStreamFrame();
+          if (frame.reliableSize.hasValue()) {
+            // We're not yet supporting the handling of RESET_STREAM_AT frames
+            throw QuicTransportException(
+                "Reliable resets not supported",
+                TransportErrorCode::PROTOCOL_VIOLATION);
+          }
           VLOG(10) << "Server received reset stream=" << frame.streamId << " "
                    << conn;
           pktHasRetransmittableData = true;
           isNonProbingPacket = true;
-          auto stream = conn.streamManager->getStream(frame.streamId);
+          auto streamResult = conn.streamManager->getStream(frame.streamId);
+          if (streamResult.hasError()) {
+            // TODO don't throw
+            throw QuicTransportException(
+                streamResult.error().message,
+                *streamResult.error().code.asTransportErrorCode());
+          }
+          auto& stream = streamResult.value();
+
           if (!stream) {
             break;
           }
-          receiveRstStreamSMHandler(*stream, frame);
+          auto result = receiveRstStreamSMHandler(*stream, frame);
+          if (result.hasError()) {
+            throw QuicTransportException(
+                result.error().message,
+                *result.error().code.asTransportErrorCode());
+          }
           break;
         }
         case QuicFrame::Type::ReadCryptoFrame: {
@@ -1132,13 +1234,25 @@ void onServerReadDataFromOpen(
                    << cryptoFrame.offset
                    << " len=" << cryptoFrame.data->computeChainDataLength()
                    << " currentReadOffset="
-                   << getCryptoStream(*conn.cryptoState, encryptionLevel)
-                          ->currentReadOffset
-                   << " " << conn;
-          appendDataToReadBuffer(
-              *getCryptoStream(*conn.cryptoState, encryptionLevel),
+                   << getCryptoStream(*conn.cryptoState, encryptionLevel);
+          auto cryptoStream =
+              getCryptoStream(*conn.cryptoState, encryptionLevel);
+          auto readBufferSize = cryptoStream->readBuffer.size();
+          auto result = appendDataToReadBuffer(
+              *cryptoStream,
               StreamBuffer(
                   std::move(cryptoFrame.data), cryptoFrame.offset, false));
+          if (result.hasError()) {
+            throw QuicTransportException(
+                result.error().message,
+                *result.error().code.asTransportErrorCode());
+          }
+          if (isQuicInitialPacket &&
+              readBufferSize != cryptoStream->readBuffer.size()) {
+            ++conn.uniqueInitialCryptoFramesReceived;
+            conn.cryptoState->lastInitialCryptoFrameReceivedTimePoint =
+                Clock::now();
+          }
           break;
         }
         case QuicFrame::Type::ReadStreamFrame: {
@@ -1149,12 +1263,25 @@ void onServerReadDataFromOpen(
                    << " fin=" << frame.fin << " " << conn;
           pktHasRetransmittableData = true;
           isNonProbingPacket = true;
-          auto stream = conn.streamManager->getStream(
+          auto streamResult = conn.streamManager->getStream(
               frame.streamId, frame.streamGroupId);
+          if (streamResult.hasError()) {
+            // TODO don't throw
+            throw QuicTransportException(
+                streamResult.error().message,
+                *streamResult.error().code.asTransportErrorCode());
+          }
+          auto& stream = streamResult.value();
           // Ignore data from closed streams that we don't have the
           // state for any more.
           if (stream) {
-            receiveReadStreamFrameSMHandler(*stream, std::move(frame));
+            auto result =
+                receiveReadStreamFrameSMHandler(*stream, std::move(frame));
+            if (result.hasError()) {
+              throw QuicTransportException(
+                  result.error().message,
+                  *result.error().code.asTransportErrorCode());
+            }
           }
           break;
         }
@@ -1181,8 +1308,14 @@ void onServerReadDataFromOpen(
           }
           pktHasRetransmittableData = true;
           isNonProbingPacket = true;
-          auto stream =
+          auto streamResult =
               conn.streamManager->getStream(streamWindowUpdate.streamId);
+          if (streamResult.hasError()) {
+            throw QuicTransportException(
+                streamResult.error().message,
+                *streamResult.error().code.asTransportErrorCode());
+          }
+          auto& stream = streamResult.value();
           if (stream) {
             handleStreamWindowUpdate(
                 *stream, streamWindowUpdate.maximumData, packetNum);
@@ -1203,7 +1336,13 @@ void onServerReadDataFromOpen(
                    << " " << conn;
           pktHasRetransmittableData = true;
           isNonProbingPacket = true;
-          auto stream = conn.streamManager->getStream(blocked.streamId);
+          auto streamResult = conn.streamManager->getStream(blocked.streamId);
+          if (streamResult.hasError()) {
+            throw QuicTransportException(
+                streamResult.error().message,
+                *streamResult.error().code.asTransportErrorCode());
+          }
+          auto& stream = streamResult.value();
           if (stream) {
             handleStreamBlocked(*stream);
           }
@@ -1254,8 +1393,14 @@ void onServerReadDataFromOpen(
               : regularPacket.header.asLong()->getDestinationConnId();
           pktHasRetransmittableData = true;
           QuicSimpleFrame& simpleFrame = *quicFrame.asQuicSimpleFrame();
-          isNonProbingPacket |= updateSimpleFrameOnPacketReceived(
+          auto simpleResult = updateSimpleFrameOnPacketReceived(
               conn, simpleFrame, dstConnId, readData.peer != conn.peerAddress);
+          if (simpleResult.hasError()) {
+            throw QuicTransportException(
+                simpleResult.error().message,
+                *simpleResult.error().code.asTransportErrorCode());
+          }
+          isNonProbingPacket |= simpleResult.value();
           break;
         }
         case QuicFrame::Type::DatagramFrame: {
@@ -1275,8 +1420,7 @@ void onServerReadDataFromOpen(
             // violation.
             throw QuicTransportException(
                 "Received IMMEDIATE_ACK frame without announcing min_ack_delay",
-                TransportErrorCode::PROTOCOL_VIOLATION,
-                FrameType::IMMEDIATE_ACK);
+                TransportErrorCode::PROTOCOL_VIOLATION);
           }
           // Send an ACK from any packet number space.
           if (conn.ackStates.initialAckState) {
@@ -1302,8 +1446,8 @@ void onServerReadDataFromOpen(
     maybeHandleIncomingKeyUpdate(conn);
 
     // Update writable limit before processing the handshake data. This is so
-    // that if we haven't decided whether or not to validate the peer, we won't
-    // increase the limit.
+    // that if we haven't decided whether or not to validate the peer, we
+    // won't increase the limit.
     updateWritableByteLimitOnRecvPacket(conn);
 
     if (conn.peerAddress != readData.peer) {
@@ -1455,16 +1599,21 @@ void onServerReadDataFromClosed(
       break;
     }
     case CodecResult::Type::NOTHING: {
-      VLOG(10) << "drop cipher unavailable, no data " << conn;
+      VLOG(10) << "drop no data, reason: "
+               << parsedPacket.nothing()->reason._to_string() << " " << conn;
       if (conn.qLogger) {
-        conn.qLogger->addPacketDrop(packetSize, kCipherUnavailable);
+        conn.qLogger->addPacketDrop(
+            packetSize, parsedPacket.nothing()->reason._to_string());
       }
       QUIC_STATS(
-          conn.statsCallback,
-          onPacketDropped,
-          PacketDropReason::UNEXPECTED_NOTHING);
+          conn.statsCallback, onPacketDropped, parsedPacket.nothing()->reason);
       break;
     }
+    case CodecResult::Type::CODEC_ERROR: {
+      throw QuicTransportException(
+          parsedPacket.codecError()->error.message,
+          *parsedPacket.codecError()->error.code.asTransportErrorCode());
+    }
     case CodecResult::Type::REGULAR_PACKET:
       break;
   }
