@@ -10,9 +10,6 @@
 #include <quic/QuicException.h>
 #include <quic/flowcontrol/QuicFlowController.h>
 
-#include <quic/state/QuicStreamUtilities.h>
-
-#include <folly/io/Cursor.h>
 #include <algorithm>
 
 namespace {
@@ -26,8 +23,8 @@ void prependToBuf(quic::Buf& buf, quic::Buf toAppend) {
 } // namespace
 
 namespace quic {
-
-void writeDataToQuicStream(QuicStreamState& stream, Buf data, bool eof) {
+folly::Expected<folly::Unit, QuicError>
+writeDataToQuicStream(QuicStreamState& stream, Buf data, bool eof) {
   auto neverWrittenBufMeta = (0 == stream.writeBufMeta.offset);
   uint64_t len = 0;
   if (data) {
@@ -50,11 +47,15 @@ void writeDataToQuicStream(QuicStreamState& stream, Buf data, bool eof) {
     auto bufferSize = stream.pendingWrites.chainLength();
     stream.finalWriteOffset = stream.currentWriteOffset + bufferSize;
   }
-  updateFlowControlOnWriteToStream(stream, len);
+  auto result = updateFlowControlOnWriteToStream(stream, len);
+  if (result.hasError()) {
+    return folly::makeUnexpected(result.error());
+  }
   stream.conn.streamManager->updateWritableStreams(stream);
+  return folly::unit;
 }
 
-void writeBufMetaToQuicStream(
+folly::Expected<folly::Unit, QuicError> writeBufMetaToQuicStream(
     QuicStreamState& stream,
     const BufferMeta& data,
     bool eof) {
@@ -78,8 +79,12 @@ void writeBufMetaToQuicStream(
         stream.writeBufMeta.offset + stream.writeBufMeta.length;
     stream.writeBufMeta.eof = true;
   }
-  updateFlowControlOnWriteToStream(stream, data.length);
+  auto result = updateFlowControlOnWriteToStream(stream, data.length);
+  if (result.hasError()) {
+    return folly::makeUnexpected(result.error());
+  }
   stream.conn.streamManager->updateWritableStreams(stream);
+  return folly::unit;
 }
 
 void writeDataToQuicStream(QuicCryptoStream& stream, Buf data) {
@@ -87,9 +92,42 @@ void writeDataToQuicStream(QuicCryptoStream& stream, Buf data) {
   stream.writeBuffer.append(std::move(data));
 }
 
-void appendDataToReadBufferCommon(
+// Helper function which appends a raw data range to what MUST be a "tail" of
+// a logic IOBuf chain, buf. The function will pack data into the available
+// tail agressively, and allocate in terms of appendLen until the push is
+// complete.
+static void pushToTail(folly::IOBuf* dst, Buf src, size_t allocSize) {
+  size_t appended = 0;
+  auto len = src->length();
+  auto data = src->data();
+  while (appended < len) {
+    // If there's no tail room or that buffer is shared, trying to use the tail
+    // will cause problems.
+    if (dst->tailroom() == 0 || dst->isSharedOne()) {
+      // If the buffer we are pushing has tail room, just use that one.
+      // Otherwise, we have to allocate one.
+      Buf newBuf;
+      if (src->tailroom() > 0 && !src->isSharedOne()) {
+        src->trimStart(appended);
+        dst->appendChain(std::move(src));
+        return;
+      }
+      newBuf = folly::IOBuf::createCombined(allocSize);
+      dst->appendChain(std::move(newBuf));
+      dst = dst->next();
+    }
+    auto toAppend = std::min(dst->tailroom(), len - appended);
+    memcpy(dst->writableTail(), data, toAppend);
+    dst->append(toAppend);
+    appended += toAppend;
+    data += toAppend;
+  }
+}
+
+folly::Expected<folly::Unit, QuicError> appendDataToReadBufferCommon(
     QuicStreamLike& stream,
     StreamBuffer buffer,
+    uint32_t coalescingSize,
     folly::Function<void(uint64_t, uint64_t)>&& connFlowControlVisitor) {
   auto& readBuffer = stream.readBuffer;
   auto it = readBuffer.begin();
@@ -101,25 +139,28 @@ void appendDataToReadBufferCommon(
     bufferEofOffset = bufferEndOffset;
   } else if (buffer.data.chainLength() == 0) {
     VLOG(10) << "Empty stream without EOF";
-    return;
+    return folly::unit;
   }
 
   if (stream.finalReadOffset && bufferEofOffset &&
       *stream.finalReadOffset != *bufferEofOffset) {
-    throw QuicTransportException(
-        "Invalid EOF", TransportErrorCode::FINAL_SIZE_ERROR);
+    return folly::makeUnexpected(QuicError(
+        QuicErrorCode(TransportErrorCode::FINAL_SIZE_ERROR),
+        std::string("Invalid EOF")));
   } else if (bufferEofOffset) {
     // Do some consistency checks on the stream.
     if (stream.maxOffsetObserved > *bufferEofOffset) {
-      throw QuicTransportException(
-          "EOF in middle of stream", TransportErrorCode::FINAL_SIZE_ERROR);
+      return folly::makeUnexpected(QuicError(
+          QuicErrorCode(TransportErrorCode::FINAL_SIZE_ERROR),
+          std::string("EOF in middle of stream")));
     }
     stream.finalReadOffset = bufferEofOffset;
   } else if (stream.finalReadOffset) {
     // We did not receive a segment with an EOF set.
     if (buffer.offset + buffer.data.chainLength() > *stream.finalReadOffset) {
-      throw QuicTransportException(
-          "Invalid data after EOF", TransportErrorCode::FINAL_SIZE_ERROR);
+      return folly::makeUnexpected(QuicError(
+          QuicErrorCode(TransportErrorCode::FINAL_SIZE_ERROR),
+          std::string("Invalid data after EOF")));
     }
   }
   // Update the flow control information before changing max offset observed on
@@ -131,7 +172,7 @@ void appendDataToReadBufferCommon(
   if (buffer.data.chainLength() == 0) {
     // Nothing more to do since we already processed the EOF
     // case.
-    return;
+    return folly::unit;
   }
 
   if (buffer.offset < stream.currentReadOffset) {
@@ -139,14 +180,14 @@ void appendDataToReadBufferCommon(
     buffer.data.trimStartAtMost(stream.currentReadOffset - buffer.offset);
     buffer.offset = stream.currentReadOffset;
     if (buffer.data.chainLength() == 0) {
-      return;
+      return folly::unit;
     }
   }
 
   // Nothing in the buffer, just append it.
   if (it == readBuffer.end()) {
     readBuffer.emplace_back(std::move(buffer));
-    return;
+    return folly::unit;
   }
 
   // Start overlap will point to the first buffer that overlaps with the
@@ -210,7 +251,20 @@ void appendDataToReadBufferCommon(
         currentEnd > itEnd) {
       // Right overlap. Not done.
       current->data.trimStartAtMost(itEnd - current->offset);
-      it->data.append(current->data.move());
+      if (coalescingSize > kDefaultUDPSendPacketLen) {
+        auto itData = it->data.move();
+        auto itDataTail = itData->prev();
+        auto currentData = current->data.move();
+        while (currentData != nullptr) {
+          auto rest = currentData->pop();
+          pushToTail(itDataTail, std::move(currentData), coalescingSize);
+          itDataTail = itData->prev();
+          currentData = std::move(rest);
+        }
+        it->data.append(std::move(itData));
+      } else {
+        it->data.append(current->data.move());
+      }
       current = &(*it);
       currentAlreadyInserted = true;
       DCHECK(!startOverlap);
@@ -226,34 +280,40 @@ void appendDataToReadBufferCommon(
         *startOverlap != readBuffer.end() || *endOverlap == readBuffer.end());
     auto insertIt = readBuffer.erase(*startOverlap, *endOverlap);
     readBuffer.emplace(insertIt, std::move(*current));
-    return;
+    return folly::unit;
   } else if (currentAlreadyInserted) {
     DCHECK(startOverlap);
     DCHECK(endOverlap);
     DCHECK(
         *startOverlap != readBuffer.end() || *endOverlap == readBuffer.end());
     readBuffer.erase(*startOverlap, *endOverlap);
-    return;
+    return folly::unit;
   }
   auto last = readBuffer.end() - 1;
   if (current->offset > last->offset + last->data.chainLength()) {
     readBuffer.emplace_back(std::move(*current));
   }
+  return folly::unit;
 }
 
-void appendDataToReadBuffer(QuicStreamState& stream, StreamBuffer buffer) {
-  appendDataToReadBufferCommon(
+folly::Expected<folly::Unit, QuicError> appendDataToReadBuffer(
+    QuicStreamState& stream,
+    StreamBuffer buffer) {
+  return appendDataToReadBufferCommon(
       stream,
       std::move(buffer),
+      stream.conn.transportSettings.readCoalescingSize,
       [&stream](uint64_t previousMaxOffsetObserved, uint64_t bufferEndOffset) {
-        updateFlowControlOnStreamData(
+        return updateFlowControlOnStreamData(
             stream, previousMaxOffsetObserved, bufferEndOffset);
       });
 }
 
-void appendDataToReadBuffer(QuicCryptoStream& stream, StreamBuffer buffer) {
-  appendDataToReadBufferCommon(
-      stream, std::move(buffer), [](uint64_t, uint64_t) {});
+folly::Expected<folly::Unit, QuicError> appendDataToReadBuffer(
+    QuicCryptoStream& stream,
+    StreamBuffer buffer) {
+  return appendDataToReadBufferCommon(
+      stream, std::move(buffer), 0, [](uint64_t, uint64_t) {});
 }
 
 std::pair<Buf, bool> readDataInOrderFromReadBuffer(
@@ -307,7 +367,7 @@ Buf readDataFromCryptoStream(QuicCryptoStream& stream, uint64_t amount) {
   return readDataInOrderFromReadBuffer(stream, amount).first;
 }
 
-std::pair<Buf, bool> readDataFromQuicStream(
+folly::Expected<std::pair<Buf, bool>, QuicError> readDataFromQuicStream(
     QuicStreamState& stream,
     uint64_t amount) {
   auto eof = stream.finalReadOffset &&
@@ -327,7 +387,11 @@ std::pair<Buf, bool> readDataFromQuicStream(
   std::tie(data, eof) = readDataInOrderFromReadBuffer(stream, amount);
   // Update flow control before handling eof as eof is not subject to flow
   // control
-  updateFlowControlOnRead(stream, lastReadOffset, Clock::now());
+  auto flowControlResult =
+      updateFlowControlOnRead(stream, lastReadOffset, Clock::now());
+  if (flowControlResult.hasError()) {
+    return folly::makeUnexpected(flowControlResult.error());
+  }
   eof = stream.finalReadOffset &&
       stream.currentReadOffset == *stream.finalReadOffset;
   if (eof) {
@@ -354,7 +418,9 @@ void peekDataFromQuicStream(
  * Same as readDataFromQuicStream(),
  * only releases existing data instead of returning it.
  */
-void consumeDataFromQuicStream(QuicStreamState& stream, uint64_t amount) {
+folly::Expected<folly::Unit, QuicError> consumeDataFromQuicStream(
+    QuicStreamState& stream,
+    uint64_t amount) {
   bool eof = stream.finalReadOffset &&
       stream.currentReadOffset >= *stream.finalReadOffset;
   if (eof) {
@@ -363,7 +429,7 @@ void consumeDataFromQuicStream(QuicStreamState& stream, uint64_t amount) {
     }
     stream.conn.streamManager->updateReadableStreams(stream);
     stream.conn.streamManager->updatePeekableStreams(stream);
-    return;
+    return folly::unit;
   }
 
   uint64_t lastReadOffset = stream.currentReadOffset;
@@ -371,7 +437,11 @@ void consumeDataFromQuicStream(QuicStreamState& stream, uint64_t amount) {
   readDataInOrderFromReadBuffer(stream, amount, true /* sinkData */);
   // Update flow control before handling eof as eof is not subject to flow
   // control
-  updateFlowControlOnRead(stream, lastReadOffset, Clock::now());
+  auto flowControlResult =
+      updateFlowControlOnRead(stream, lastReadOffset, Clock::now());
+  if (flowControlResult.hasError()) {
+    return folly::makeUnexpected(flowControlResult.error());
+  }
   eof = stream.finalReadOffset &&
       stream.currentReadOffset == *stream.finalReadOffset;
   if (eof) {
@@ -379,6 +449,7 @@ void consumeDataFromQuicStream(QuicStreamState& stream, uint64_t amount) {
   }
   stream.conn.streamManager->updateReadableStreams(stream);
   stream.conn.streamManager->updatePeekableStreams(stream);
+  return folly::unit;
 }
 
 bool allBytesTillFinAcked(const QuicStreamState& stream) {
@@ -399,7 +470,8 @@ bool allBytesTillFinAcked(const QuicStreamState& stream) {
 void appendPendingStreamReset(
     QuicConnectionStateBase& conn,
     const QuicStreamState& stream,
-    ApplicationErrorCode errorCode) {
+    ApplicationErrorCode errorCode,
+    Optional<uint64_t> reliableSize) {
   /*
    * When BufMetas are written to the transport, but before they are written to
    * the network, writeBufMeta.offset would be assigned a value >
@@ -412,17 +484,40 @@ void appendPendingStreamReset(
   bool writeBufWritten = stream.writeBufMeta.offset &&
       (stream.currentWriteOffset + stream.pendingWrites.chainLength() !=
        stream.writeBufMeta.offset);
+
+  /*
+   * The spec mandates that with multiple RESET_STREAM_AT or RESET_STREAM
+   * frames, we must use the same value of finalSize. Although we don't store
+   * previous values of finalSize to ensure that we use that same value
+   * throughout, we still maintain this property, due to the following facts:
+   *
+   * 1. We only egress a RESET_STREAM_AT or RESET_STREAM frame when all data
+   * upto the reliable reset offset has been egressed.
+   * 2. The user is not allowed to increase the reliable size in subsequent
+   * calls to resetStream().
+   *
+   * Therefore, if a RESET_STREAM_AT frame has been egressed, it means that the
+   * data until the reliableSize in that frame has been egressed, so we end up
+   * with the same calculation of the finalSize in subsequent RESET_STREAM or
+   * RESET_STREAM_AT frames by virtue of the fact that it's the maxiumum of the
+   * current write offset and the new reliable size.
+   */
+  uint64_t finalSize =
+      writeBufWritten ? stream.writeBufMeta.offset : stream.currentWriteOffset;
+  if (reliableSize) {
+    // It's possible that we've queued up data at the socket, but haven't yet
+    // written it out to the wire, so stream.currentWriteOffset could be
+    // lagging behind reliableSize
+    finalSize = std::max(finalSize, *reliableSize);
+  }
+  finalSize = std::min(
+      finalSize,
+      stream.finalWriteOffset.value_or(std::numeric_limits<uint64_t>::max()));
+
   conn.pendingEvents.resets.emplace(
       std::piecewise_construct,
       std::forward_as_tuple(stream.id),
-      std::forward_as_tuple(
-          stream.id,
-          errorCode,
-          std::min(
-              writeBufWritten ? stream.writeBufMeta.offset
-                              : stream.currentWriteOffset,
-              stream.finalWriteOffset.value_or(
-                  std::numeric_limits<uint64_t>::max()))));
+      std::forward_as_tuple(stream.id, errorCode, finalSize, reliableSize));
 }
 
 uint64_t getLargestWriteOffsetSeen(const QuicStreamState& stream) {
