@@ -91,6 +91,7 @@ struct OutstandingsInfo {
     packetCount = {};
     clonedPacketCount = {};
     declaredLostCount = 0;
+    scheduledForDestructionCount = 0;
     dsrCount = 0;
   }
 };
@@ -250,6 +251,9 @@ struct QuicCryptoState {
 
   // Stream to exchange handshake data encrypted with 1-rtt keys.
   QuicCryptoStream oneRttStream;
+
+  // Time point that the last initial packet with crypto frame received.
+  TimePoint lastInitialCryptoFrameReceivedTimePoint;
 };
 
 struct ConnectionCloseEvent {
@@ -449,7 +453,7 @@ struct QuicConnectionStateBase : public folly::DelayedDestruction {
     // Number of probing packets to send after PTO
     EnumArray<PacketNumberSpace, uint8_t> numProbePackets{};
 
-    FOLLY_NODISCARD bool anyProbePackets() const {
+    [[nodiscard]] bool anyProbePackets() const {
       return numProbePackets[PacketNumberSpace::Initial] +
           numProbePackets[PacketNumberSpace::Handshake] +
           numProbePackets[PacketNumberSpace::AppData];
@@ -699,7 +703,17 @@ struct QuicConnectionStateBase : public folly::DelayedDestruction {
 
   Optional<AckReceiveTimestampsConfig> maybePeerAckReceiveTimestampsConfig;
 
+  bool peerAdvertisedReliableStreamResetSupport{false};
+
   bool peerAdvertisedKnobFrameSupport{false};
+
+  ExtendedAckFeatureMaskType peerAdvertisedExtendedAckFeatures{0};
+
+  // Negotiated ACK related config. These don't change throughout the connection
+  // so cache them once we've receive the relevant transport parameters.
+  bool negotiatedAckReceiveTimestampSupport{false};
+  ExtendedAckFeatureMaskType negotiatedExtendedAckFeatures{0};
+
   // Retransmission policies map.
   folly::F14FastMap<StreamGroupId, QuicStreamGroupRetransmissionPolicy>
       retransmissionPolicies;
@@ -711,18 +725,28 @@ struct QuicConnectionStateBase : public folly::DelayedDestruction {
     // for multiple writes.
     uint64_t targetWriteCount;
   };
+
   SocketCmsgsState socketCmsgsState;
 
   ECNState ecnState{ECNState::NotAttempted};
   std::shared_ptr<EcnL4sTracker> ecnL4sTracker;
+
   union TosHeader {
     uint8_t value{0};
+
     struct components {
       unsigned int ecn : 2;
       unsigned int dscp : 6;
     } fields;
   };
+
   TosHeader socketTos;
+
+  // Number of QUIC initial packets received.
+  uint16_t initialPacketsReceived{0};
+
+  // Number of QUIC unique crypto frame received with initial package.
+  uint16_t uniqueInitialCryptoFramesReceived{0};
 };
 
 std::ostream& operator<<(std::ostream& os, const QuicConnectionStateBase& st);
@@ -743,7 +767,9 @@ struct AckStateVersion {
   bool operator!=(const AckStateVersion& other) const;
 };
 
-using LossVisitor = std::function<
-    void(QuicConnectionStateBase&, RegularQuicWritePacket&, bool)>;
+using LossVisitor = std::function<folly::Expected<folly::Unit, QuicError>(
+    QuicConnectionStateBase& conn,
+    RegularQuicWritePacket& packet,
+    bool processed)>;
 
 } // namespace quic
