@@ -46,10 +46,13 @@ std::unique_ptr<QuicServerConnectionState> createConn() {
       kDefaultStreamFlowControlWindow;
   conn->flowControlState.peerAdvertisedMaxOffset =
       kDefaultConnectionFlowControlWindow;
-  conn->streamManager->setMaxLocalBidirectionalStreams(
-      kDefaultMaxStreamsBidirectional);
-  conn->streamManager->setMaxLocalUnidirectionalStreams(
-      kDefaultMaxStreamsUnidirectional);
+  CHECK(!conn->streamManager
+             ->setMaxLocalBidirectionalStreams(kDefaultMaxStreamsBidirectional)
+             .hasError());
+  CHECK(
+      !conn->streamManager
+           ->setMaxLocalUnidirectionalStreams(kDefaultMaxStreamsUnidirectional)
+           .hasError());
   return conn;
 }
 
@@ -63,7 +66,8 @@ TEST_F(QuicOpenStateTest, ReadStreamDataNotFin) {
   bool fin = false;
   ReadStreamFrame frame(id, offset, fin);
   frame.data = IOBuf::copyBuffer("hey");
-  receiveReadStreamFrameSMHandler(stream, std::move(frame));
+  auto result = receiveReadStreamFrameSMHandler(stream, std::move(frame));
+  ASSERT_FALSE(result.hasError());
   EXPECT_TRUE(stream.hasReadableData());
   EXPECT_TRUE(stream.hasPeekableData());
   EXPECT_EQ(stream.recvState, StreamRecvState::Open);
@@ -79,16 +83,17 @@ TEST_F(QuicOpenStateTest, ReadInvalidData) {
   // EOF in middle of stream
   ReadStreamFrame frame1(id, offset1, fin1);
   frame1.data = IOBuf::copyBuffer("hey");
-  receiveReadStreamFrameSMHandler(stream, std::move(frame1));
+  auto result1 = receiveReadStreamFrameSMHandler(stream, std::move(frame1));
+  ASSERT_FALSE(result1.hasError());
   EXPECT_EQ(stream.recvState, StreamRecvState::Open);
 
   uint64_t offset2 = 1;
   bool fin2 = true;
   ReadStreamFrame frame2(id, offset2, fin2);
   frame2.data = IOBuf::copyBuffer("e");
-  EXPECT_THROW(
-      receiveReadStreamFrameSMHandler(stream, std::move(frame2)),
-      QuicTransportException);
+  auto result = receiveReadStreamFrameSMHandler(stream, std::move(frame2));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicOpenStateTest, InvalidEvent) {
@@ -96,7 +101,9 @@ TEST_F(QuicOpenStateTest, InvalidEvent) {
   StreamId id = 5;
   QuicStreamState stream(id, *conn);
   RstStreamFrame frame(1, GenericApplicationErrorCode::UNKNOWN, 0);
-  EXPECT_THROW(sendRstAckSMHandler(stream), QuicTransportException);
+  auto result = sendRstAckSMHandler(stream, folly::none);
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicOpenStateTest, ReceiveStreamFrameWithFIN) {
@@ -109,7 +116,9 @@ TEST_F(QuicOpenStateTest, ReceiveStreamFrameWithFIN) {
   ReadStreamFrame receivedStreamFrame(stream->id, 100, true);
   receivedStreamFrame.data = folly::IOBuf::create(10);
   receivedStreamFrame.data->append(10);
-  receiveReadStreamFrameSMHandler(*stream, std::move(receivedStreamFrame));
+  auto result =
+      receiveReadStreamFrameSMHandler(*stream, std::move(receivedStreamFrame));
+  ASSERT_FALSE(result.hasError());
   ASSERT_EQ(stream->recvState, StreamRecvState::Closed);
 }
 
@@ -123,7 +132,9 @@ TEST_F(QuicOpenStateTest, ReceiveStreamFrameWithFINReadbuffHole) {
   ReadStreamFrame receivedStreamFrame(stream->id, 200, true);
   receivedStreamFrame.data = folly::IOBuf::create(10);
   receivedStreamFrame.data->append(10);
-  receiveReadStreamFrameSMHandler(*stream, std::move(receivedStreamFrame));
+  auto result =
+      receiveReadStreamFrameSMHandler(*stream, std::move(receivedStreamFrame));
+  ASSERT_FALSE(result.hasError());
   ASSERT_EQ(stream->recvState, StreamRecvState::Open);
 }
 
@@ -137,7 +148,9 @@ TEST_F(QuicOpenStateTest, ReceiveStreamFrameWithoutFIN) {
   ReadStreamFrame receivedStreamFrame(stream->id, 100, false);
   receivedStreamFrame.data = folly::IOBuf::create(10);
   receivedStreamFrame.data->append(10);
-  receiveReadStreamFrameSMHandler(*stream, std::move(receivedStreamFrame));
+  auto result =
+      receiveReadStreamFrameSMHandler(*stream, std::move(receivedStreamFrame));
+  ASSERT_FALSE(result.hasError());
   ASSERT_EQ(stream->recvState, StreamRecvState::Open);
 }
 
@@ -169,10 +182,12 @@ TEST_F(QuicOpenStateTest, AckStream) {
            ->packet.frames.front()
            .asWriteStreamFrame();
 
-  sendAckSMHandler(*stream, streamFrame);
+  auto result1 = sendAckSMHandler(*stream, streamFrame);
+  ASSERT_FALSE(result1.hasError());
   ASSERT_EQ(stream->sendState, StreamSendState::Closed);
 
-  sendAckSMHandler(*stream, streamFrame);
+  auto result2 = sendAckSMHandler(*stream, streamFrame);
+  ASSERT_FALSE(result2.hasError());
   ASSERT_EQ(stream->sendState, StreamSendState::Closed);
 }
 
@@ -218,7 +233,8 @@ TEST_F(QuicOpenStateTest, AckStreamMulti) {
   auto& streamFrame3 =
       *conn->outstandings.packets[2].packet.frames[0].asWriteStreamFrame();
 
-  sendAckSMHandler(*stream, streamFrame3);
+  auto result1 = sendAckSMHandler(*stream, streamFrame3);
+  ASSERT_FALSE(result1.hasError());
   ASSERT_EQ(stream->sendState, StreamSendState::Open);
   ASSERT_EQ(stream->ackedIntervals.front().start, 10);
   ASSERT_EQ(stream->ackedIntervals.front().end, 20);
@@ -226,7 +242,8 @@ TEST_F(QuicOpenStateTest, AckStreamMulti) {
   auto& streamFrame2 =
       *conn->outstandings.packets[1].packet.frames[0].asWriteStreamFrame();
 
-  sendAckSMHandler(*stream, streamFrame2);
+  auto result2 = sendAckSMHandler(*stream, streamFrame2);
+  ASSERT_FALSE(result2.hasError());
   ASSERT_EQ(stream->sendState, StreamSendState::Open);
   ASSERT_EQ(stream->ackedIntervals.front().start, 5);
   ASSERT_EQ(stream->ackedIntervals.front().end, 20);
@@ -234,7 +251,8 @@ TEST_F(QuicOpenStateTest, AckStreamMulti) {
   auto& streamFrame1 =
       *conn->outstandings.packets[0].packet.frames[0].asWriteStreamFrame();
 
-  sendAckSMHandler(*stream, streamFrame1);
+  auto result3 = sendAckSMHandler(*stream, streamFrame1);
+  ASSERT_FALSE(result3.hasError());
   ASSERT_EQ(stream->sendState, StreamSendState::Open);
   ASSERT_EQ(stream->ackedIntervals.front().start, 0);
   ASSERT_EQ(stream->ackedIntervals.front().end, 20);
@@ -282,7 +300,8 @@ TEST_F(QuicOpenStateTest, RetxBufferSortedAfterAck) {
   auto streamFrame = *conn->outstandings.packets[std::rand() % 3]
                           .packet.frames.front()
                           .asWriteStreamFrame();
-  sendAckSMHandler(*stream, streamFrame);
+  auto result = sendAckSMHandler(*stream, streamFrame);
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(2, stream->retransmissionBuffer.size());
 }
 
@@ -299,11 +318,179 @@ TEST_F(QuicResetSentStateTest, RstAck) {
   stream.readBuffer.emplace_back(
       folly::IOBuf::copyBuffer("One more thing"), 0xABCD, false);
   RstStreamFrame frame(id, GenericApplicationErrorCode::UNKNOWN, 0);
-  sendRstAckSMHandler(stream);
+  auto result = sendRstAckSMHandler(stream, folly::none);
+  ASSERT_FALSE(result.hasError());
 
   EXPECT_EQ(stream.sendState, StreamSendState::Closed);
   EXPECT_FALSE(stream.finalReadOffset);
   EXPECT_FALSE(stream.readBuffer.empty());
+  EXPECT_EQ(*stream.minReliableSizeAcked, 0);
+}
+
+// A reset with a reliable size of 3 was previously ACKed.
+// Now, we're getting an ACK for a reset with a reliable size of 5,
+// and should therefore not see a reduction in minReliableSizeAcked
+TEST_F(QuicResetSentStateTest, ReliableRstAckNoReduction) {
+  auto conn = createConn();
+  StreamId id = 5;
+
+  QuicStreamState stream(id, *conn);
+  stream.minReliableSizeAcked = 3;
+  stream.sendState = StreamSendState::ResetSent;
+  stream.currentReadOffset = 0xABCD;
+  stream.finalWriteOffset = 0xACDC;
+  stream.readBuffer.emplace_back(
+      folly::IOBuf::copyBuffer("One more thing"), 0xABCD, false);
+  RstStreamFrame frame(id, GenericApplicationErrorCode::UNKNOWN, 0);
+  stream.updateAckedIntervals(0, 3, false);
+  auto result = sendRstAckSMHandler(stream, 5);
+  ASSERT_FALSE(result.hasError());
+
+  EXPECT_EQ(stream.sendState, StreamSendState::Closed);
+  EXPECT_FALSE(stream.finalReadOffset);
+  EXPECT_FALSE(stream.readBuffer.empty());
+  EXPECT_EQ(*stream.minReliableSizeAcked, 3);
+}
+
+// A reset with a reliable size of 3 was previously ACKed.
+// Now, we're getting an ACK for a reset with a reliable size
+// of 1, and should therefore see a reduction in minReliableSizeAcked
+TEST_F(QuicResetSentStateTest, ReliableRstAckReduction) {
+  auto conn = createConn();
+  StreamId id = 5;
+
+  QuicStreamState stream(id, *conn);
+  stream.minReliableSizeAcked = 3;
+  stream.sendState = StreamSendState::ResetSent;
+  stream.currentReadOffset = 0xABCD;
+  stream.finalWriteOffset = 0xACDC;
+  stream.readBuffer.emplace_back(
+      folly::IOBuf::copyBuffer("One more thing"), 0xABCD, false);
+  RstStreamFrame frame(id, GenericApplicationErrorCode::UNKNOWN, 0);
+  stream.updateAckedIntervals(0, 1, false);
+  auto result = sendRstAckSMHandler(stream, 1);
+  ASSERT_FALSE(result.hasError());
+
+  EXPECT_EQ(stream.sendState, StreamSendState::Closed);
+  EXPECT_FALSE(stream.finalReadOffset);
+  EXPECT_FALSE(stream.readBuffer.empty());
+  EXPECT_EQ(*stream.minReliableSizeAcked, 1);
+}
+
+// There were no previously ACKed resets. Therefore, when we get an
+// ACK for a reset with a reliable size of 1, we should set the
+// minReliableSizeAcked to 1.
+TEST_F(QuicResetSentStateTest, ReliableRstAckFirstTime) {
+  auto conn = createConn();
+  StreamId id = 5;
+
+  QuicStreamState stream(id, *conn);
+  stream.sendState = StreamSendState::ResetSent;
+  stream.currentReadOffset = 0xABCD;
+  stream.finalWriteOffset = 0xACDC;
+  stream.readBuffer.emplace_back(
+      folly::IOBuf::copyBuffer("One more thing"), 0xABCD, false);
+  RstStreamFrame frame(id, GenericApplicationErrorCode::UNKNOWN, 0);
+  auto result = sendRstAckSMHandler(stream, 1);
+  ASSERT_FALSE(result.hasError());
+
+  EXPECT_EQ(stream.sendState, StreamSendState::ResetSent);
+  EXPECT_FALSE(stream.finalReadOffset);
+  EXPECT_FALSE(stream.readBuffer.empty());
+  EXPECT_EQ(*stream.minReliableSizeAcked, 1);
+}
+
+// A reset with a reliable size of 3 was previously ACKed.
+// Now, we're getting an ACK for a non-reliable reset, and should
+// therefore set the minReliableSizeAcked to 0.
+TEST_F(QuicResetSentStateTest, RstAfterReliableRst) {
+  auto conn = createConn();
+  StreamId id = 5;
+
+  QuicStreamState stream(id, *conn);
+  stream.minReliableSizeAcked = 3;
+  stream.sendState = StreamSendState::ResetSent;
+  stream.currentReadOffset = 0xABCD;
+  stream.finalWriteOffset = 0xACDC;
+  stream.readBuffer.emplace_back(
+      folly::IOBuf::copyBuffer("One more thing"), 0xABCD, false);
+  RstStreamFrame frame(id, GenericApplicationErrorCode::UNKNOWN, 0);
+  auto result = sendRstAckSMHandler(stream, folly::none);
+  ASSERT_FALSE(result.hasError());
+
+  EXPECT_EQ(stream.sendState, StreamSendState::Closed);
+  EXPECT_FALSE(stream.finalReadOffset);
+  EXPECT_FALSE(stream.readBuffer.empty());
+  EXPECT_EQ(*stream.minReliableSizeAcked, 0);
+}
+
+// A reliable RESET has been ACKed, and all bytes till the reliable
+// size have been ACKed.
+TEST_F(QuicResetSentStateTest, ResetSentToClosedTransition1) {
+  auto conn = createConn();
+  StreamId id = 5;
+  QuicStreamState stream(id, *conn);
+  stream.sendState = StreamSendState::ResetSent;
+  stream.updateAckedIntervals(0, 5, false);
+  auto result = sendRstAckSMHandler(stream, 5);
+  ASSERT_FALSE(result.hasError());
+  EXPECT_EQ(stream.sendState, StreamSendState::Closed);
+}
+
+// A reliable RESET has been ACKed, but not all bytes till the
+// reliable size have been ACKed.
+TEST_F(QuicResetSentStateTest, ResetSentToClosedTransition2) {
+  auto conn = createConn();
+  StreamId id = 5;
+  QuicStreamState stream(id, *conn);
+  stream.sendState = StreamSendState::ResetSent;
+  stream.updateAckedIntervals(0, 4, false);
+  auto result = sendRstAckSMHandler(stream, 5);
+  ASSERT_FALSE(result.hasError());
+  EXPECT_EQ(stream.sendState, StreamSendState::ResetSent);
+}
+
+// A reliable RESET with a reliable size was ACKed previously, and
+// now we're getting an ACK for stream data until that reliable size
+TEST_F(QuicResetSentStateTest, ResetSentToClosedTransition3) {
+  auto conn = createConn();
+  StreamId id = 5;
+  QuicStreamState stream(id, *conn);
+  stream.sendState = StreamSendState::ResetSent;
+  stream.minReliableSizeAcked = 7;
+  WriteStreamFrame streamFrame(id, 0, 7, false);
+  auto buf = folly::IOBuf::create(7);
+  buf->append(7);
+  stream.retransmissionBuffer.emplace(
+      std::piecewise_construct,
+      std::forward_as_tuple(0),
+      std::forward_as_tuple(std::make_unique<WriteStreamBuffer>(
+          ChainedByteRangeHead(buf), 0, false)));
+  auto result = sendAckSMHandler(stream, streamFrame);
+  ASSERT_FALSE(result.hasError());
+  EXPECT_EQ(stream.sendState, StreamSendState::Closed);
+}
+
+// A reliable RESET with a reliable size was ACKed previously, and
+// now we're getting an ACK for stream data, but not until the
+// reliable size
+TEST_F(QuicResetSentStateTest, ResetSentToClosedTransition4) {
+  auto conn = createConn();
+  StreamId id = 5;
+  QuicStreamState stream(id, *conn);
+  stream.sendState = StreamSendState::ResetSent;
+  stream.minReliableSizeAcked = 8;
+  WriteStreamFrame streamFrame(id, 0, 7, false);
+  auto buf = folly::IOBuf::create(7);
+  buf->append(7);
+  stream.retransmissionBuffer.emplace(
+      std::piecewise_construct,
+      std::forward_as_tuple(0),
+      std::forward_as_tuple(std::make_unique<WriteStreamBuffer>(
+          ChainedByteRangeHead(buf), 0, false)));
+  auto result = sendAckSMHandler(stream, streamFrame);
+  ASSERT_FALSE(result.hasError());
+  EXPECT_EQ(stream.sendState, StreamSendState::ResetSent);
 }
 
 class QuicClosedStateTest : public Test {};
@@ -314,7 +501,8 @@ TEST_F(QuicClosedStateTest, RstAck) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Closed;
   RstStreamFrame frame(id, GenericApplicationErrorCode::UNKNOWN, 0);
-  sendRstAckSMHandler(stream);
+  auto result = sendRstAckSMHandler(stream, folly::none);
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.sendState, StreamSendState::Closed);
 }
 
@@ -332,7 +520,9 @@ TEST_F(QuicHalfClosedLocalStateTest, ReceiveStreamFrameWithFIN) {
   ReadStreamFrame receivedStreamFrame(stream->id, 100, true);
   receivedStreamFrame.data = folly::IOBuf::create(10);
   receivedStreamFrame.data->append(10);
-  receiveReadStreamFrameSMHandler(*stream, std::move(receivedStreamFrame));
+  auto result =
+      receiveReadStreamFrameSMHandler(*stream, std::move(receivedStreamFrame));
+  ASSERT_FALSE(result.hasError());
   ASSERT_EQ(stream->sendState, StreamSendState::Closed);
   ASSERT_EQ(stream->recvState, StreamRecvState::Closed);
 }
@@ -349,7 +539,9 @@ TEST_F(QuicHalfClosedLocalStateTest, ReceiveStreamFrameWithFINReadbuffHole) {
   ReadStreamFrame receivedStreamFrame(stream->id, 200, true);
   receivedStreamFrame.data = folly::IOBuf::create(10);
   receivedStreamFrame.data->append(10);
-  receiveReadStreamFrameSMHandler(*stream, std::move(receivedStreamFrame));
+  auto result =
+      receiveReadStreamFrameSMHandler(*stream, std::move(receivedStreamFrame));
+  ASSERT_FALSE(result.hasError());
   ASSERT_EQ(stream->sendState, StreamSendState::Closed);
   ASSERT_EQ(stream->recvState, StreamRecvState::Open);
 }
@@ -366,7 +558,9 @@ TEST_F(QuicHalfClosedLocalStateTest, ReceiveStreamFrameWithoutFIN) {
   ReadStreamFrame receivedStreamFrame(stream->id, 100, false);
   receivedStreamFrame.data = folly::IOBuf::create(10);
   receivedStreamFrame.data->append(10);
-  receiveReadStreamFrameSMHandler(*stream, std::move(receivedStreamFrame));
+  auto result =
+      receiveReadStreamFrameSMHandler(*stream, std::move(receivedStreamFrame));
+  ASSERT_FALSE(result.hasError());
 
   ASSERT_EQ(stream->sendState, StreamSendState::Closed);
   ASSERT_EQ(stream->recvState, StreamRecvState::Open);
@@ -406,10 +600,12 @@ TEST_F(QuicHalfClosedRemoteStateTest, AckStream) {
            ->packet.frames.front()
            .asWriteStreamFrame();
 
-  sendAckSMHandler(*stream, streamFrame);
+  auto result = sendAckSMHandler(*stream, streamFrame);
+  ASSERT_FALSE(result.hasError());
   ASSERT_EQ(stream->sendState, StreamSendState::Closed);
 
-  sendAckSMHandler(*stream, streamFrame);
+  result = sendAckSMHandler(*stream, streamFrame);
+  ASSERT_FALSE(result.hasError());
   ASSERT_EQ(stream->sendState, StreamSendState::Closed);
 }
 
@@ -419,7 +615,8 @@ TEST_F(QuicSendResetTest, FromOpen) {
   auto conn = createConn();
   StreamId id = 5;
   QuicStreamState stream(id, *conn);
-  sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN);
+  auto result = sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN);
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.sendState, StreamSendState::ResetSent);
 }
 
@@ -430,7 +627,8 @@ TEST_F(QuicSendResetTest, FromHalfCloseRemote) {
   stream.sendState = StreamSendState::Open;
   stream.recvState = StreamRecvState::Closed;
 
-  sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN);
+  auto result = sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN);
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.sendState, StreamSendState::ResetSent);
 }
 
@@ -440,7 +638,8 @@ TEST_F(QuicSendResetTest, FromHalfCloseLocal) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Closed;
   stream.recvState = StreamRecvState::Open;
-  sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN);
+  auto result = sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN);
+  ASSERT_FALSE(result.hasError());
 
   // You cannot send a reset after FIN has been acked
   EXPECT_EQ(stream.sendState, StreamSendState::Closed);
@@ -452,7 +651,8 @@ TEST_F(QuicSendResetTest, FromClosed) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Closed;
 
-  sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN);
+  auto result = sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN);
+  ASSERT_FALSE(result.hasError());
 }
 
 TEST_F(QuicSendResetTest, FromResetSent) {
@@ -460,7 +660,8 @@ TEST_F(QuicSendResetTest, FromResetSent) {
   StreamId id = 5;
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::ResetSent;
-  sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN);
+  auto result = sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN);
+  ASSERT_FALSE(result.hasError());
 }
 
 class QuicRecvResetTest : public Test {};
@@ -471,7 +672,8 @@ TEST_F(QuicRecvResetTest, FromOpen) {
   StreamId rstStream = 1;
   QuicStreamState stream(id, *conn);
   RstStreamFrame rst(rstStream, GenericApplicationErrorCode::UNKNOWN, 100);
-  receiveRstStreamSMHandler(stream, std::move(rst));
+  auto result = receiveRstStreamSMHandler(stream, std::move(rst));
+  ASSERT_FALSE(result.hasError());
 
   EXPECT_EQ(stream.sendState, StreamSendState::Open);
   EXPECT_EQ(stream.recvState, StreamRecvState::Closed);
@@ -485,9 +687,9 @@ TEST_F(QuicRecvResetTest, FromOpenReadEOFMismatch) {
   QuicStreamState stream(id, *conn);
   RstStreamFrame rst(1, GenericApplicationErrorCode::UNKNOWN, 100);
   stream.finalReadOffset = 1024;
-  EXPECT_THROW(
-      receiveRstStreamSMHandler(stream, std::move(rst)),
-      QuicTransportException);
+  auto result = receiveRstStreamSMHandler(stream, std::move(rst));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicRecvResetTest, FromHalfClosedRemoteNoReadOffsetYet) {
@@ -496,8 +698,9 @@ TEST_F(QuicRecvResetTest, FromHalfClosedRemoteNoReadOffsetYet) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Open;
   stream.recvState = StreamRecvState::Closed;
-  receiveRstStreamSMHandler(
+  auto result = receiveRstStreamSMHandler(
       stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 100));
+  ASSERT_FALSE(result.hasError());
 
   EXPECT_EQ(stream.sendState, StreamSendState::Open);
   EXPECT_EQ(stream.recvState, StreamRecvState::Closed);
@@ -512,8 +715,9 @@ TEST_F(QuicRecvResetTest, FromHalfClosedRemoteReadOffsetMatch) {
   stream.recvState = StreamRecvState::Closed;
   stream.finalReadOffset = 1024;
 
-  receiveRstStreamSMHandler(
+  auto result = receiveRstStreamSMHandler(
       stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 1024));
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.sendState, StreamSendState::Open);
   EXPECT_EQ(stream.recvState, StreamRecvState::Closed);
   verifyStreamReset(stream, 1024);
@@ -526,10 +730,10 @@ TEST_F(QuicRecvResetTest, FromHalfClosedRemoteReadOffsetMismatch) {
   stream.sendState = StreamSendState::Open;
   stream.recvState = StreamRecvState::Closed;
   stream.finalReadOffset = 1024;
-  EXPECT_THROW(
-      receiveRstStreamSMHandler(
-          stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 100)),
-      QuicTransportException);
+  auto result = receiveRstStreamSMHandler(
+      stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 100));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicRecvResetTest, FromHalfClosedLocal) {
@@ -538,8 +742,9 @@ TEST_F(QuicRecvResetTest, FromHalfClosedLocal) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Closed;
   stream.recvState = StreamRecvState::Open;
-  receiveRstStreamSMHandler(
+  auto result = receiveRstStreamSMHandler(
       stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 200));
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.sendState, StreamSendState::Closed);
   EXPECT_EQ(stream.recvState, StreamRecvState::Closed);
   verifyStreamReset(stream, 200);
@@ -552,10 +757,10 @@ TEST_F(QuicRecvResetTest, FromHalfClosedLocalReadEOFMismatch) {
   stream.sendState = StreamSendState::Closed;
   stream.recvState = StreamRecvState::Open;
   stream.finalReadOffset = 2014;
-  EXPECT_THROW(
-      receiveRstStreamSMHandler(
-          stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 200)),
-      QuicTransportException);
+  auto result = receiveRstStreamSMHandler(
+      stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 200));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicRecvResetTest, FromResetSentNoReadOffsetYet) {
@@ -565,8 +770,9 @@ TEST_F(QuicRecvResetTest, FromResetSentNoReadOffsetYet) {
   stream.sendState = StreamSendState::ResetSent;
   stream.recvState = StreamRecvState::Open;
 
-  receiveRstStreamSMHandler(
+  auto result = receiveRstStreamSMHandler(
       stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 200));
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.sendState, StreamSendState::ResetSent);
   EXPECT_EQ(stream.recvState, StreamRecvState::Closed);
   verifyStreamReset(stream, 200);
@@ -580,8 +786,9 @@ TEST_F(QuicRecvResetTest, FromResetSentOffsetMatch) {
   stream.recvState = StreamRecvState::Open;
   stream.finalReadOffset = 200;
 
-  receiveRstStreamSMHandler(
+  auto result = receiveRstStreamSMHandler(
       stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 200));
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.sendState, StreamSendState::ResetSent);
   EXPECT_EQ(stream.recvState, StreamRecvState::Closed);
   verifyStreamReset(stream, 200);
@@ -594,10 +801,10 @@ TEST_F(QuicRecvResetTest, FromResetSentOffsetMismatch) {
   stream.sendState = StreamSendState::ResetSent;
   stream.recvState = StreamRecvState::Open;
   stream.finalReadOffset = 300;
-  EXPECT_THROW(
-      receiveRstStreamSMHandler(
-          stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 200)),
-      QuicTransportException);
+  auto result = receiveRstStreamSMHandler(
+      stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 200));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicRecvResetTest, FromClosedNoReadOffsetYet) {
@@ -606,8 +813,9 @@ TEST_F(QuicRecvResetTest, FromClosedNoReadOffsetYet) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Closed;
   stream.recvState = StreamRecvState::Closed;
-  receiveRstStreamSMHandler(
+  auto result = receiveRstStreamSMHandler(
       stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 200));
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.sendState, StreamSendState::Closed);
   EXPECT_EQ(stream.recvState, StreamRecvState::Closed);
   verifyStreamReset(stream, 200);
@@ -620,8 +828,9 @@ TEST_F(QuicRecvResetTest, FromClosedOffsetMatch) {
   stream.sendState = StreamSendState::Closed;
   stream.recvState = StreamRecvState::Closed;
   stream.finalReadOffset = 1234;
-  receiveRstStreamSMHandler(
+  auto result = receiveRstStreamSMHandler(
       stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 1234));
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.sendState, StreamSendState::Closed);
   EXPECT_EQ(stream.recvState, StreamRecvState::Closed);
   verifyStreamReset(stream, 1234);
@@ -634,11 +843,66 @@ TEST_F(QuicRecvResetTest, FromClosedOffsetMismatch) {
   stream.sendState = StreamSendState::Closed;
   stream.recvState = StreamRecvState::Closed;
   stream.finalReadOffset = 123;
-  EXPECT_THROW(
-      receiveRstStreamSMHandler(
-          stream,
-          RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 1234)),
-      QuicTransportException);
+  auto result = receiveRstStreamSMHandler(
+      stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 1234));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
+}
+
+class QuicReliableResetTransitionTest : public Test {};
+
+TEST_F(QuicReliableResetTransitionTest, FromOpenReliableDataNotYetReceived) {
+  auto conn = createConn();
+  StreamId id = 5;
+  QuicStreamState stream(id, *conn);
+  RstStreamFrame rst(id, GenericApplicationErrorCode::UNKNOWN, 100, 10);
+  stream.currentReadOffset = 9;
+  auto result = receiveRstStreamSMHandler(stream, rst);
+  ASSERT_FALSE(result.hasError());
+
+  EXPECT_EQ(stream.sendState, StreamSendState::Open);
+  EXPECT_EQ(stream.recvState, StreamRecvState::Open);
+}
+
+TEST_F(QuicReliableResetTransitionTest, FromOpenReliableDataReceived) {
+  auto conn = createConn();
+  StreamId id = 5;
+  QuicStreamState stream(id, *conn);
+  RstStreamFrame rst(id, GenericApplicationErrorCode::UNKNOWN, 100, 10);
+  stream.currentReadOffset = 10;
+  auto result = receiveRstStreamSMHandler(stream, rst);
+  ASSERT_FALSE(result.hasError());
+
+  EXPECT_EQ(stream.sendState, StreamSendState::Open);
+  EXPECT_EQ(stream.recvState, StreamRecvState::Closed);
+}
+
+TEST_F(QuicReliableResetTransitionTest, DataReceivedTillReliableSize) {
+  auto conn = createConn();
+  StreamId id = 5;
+  QuicStreamState stream(id, *conn);
+  stream.reliableSizeFromPeer = 10;
+  stream.currentReadOffset = 1;
+  auto result = receiveReadStreamFrameSMHandler(
+      stream,
+      ReadStreamFrame(id, 1, folly::IOBuf::copyBuffer("999999999"), false));
+  ASSERT_FALSE(result.hasError());
+  EXPECT_EQ(stream.sendState, StreamSendState::Open);
+  EXPECT_EQ(stream.recvState, StreamRecvState::Closed);
+}
+
+TEST_F(QuicReliableResetTransitionTest, DataNotReceivedTillReliableSize) {
+  auto conn = createConn();
+  StreamId id = 5;
+  QuicStreamState stream(id, *conn);
+  stream.reliableSizeFromPeer = 10;
+  stream.currentReadOffset = 1;
+  auto result = receiveReadStreamFrameSMHandler(
+      stream,
+      ReadStreamFrame(id, 1, folly::IOBuf::copyBuffer("99999999"), false));
+  ASSERT_FALSE(result.hasError());
+  EXPECT_EQ(stream.sendState, StreamSendState::Open);
+  EXPECT_EQ(stream.recvState, StreamRecvState::Open);
 }
 
 class QuicUnidirectionalStreamTest : public Test {};
@@ -649,9 +913,10 @@ TEST_F(QuicUnidirectionalStreamTest, OpenInvalidReadStream) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Open;
   stream.recvState = StreamRecvState::Invalid;
-  EXPECT_THROW(
-      receiveReadStreamFrameSMHandler(stream, ReadStreamFrame(id, 1, false)),
-      QuicTransportException);
+  auto result =
+      receiveReadStreamFrameSMHandler(stream, ReadStreamFrame(id, 1, false));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicUnidirectionalStreamTest, OpenInvalidRstStream) {
@@ -660,11 +925,10 @@ TEST_F(QuicUnidirectionalStreamTest, OpenInvalidRstStream) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Open;
   stream.recvState = StreamRecvState::Invalid;
-  EXPECT_THROW(
-      receiveRstStreamSMHandler(
-          stream,
-          RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 1234)),
-      QuicTransportException);
+  auto result = receiveRstStreamSMHandler(
+      stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 1234));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicUnidirectionalStreamTest, OpenInvalidSendReset) {
@@ -674,9 +938,9 @@ TEST_F(QuicUnidirectionalStreamTest, OpenInvalidSendReset) {
   stream.sendState = StreamSendState::Invalid;
   stream.recvState = StreamRecvState::Open;
 
-  EXPECT_THROW(
-      sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN),
-      QuicTransportException);
+  auto result = sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN);
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicUnidirectionalStreamTest, OpenInvalidAckStreamFrame) {
@@ -686,7 +950,9 @@ TEST_F(QuicUnidirectionalStreamTest, OpenInvalidAckStreamFrame) {
   stream.sendState = StreamSendState::Invalid;
   stream.recvState = StreamRecvState::Open;
   WriteStreamFrame ackedFrame(id, 0, 0, false);
-  EXPECT_THROW(sendAckSMHandler(stream, ackedFrame), QuicTransportException);
+  auto result = sendAckSMHandler(stream, ackedFrame);
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicUnidirectionalStreamTest, OpenInvalidStopSending) {
@@ -695,10 +961,10 @@ TEST_F(QuicUnidirectionalStreamTest, OpenInvalidStopSending) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Invalid;
   stream.recvState = StreamRecvState::Open;
-  EXPECT_THROW(
-      sendStopSendingSMHandler(
-          stream, StopSendingFrame(id, GenericApplicationErrorCode::UNKNOWN)),
-      QuicTransportException);
+  auto result = sendStopSendingSMHandler(
+      stream, StopSendingFrame(id, GenericApplicationErrorCode::UNKNOWN));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicUnidirectionalStreamTest, ClosedInvalidReadStream) {
@@ -707,9 +973,10 @@ TEST_F(QuicUnidirectionalStreamTest, ClosedInvalidReadStream) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Open;
   stream.recvState = StreamRecvState::Invalid;
-  EXPECT_THROW(
-      receiveReadStreamFrameSMHandler(stream, ReadStreamFrame(id, 1, false)),
-      QuicTransportException);
+  auto result =
+      receiveReadStreamFrameSMHandler(stream, ReadStreamFrame(id, 1, false));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicUnidirectionalStreamTest, ClosedInvalidRstStream) {
@@ -718,11 +985,10 @@ TEST_F(QuicUnidirectionalStreamTest, ClosedInvalidRstStream) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Open;
   stream.recvState = StreamRecvState::Invalid;
-  EXPECT_THROW(
-      receiveRstStreamSMHandler(
-          stream,
-          RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 1234)),
-      QuicTransportException);
+  auto result = receiveRstStreamSMHandler(
+      stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 1234));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicUnidirectionalStreamTest, ClosedInvalidSendReset) {
@@ -731,9 +997,9 @@ TEST_F(QuicUnidirectionalStreamTest, ClosedInvalidSendReset) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Invalid;
   stream.recvState = StreamRecvState::Closed;
-  EXPECT_THROW(
-      sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN),
-      QuicTransportException);
+  auto result = sendRstSMHandler(stream, GenericApplicationErrorCode::UNKNOWN);
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicUnidirectionalStreamTest, ClosedInvalidAckStreamFrame) {
@@ -744,7 +1010,9 @@ TEST_F(QuicUnidirectionalStreamTest, ClosedInvalidAckStreamFrame) {
   stream.recvState = StreamRecvState::Closed;
 
   WriteStreamFrame ackedFrame(id, 0, 0, false);
-  EXPECT_THROW(sendAckSMHandler(stream, ackedFrame), QuicTransportException);
+  auto result = sendAckSMHandler(stream, ackedFrame);
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicUnidirectionalStreamTest, ClosedInvalidStopSending) {
@@ -753,10 +1021,10 @@ TEST_F(QuicUnidirectionalStreamTest, ClosedInvalidStopSending) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::Invalid;
   stream.recvState = StreamRecvState::Closed;
-  EXPECT_THROW(
-      sendStopSendingSMHandler(
-          stream, StopSendingFrame(id, GenericApplicationErrorCode::UNKNOWN)),
-      QuicTransportException);
+  auto result = sendStopSendingSMHandler(
+      stream, StopSendingFrame(id, GenericApplicationErrorCode::UNKNOWN));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicUnidirectionalStreamTest, OpenReadStreamFin) {
@@ -769,7 +1037,9 @@ TEST_F(QuicUnidirectionalStreamTest, OpenReadStreamFin) {
   ReadStreamFrame receivedStreamFrame(stream.id, 100, true);
   receivedStreamFrame.data = folly::IOBuf::create(10);
   receivedStreamFrame.data->append(10);
-  receiveReadStreamFrameSMHandler(stream, std::move(receivedStreamFrame));
+  auto result =
+      receiveReadStreamFrameSMHandler(stream, std::move(receivedStreamFrame));
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.sendState, StreamSendState::Invalid);
   EXPECT_EQ(stream.recvState, StreamRecvState::Closed);
 }
@@ -781,8 +1051,9 @@ TEST_F(QuicUnidirectionalStreamTest, OpenRstStream) {
   stream.sendState = StreamSendState::Invalid;
   stream.recvState = StreamRecvState::Open;
 
-  receiveRstStreamSMHandler(
+  auto result = receiveRstStreamSMHandler(
       stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 1234));
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.sendState, StreamSendState::Invalid);
   EXPECT_EQ(stream.recvState, StreamRecvState::Closed);
 }
@@ -803,7 +1074,8 @@ TEST_F(QuicUnidirectionalStreamTest, OpenFinalAckStreamFrame) {
       std::forward_as_tuple(1),
       std::forward_as_tuple(std::make_unique<WriteStreamBuffer>(
           ChainedByteRangeHead(buf), 1, false)));
-  sendAckSMHandler(stream, streamFrame);
+  auto result = sendAckSMHandler(stream, streamFrame);
+  ASSERT_FALSE(result.hasError());
   EXPECT_EQ(stream.sendState, StreamSendState::Closed);
   EXPECT_EQ(stream.recvState, StreamRecvState::Invalid);
 }
@@ -814,9 +1086,10 @@ TEST_F(QuicUnidirectionalStreamTest, ResetSentInvalidReadStream) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::ResetSent;
   stream.recvState = StreamRecvState::Invalid;
-  EXPECT_THROW(
-      receiveReadStreamFrameSMHandler(stream, ReadStreamFrame(id, 1, false)),
-      QuicTransportException);
+  auto result =
+      receiveReadStreamFrameSMHandler(stream, ReadStreamFrame(id, 1, false));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicUnidirectionalStreamTest, ResetSentInvalidRstStream) {
@@ -825,11 +1098,10 @@ TEST_F(QuicUnidirectionalStreamTest, ResetSentInvalidRstStream) {
   QuicStreamState stream(id, *conn);
   stream.sendState = StreamSendState::ResetSent;
   stream.recvState = StreamRecvState::Invalid;
-  EXPECT_THROW(
-      receiveRstStreamSMHandler(
-          stream,
-          RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 1234)),
-      QuicTransportException);
+  auto result = receiveRstStreamSMHandler(
+      stream, RstStreamFrame(1, GenericApplicationErrorCode::UNKNOWN, 1234));
+  ASSERT_TRUE(result.hasError());
+  EXPECT_NE(result.error().code.asTransportErrorCode(), nullptr);
 }
 
 TEST_F(QuicOpenStateTest, DSRStreamAcked) {
@@ -837,11 +1109,13 @@ TEST_F(QuicOpenStateTest, DSRStreamAcked) {
   conn->clientConnectionId = getTestConnectionId(0);
   conn->serverConnectionId = getTestConnectionId(1);
   auto stream = conn->streamManager->createNextBidirectionalStream().value();
-  writeDataToQuicStream(
-      *stream,
-      folly::IOBuf::copyBuffer("Big ship stucks in small water"),
-      false);
-  writeBufMetaToQuicStream(*stream, BufferMeta(1000), true);
+  ASSERT_FALSE(writeDataToQuicStream(
+                   *stream,
+                   folly::IOBuf::copyBuffer("Big ship stucks in small water"),
+                   false)
+                   .hasError());
+  ASSERT_FALSE(
+      writeBufMetaToQuicStream(*stream, BufferMeta(1000), true).hasError());
   auto bufMetaStartingOffset = stream->writeBufMeta.offset;
   handleStreamBufMetaWritten(
       *conn,
@@ -856,7 +1130,8 @@ TEST_F(QuicOpenStateTest, DSRStreamAcked) {
       stream->retransmissionBufMetas.find(bufMetaStartingOffset));
   WriteStreamFrame frame(stream->id, bufMetaStartingOffset, 300, false);
   frame.fromBufMeta = true;
-  sendAckSMHandler(*stream, frame);
+  auto result = sendAckSMHandler(*stream, frame);
+  ASSERT_FALSE(result.hasError());
   EXPECT_TRUE(stream->retransmissionBufMetas.empty());
   EXPECT_EQ(stream->sendState, StreamSendState::Open);
 }
@@ -868,11 +1143,14 @@ TEST_F(QuicOpenStateTest, DSRFullStreamAcked) {
   auto stream = conn->streamManager->createNextBidirectionalStream().value();
   auto buf = folly::IOBuf::copyBuffer("Big ship stucks in small water");
   size_t len = buf->computeChainDataLength();
-  writeDataToQuicStream(*stream, std::move(buf), false);
-  handleStreamWritten(
-      *conn, *stream, 0, len, false, 1, PacketNumberSpace::AppData);
+  ASSERT_FALSE(
+      writeDataToQuicStream(*stream, std::move(buf), false).hasError());
+  ASSERT_FALSE(handleStreamWritten(
+                   *conn, *stream, 0, len, false, 1, PacketNumberSpace::AppData)
+                   .hasError());
   ASSERT_EQ(stream->retransmissionBuffer.size(), 1);
-  writeBufMetaToQuicStream(*stream, BufferMeta(1000), true);
+  ASSERT_FALSE(
+      writeBufMetaToQuicStream(*stream, BufferMeta(1000), true).hasError());
   auto bufMetaStartingOffset = stream->writeBufMeta.offset;
   handleStreamBufMetaWritten(
       *conn,
@@ -888,12 +1166,14 @@ TEST_F(QuicOpenStateTest, DSRFullStreamAcked) {
       stream->retransmissionBufMetas.find(bufMetaStartingOffset));
   WriteStreamFrame frame(stream->id, bufMetaStartingOffset, 1000, true);
   frame.fromBufMeta = true;
-  sendAckSMHandler(*stream, frame);
+  auto result = sendAckSMHandler(*stream, frame);
+  ASSERT_FALSE(result.hasError());
   frame.offset = 0;
   frame.len = len;
   frame.fin = false;
   frame.fromBufMeta = false;
-  sendAckSMHandler(*stream, frame);
+  result = sendAckSMHandler(*stream, frame);
+  ASSERT_FALSE(result.hasError());
   EXPECT_TRUE(stream->retransmissionBuffer.empty());
   EXPECT_TRUE(stream->retransmissionBufMetas.empty());
   EXPECT_EQ(stream->sendState, StreamSendState::Closed);
